@@ -2,10 +2,13 @@
 
    <abstract>Standard panel for querying user about page layout.</abstract>
 
-   Copyright (C) 2001 Free Software Foundation, Inc.
+   Copyright (C) 2001,2004 Free Software Foundation, Inc.
 
    Written By: Adam Fedor <fedor@gnu.org>
    Date: Oct 2001
+   Author: Chad Hardin <cehardin@mac.com>
+   Date: July 2004
+   
    
    This file is part of the GNUstep GUI Library.
 
@@ -30,33 +33,40 @@
 
 #include <AppKit/NSApplication.h>
 #include <AppKit/NSPanel.h>
+#include <AppKit/NSView.h>
 
 @class NSPrintInfo;
-@class NSView;
 
+
+//
+// Note: not all of these tags are currently used in the
+// NSPageLayout panel, see below for details.
+//
 enum {
-  NSPLImageButton,
-  NSPLTitleField,
-  NSPLPaperNameButton,
-  NSPLUnitsButton,
-  NSPLWidthForm,
-  NSPLHeightForm,
-  NSPLOrientationMatrix,
-  NSPLCancelButton,
-  NSPLOKButton 
+  NSPLImageButton       = 1,  //Used.
+  NSPLTitleField        = 2,  //Used.
+  NSPLPaperNameButton   = 3,  //Used, only for the standard paper size 
+                              //NSPopUpButton though.  Not the 
+					//NSPopUpButton used to select custom 
+					//paper sizes.
+  NSPLUnitsButton       = 4,  //NOT used.
+  NSPLWidthField        = 5,  //NOT used.
+  NSPLHeightField       = 6,  //NOT used.
+  NSPLOrientationMatrix = 7,  //Used.
+  NSPLCancelButton      = 8,  //Used.
+  NSPLOKButton          = 9,  //Used.
+  NSPLPageLayout        = 10,  //NOT used.
+  NSPLScaleField        = 11  //Used.
 };
 
+
 @interface NSApplication (NSPageLayout)
 - (void) runPageLayout: (id)sender;
 @end
 
 @interface NSPageLayout : NSPanel
 {
-  NSSize _size;
-  NSPrintInfo *_printInfo;
-  NSView *_accessoryView;
-  id _panel;
-  int _picked;
+  id _controller; 
 }
 
 //
@@ -71,10 +81,10 @@ enum {
 - (int)runModalWithPrintInfo:(NSPrintInfo *)pInfo;
 #ifndef	STRICT_OPENSTEP
 - (void)beginSheetWithPrintInfo:(NSPrintInfo *)printInfo
-		 modalForWindow:(NSWindow *)docWindow
-		       delegate:(id)delegate
-		 didEndSelector:(SEL)didEndSelector
-		    contextInfo:(void *)contextInfo;
+                 modalForWindow:(NSWindow *)docWindow
+                       delegate:(id)delegate
+                 didEndSelector:(SEL)didEndSelector
+                    contextInfo:(void *)contextInfo;
 #endif
 
 //
@@ -87,7 +97,7 @@ enum {
 // Updating the Panel's Display 
 //
 - (void)convertOldFactor:(float *)old
-	       newFactor:(float *)new;
+               newFactor:(float *)new;
 - (void)pickedButton:(id)sender;
 - (void)pickedOrientation:(id)sender;
 - (void)pickedPaperSize:(id)sender;
