@@ -30,12 +30,14 @@
 #include <Foundation/NSBundle.h>
 #include <Foundation/NSProcessInfo.h>
 
-#include <AppKit/NSApplication.h>
-#include <AppKit/NSEvent.h>
-#include <AppKit/NSGraphicsContext.h>
-#include <AppKit/NSGraphics.h>
-#include <AppKit/NSColor.h>
-#include <AppKit/DPSOperators.h>
+#include "AppKit/NSApplication.h"
+#include "AppKit/NSEvent.h"
+#include "AppKit/NSGraphicsContext.h"
+#include "AppKit/NSGraphics.h"
+#include "AppKit/NSColor.h"
+#include "AppKit/NSView.h"
+#include "AppKit/NSWindow.h"
+#include "AppKit/DPSOperators.h"
 
 char **NSArgv = NULL;
 
@@ -298,9 +300,92 @@ NSPlanarFromDepth(NSWindowDepth depth)
   return planar;
 }
 
+/* Graphic Ops */
+NSColor* NSReadPixel(NSPoint location)
+{
+  NSLog(@"NSReadPixel not implemented");
+  return nil;
+}
+
+void NSCopyBitmapFromGState(int srcGstate, NSRect srcRect, NSRect destRect)
+{
+  NSLog(@"NSCopyBitmapFromGState not implemented");
+}
+
+void NSCopyBits(int srcGstate, NSRect srcRect, NSPoint destPoint)
+{
+  float x, y, w, h;
+  NSGraphicsContext *ctxt = GSCurrentContext();
+
+  x = NSMinX(srcRect);
+  y = NSMinY(srcRect);
+  w = NSWidth(srcRect);
+  h = NSHeight(srcRect);
+
+  DPScomposite(ctxt, x, y, w, h, srcGstate, destPoint.x, destPoint.y,
+	       NSCompositeCopy);
+}
+
 /*
  * Rectangle Drawing 
  */
+void NSEraseRect(NSRect aRect)
+{
+  NSGraphicsContext *ctxt = GSCurrentContext();
+  DPSgsave(ctxt);
+  DPSsetgray(ctxt, NSWhite);
+  NSRectFill(aRect);
+  DPSgrestore(ctxt);
+}
+
+void NSHighlightRect(NSRect aRect)
+{
+  NSGraphicsContext *ctxt = GSCurrentContext();
+  DPScompositerect(ctxt, NSMinX(aRect), NSMinY(aRect), 
+		   NSWidth(aRect), NSHeight(aRect), 
+		   NSCompositeHighlight);
+}
+
+void NSRectClip(NSRect aRect)
+{
+  NSGraphicsContext *ctxt = GSCurrentContext();
+  DPSrectclip(ctxt, NSMinX(aRect), NSMinY(aRect), 
+	      NSWidth(aRect), NSHeight(aRect));
+  DPSnewpath(ctxt);
+}
+
+void NSRectClipList(const NSRect *rects, int count)
+{
+  int i;
+  NSRect union_rect;
+
+  if (count == 0)
+    return;
+
+  /* 
+     The specification is not clear if the union of the rects 
+     should produce the new clip rect or if the outline of all rects 
+     should be used as clip path.
+  */
+  union_rect = rects[0];
+  for (i = 1; i < count; i++)
+    union_rect = NSUnionRect(union_rect, rects[i]);
+
+  NSRectClip(union_rect);
+}
+
+void NSRectFill(NSRect aRect)
+{
+  NSGraphicsContext *ctxt = GSCurrentContext();
+  DPSrectfill(ctxt, NSMinX(aRect), NSMinY(aRect), 
+	      NSWidth(aRect), NSHeight(aRect));
+}
+
+void NSRectFillList(const NSRect *rects, int count)
+{
+  NSGraphicsContext *ctxt = GSCurrentContext();
+  GSRectFillList(ctxt, rects, count);
+}
 
 void 
 NSRectFillListWithColors(const NSRect *rects, NSColor **colors, int count)
@@ -314,6 +399,28 @@ NSRectFillListWithColors(const NSRect *rects, NSColor **colors, int count)
     }
 }
 
+void NSRectFillListWithGrays(const NSRect *rects, const float *grays, 
+			     int count)
+{
+  int i;
+  NSGraphicsContext *ctxt = GSCurrentContext();
+
+  for (i = 0; i < count; i++)
+    {
+      DPSsetgray(ctxt, grays[i]);
+      DPSrectfill(ctxt,  NSMinX(rects[i]), NSMinY(rects[i]), 
+		  NSWidth(rects[i]), NSHeight(rects[i]));
+    }
+}
+
+void NSRectFillUsingOperation(NSRect aRect, NSCompositingOperation op)
+{
+  NSGraphicsContext *ctxt = GSCurrentContext();
+  DPScompositerect(ctxt, NSMinX(aRect), NSMinY(aRect), 
+		   NSWidth(aRect), NSHeight(aRect), op);
+}
+
+
 void 
 NSRectFillListUsingOperation(const NSRect *rects, int count, 
 			     NSCompositingOperation op)
@@ -341,6 +448,37 @@ NSRectFillListWithColorsUsingOperation(const NSRect *rects,
     }
 }
 
+/*
+ * Draw a Bordered Rectangle
+ */
+void NSDottedFrameRect(const NSRect aRect)
+{
+  float dot_dash[] = {1.0, 1.0};
+  NSGraphicsContext *ctxt = GSCurrentContext();
+
+  DPSsetgray(ctxt, NSBlack);
+  DPSsetlinewidth(ctxt, 1.0);
+  // FIXME
+  DPSsetdash(ctxt, dot_dash, 2, 0.0);
+  DPSrectstroke(ctxt,  NSMinX(aRect), NSMinY(aRect), 
+		NSWidth(aRect), NSHeight(aRect));
+}
+
+void NSFrameRect(const NSRect aRect)
+{
+  NSFrameRectWithWidth(aRect, 1.0);
+}
+
+void NSFrameRectWithWidth(const NSRect aRect, float frameWidth)
+{
+  float width;
+  NSGraphicsContext *ctxt = GSCurrentContext();
+  DPScurrentlinewidth(ctxt, &width);
+  DPSsetlinewidth(ctxt, frameWidth);
+  DPSrectstroke(ctxt,  NSMinX(aRect), NSMinY(aRect), 
+		NSWidth(aRect), NSHeight(aRect));
+  DPSsetlinewidth(ctxt, width);
+}
 
 //*****************************************************************************
 //
@@ -507,12 +645,12 @@ NSDrawGroove(const NSRect aRect, const NSRect clipRect)
 void 
 NSDrawWhiteBezel(const NSRect aRect,  const NSRect clipRect)
 {
-  NSRectEdge up_sides[] = {NSMaxXEdge, NSMinYEdge, NSMinXEdge, NSMaxYEdge, 
-  			   NSMaxXEdge, NSMinYEdge, NSMinXEdge, NSMaxYEdge};
-  NSRectEdge down_sides[] = {NSMaxXEdge, NSMaxYEdge, NSMinXEdge, NSMinYEdge, 
-  			     NSMaxXEdge, NSMaxYEdge, NSMinXEdge, NSMinYEdge};
-  float grays[] = {NSWhite, NSWhite, NSDarkGray, NSDarkGray,
-  		   NSLightGray, NSLightGray, NSDarkGray, NSDarkGray};
+  NSRectEdge up_sides[] = {NSMaxYEdge, NSMaxXEdge, NSMinYEdge, NSMinXEdge,
+  			   NSMaxYEdge, NSMaxXEdge, NSMinYEdge, NSMinXEdge};
+  NSRectEdge down_sides[] = {NSMinYEdge, NSMaxXEdge, NSMaxYEdge, NSMinXEdge, 
+  			     NSMinYEdge, NSMaxXEdge, NSMaxYEdge, NSMinXEdge};
+  float grays[] = {NSDarkGray, NSWhite, NSWhite, NSDarkGray, 
+  		   NSDarkGray, NSLightGray, NSLightGray, NSDarkGray};
   NSRect rect;
   NSGraphicsContext *ctxt = GSCurrentContext();
 
@@ -535,11 +673,75 @@ NSDrawWhiteBezel(const NSRect aRect,  const NSRect clipRect)
 void 
 NSDrawDarkBezel(NSRect aRect, NSRect clipRect)
 {
+  NSRectEdge up_sides[] = {NSMaxXEdge, NSMinYEdge, NSMinXEdge, NSMaxYEdge,
+			   NSMaxXEdge, NSMinYEdge, NSMinXEdge, NSMaxYEdge};
+  NSRectEdge down_sides[] = {NSMaxXEdge, NSMaxYEdge, NSMinXEdge, NSMinYEdge,
+			     NSMaxXEdge, NSMaxYEdge, NSMinXEdge, NSMinYEdge};
+  // FIXME: The actual colour used for the 3 + 4 line 
+  // (and the two additional points) is a bit darker. 
+  float grays[] = {NSWhite, NSWhite, NSLightGray, NSLightGray,
+		   NSLightGray, NSLightGray, NSBlack, NSBlack};
+  NSRect rect;
+  NSGraphicsContext *ctxt = GSCurrentContext();
+
+  if (GSWViewIsFlipped(ctxt) == YES)
+    {
+      rect = NSDrawTiledRects(aRect, clipRect,
+			       down_sides, grays, 8);
+      // to give a really clean look we add 2 light gray points
+      DPSsetgray(ctxt, NSLightGray);
+      DPSrectfill(ctxt, NSMinX(aRect) + 1., NSMaxY(aRect) - 2., 1., 1.);
+      DPSrectfill(ctxt, NSMaxX(aRect) - 2., NSMinY(aRect) + 1., 1., 1.);
+    }
+  else
+    {
+      rect = NSDrawTiledRects(aRect, clipRect,
+			       up_sides, grays, 8);
+      // to give a really clean look we add 2 light gray points
+      DPSsetgray(ctxt, NSLightGray);
+      DPSrectfill(ctxt, NSMinX(aRect) + 1., NSMinY(aRect) + 1., 1., 1.);
+      DPSrectfill(ctxt, NSMaxX(aRect) - 2., NSMaxY(aRect) - 2., 1., 1.);
+    }
+
+  DPSsetgray(ctxt, NSLightGray);
+  DPSrectfill(ctxt, NSMinX(rect), NSMinY(rect), 
+	      NSWidth(rect), NSHeight(rect));
 }
 
 void 
 NSDrawLightBezel(NSRect aRect, NSRect clipRect)
 {
+  NSRectEdge up_sides[] = {NSMaxXEdge, NSMinYEdge, NSMinXEdge, NSMaxYEdge, 
+  			   NSMaxXEdge, NSMinYEdge, NSMinXEdge, NSMaxYEdge};
+  NSRectEdge down_sides[] = {NSMaxXEdge, NSMaxYEdge, NSMinXEdge, NSMinYEdge, 
+  			     NSMaxXEdge, NSMaxYEdge, NSMinXEdge, NSMinYEdge};
+  float grays[] = {NSWhite, NSWhite, NSGray, NSGray,
+  		   NSBlack, NSBlack, NSBlack, NSBlack};
+  NSRect rect;
+  NSGraphicsContext *ctxt = GSCurrentContext();
+
+  if (GSWViewIsFlipped(ctxt) == YES)
+    {
+      rect = NSDrawTiledRects(aRect, clipRect,
+			       down_sides, grays, 8);
+      // to give a really clean look we add 2 light gray points
+      DPSsetgray(ctxt, NSLightGray);
+      DPSrectfill(ctxt, NSMinX(aRect), NSMaxY(aRect) - 1., 1., 1.);
+      DPSrectfill(ctxt, NSMaxX(aRect) - 1., NSMinY(aRect), 1., 1.);
+    }
+  else
+    {
+      rect = NSDrawTiledRects(aRect, clipRect,
+			       up_sides, grays, 8);
+      // to give a really clean look we add 2 light gray points
+      DPSsetgray(ctxt, NSLightGray);
+      DPSrectfill(ctxt, NSMinX(aRect), NSMinY(aRect), 1., 1.);
+      DPSrectfill(ctxt, NSMaxX(aRect) - 1., NSMaxY(aRect) - 1., 1., 1.);
+    }
+
+  DPSsetgray(ctxt, NSWhite);
+  DPSrectfill(ctxt, NSMinX(rect), NSMinY(rect), 
+	      NSWidth(rect), NSHeight(rect));
 }
 
 void
@@ -577,7 +779,8 @@ NSDrawFramePhoto(const NSRect aRect, const NSRect clipRect)
 void 
 NSDrawWindowBackground(NSRect aRect)
 {
-// TODO
+  [[NSColor windowBackgroundColor] set];
+  NSRectFill(aRect);  
 }
 
 void 
