@@ -42,6 +42,7 @@
 #include <Foundation/NSKeyValueCoding.h>
 #include <Foundation/NSNotification.h>
 #include <Foundation/NSArchiver.h>
+#include <Foundation/NSSet.h>
 #include <AppKit/NSMenu.h>
 #include <AppKit/NSView.h>
 #include <AppKit/NSTextView.h>
@@ -54,6 +55,10 @@
 
 static const int currentVersion = 1; // GSNibItem version number...
 
+@interface NSObject (GSNibPrivateMethods)
+- (BOOL) isInInterfaceBuilder;
+@end
+
 @interface NSApplication (GSNibContainer)
 - (void)_deactivateVisibleWindow: (NSWindow *)win;
 @end
@@ -70,6 +75,68 @@ static const int currentVersion = 1; // GSNibItem version number...
 }
 @end
 
+/*
+ * This private class is used to collect the nib items while the 
+ * .gorm file is being unarchived.  This is done to allow only
+ * the top level items to be retained in a clean way.  The reason it's
+ * being done this way is because old .gorm files don't have any
+ * array within the nameTable which indicates the objects which are
+ * considered top level, so there is no clean and generic way to determine
+ * this.   Basically the top level items are any instances of or instances
+ * of subclasses of NSMenu, NSWindow, or any controller class.
+ * It's the last one that's hairy.  Controller classes are
+ * represented in .gorm files by the GSNibItem class, but once they transform
+ * into the actual class instance it's not easy to tell if it should be 
+ * retained or not since there are a lot of other things stored in the nameTable
+ * as well.  GJC
+ */
+
+static NSString *GSInternalNibItemAddedNotification = @"_GSInternalNibItemAddedNotification";
+
+@interface GSNibItemCollector : NSObject
+{
+  NSMutableArray *items;
+}
+- (void) handleNotification: (NSNotification *)notification;
+- (NSMutableArray *)items;
+@end
+
+@implementation GSNibItemCollector
+- (void) handleNotification: (NSNotification *)notification;
+{
+  id obj = [notification object];
+  [items addObject: obj];
+}
+
+- init
+{
+  if((self = [super init]) != nil)
+    {
+      NSNotificationCenter *nc = [NSNotificationCenter defaultCenter];
+
+      // add myself as an observer and initialize the items array.
+      [nc addObserver: self
+	  selector: @selector(handleNotification:)
+	  name: GSInternalNibItemAddedNotification 
+	  object: nil];
+      items = [[NSMutableArray alloc] init];
+    }
+  return self;
+}
+
+- (void) dealloc
+{
+  [[NSNotificationCenter defaultCenter] removeObserver: self];
+  RELEASE(items);
+  [super dealloc];
+}
+
+- (NSMutableArray *)items
+{
+  return items;
+}
+@end
+
 /*
  *	The GSNibContainer class manages the internals of a nib file.
  */
@@ -84,19 +151,18 @@ static const int currentVersion = 1; // GSNibItem version number...
 }
 
 - (void) awakeWithContext: (NSDictionary *)context
-	    topLevelItems: (NSArray *)items
 {
-  if (_isAwake == NO)
+  if (isAwake == NO)
     {
       NSEnumerator	*enumerator;
       NSNibConnector	*connection;
       NSString		*key;
       NSArray		*visible;
       NSMenu		*menu;
-      NSMutableArray    *topLevelObjects; 
+      NSMutableArray    *topObjects; 
       id                 obj;
 
-      _isAwake = YES;
+      isAwake = YES;
       /*
        *	Add local entries into name table.
        */
@@ -153,6 +219,24 @@ static const int currentVersion = 1; // GSNibItem version number...
 	  [NSApp setWindowsMenu: menu];
 	}
 
+
+      /* 
+       * See if the user has passed in the NSTopLevelObjects key.
+       * This is an implementation of an undocumented, but commonly used feature
+       * of nib files to allow the release of the top level objects in the nib
+       * file.
+       */
+      obj = [context objectForKey: @"NSTopLevelObjects"];
+      if([obj isKindOfClass: [NSMutableArray class]])
+	{
+	  topObjects = obj;
+	}
+      else
+	{
+	  topObjects = nil; 
+	}
+
+
       /*
        * Now tell all the objects that they have been loaded from
        * a nib.
@@ -168,76 +252,57 @@ static const int currentVersion = 1; // GSNibItem version number...
 		 [key isEqualToString: @"NSVisible"] == NO && // also exclude any other special parts of the nameTable.
 		 [key isEqualToString: @"NSDeferred"] == NO &&
 		 [key isEqualToString: @"NSTopLevelObjects"] == NO &&
-		 [key isEqualToString: @"GSCustomClassMap"] == NO)		
+		 [key isEqualToString: @"GSCustomClassMap"] == NO)
 		{
 		  id o = [nameTable objectForKey: key];
+
+		  // send the awake message, if it responds...
 		  if ([o respondsToSelector: @selector(awakeFromNib)])
 		    {
 		      [o awakeFromNib];
 		    }
+
+		  /*
+		   * Retain all "top level" items so that, when the container 
+		   * is released, they will remain. The GSNibItems instantiated in the gorm need 
+		   * to be retained, since we are deallocating the container.  
+		   * We don't want to retain the owner.
+		   *
+		   * Please note: It is encumbent upon the developer of an application to 
+		   * release these objects.   Instantiating a window manually or loading in a .gorm 
+		   * file are equivalent processes.  These objects need to be released in their 
+		   * respective controllers.  If the developer has used the "NSTopLevelObjects" feature, 
+		   * then he will get the objects back in an array which he merely must release in
+		   * order to release the objects held within.  GJC
+		   */
+		  if([key isEqualToString: @"NSOwner"] == NO)
+		    {
+		      if([topLevelObjects containsObject: o]) // anything already designated a top level item..
+			{
+			  if(topObjects == nil)
+			    {
+			      // It is expected, if the NSTopLevelObjects key is not passed in,
+			      // that the user has opted to either allow these objects to leak or
+			      // to release them explicitly.
+			      RETAIN(o);
+			    }
+			  else
+			    {
+			      // We don't want to do the extra retain if the items are added to the
+			      // array, since the array will do the retain for us.   When the array
+			      // is released, the top level objects should be released as well.
+			      [topObjects addObject: o];
+			    }
+			}
+		    }
 		}
 	    }
 	}
       
-      /* 
-       * See if the user has passed in the NSTopLevelObjects key.
-       * This is an implementation of an undocumented, but commonly used feature
-       * of nib files to allow the release of the top level objects in the nib
-       * file.
-       */
-      obj = [context objectForKey: @"NSTopLevelObjects"];
-      if([obj isKindOfClass: [NSMutableArray class]])
-	{
-	  topLevelObjects = obj;
-	}
-      else
-	{
-	  topLevelObjects = nil; 
-	}
-
-      /*
-       * Retain all "top level" items so that, when the container is released, they will remain.
-       * The GSNibItems instantiated in the gorm need to be retained,
-       * since we are deallocating the container.
-       */
-      enumerator = [nameTable keyEnumerator];
-      while ((key = [enumerator nextObject]) != nil)
-	{
-	  if ([context objectForKey: key] == nil &&
-	      [key isEqualToString: @"NSWindowsMenu"] == NO && // exclude special sections.
-	      [key isEqualToString: @"NSServicesMenu"] == NO &&
-	      [key isEqualToString: @"NSVisible"] == NO && 
-	      [key isEqualToString: @"NSDeferred"] == NO &&
-	      [key isEqualToString: @"NSTopLevelObjects"] == NO &&
-	      [key isEqualToString: @"GSCustomClassMap"] == NO)
-	    {
-	      id o = [nameTable objectForKey: key];
-	      // RETAIN all top-level items...
-	      if (([o isKindOfClass: [NSMenu class]] == YES &&
-		   [key isEqualToString: @"NSMenu"] == YES) || // the main menu...
-		  ([o isKindOfClass: [NSWindow class]] == YES) || // any windows...
-		  ([items containsObject: o] == YES)) 
-		{
-		  if(topLevelObjects == nil)
-		    {
-		      // It is expected, if the NSTopLevelObjects key is not passed in,
-		      // that the user has opted to either allow these objects to leak or
-		      // to release them explicitly.
-		      RETAIN(o);
-		    }
-		  else
-		    {
-		      // We don't want to do the extra retain if the items are added to the
-		      // array, since the array will do the retain for us.   When the array
-		      // is released, the top level objects should be released as well.
-		      [topLevelObjects addObject: o];
-		    }
-		}
-	    }
-	}
-
       /*
        * See if there are objects that should be made visible.
+       * This is the last thing we should do since changes might be made
+       * in the awakeFromNib methods which are called on all of the objects.
        */
       visible = [nameTable objectForKey: @"NSVisible"];
       if (visible != nil
@@ -274,13 +339,25 @@ static const int currentVersion = 1; // GSNibItem version number...
 {
   RELEASE(nameTable);
   RELEASE(connections);
+  RELEASE(topLevelObjects);
   [super dealloc];
 }
 
 - (void) encodeWithCoder: (NSCoder*)aCoder
 {
-  [aCoder encodeObject: nameTable];
-  [aCoder encodeObject: connections];
+  int version = [GSNibContainer version];
+  if(version == GNUSTEP_NIB_VERSION)
+    {
+      [aCoder encodeObject: nameTable];
+      [aCoder encodeObject: connections];
+      [aCoder encodeObject: topLevelObjects];
+    }
+  else
+    {
+      // encode it as a version 0 file...
+      [aCoder encodeObject: nameTable];
+      [aCoder encodeObject: connections];
+    }
 }
 
 - (id) init
@@ -289,6 +366,7 @@ static const int currentVersion = 1; // GSNibItem version number...
     {
       nameTable = [[NSMutableDictionary alloc] initWithCapacity: 8];
       connections = [[NSMutableArray alloc] initWithCapacity: 8];
+      topLevelObjects = [[NSMutableSet alloc] initWithCapacity: 8];
     }
   return self;
 }
@@ -296,11 +374,45 @@ static const int currentVersion = 1; // GSNibItem version number...
 - (id) initWithCoder: (NSCoder*)aCoder
 {
   int version = [aCoder versionForClassName: @"GSNibContainer"]; 
-  
+
+  // save the version to the ivar, we need it later.
   if(version == GNUSTEP_NIB_VERSION)
     {
       [aCoder decodeValueOfObjCType: @encode(id) at: &nameTable];
       [aCoder decodeValueOfObjCType: @encode(id) at: &connections];
+      [aCoder decodeValueOfObjCType: @encode(id) at: &topLevelObjects];
+    }
+  else if(version == 0)
+    {
+      GSNibItemCollector *nibitems = [[GSNibItemCollector alloc] init];
+      NSEnumerator *en;
+      NSString *key;
+      
+      // initialize the set of top level objects...
+      topLevelObjects = [[NSMutableSet alloc] initWithCapacity: 8];
+
+      // unarchive...
+      [aCoder decodeValueOfObjCType: @encode(id) at: &nameTable];
+      [aCoder decodeValueOfObjCType: @encode(id) at: &connections];
+      [topLevelObjects addObjectsFromArray: [nibitems items]]; // get the top level items here...
+      RELEASE(nibitems);
+
+      // iterate through the objects returned
+      en = [nameTable keyEnumerator];
+      while((key = [en nextObject]) != nil)
+	{
+	  id o = [nameTable objectForKey: key];
+	  if(([o isKindOfClass: [NSMenu class]] && [key isEqual: @"NSMenu"]) ||
+	     [o isKindOfClass: [NSWindow class]])
+	    {
+	      [topLevelObjects addObject: o]; // if it's a top level object, add it.
+	    }
+	}
+    }
+  else
+    {
+      [NSException raise: NSInternalInconsistencyException
+		   format: @"Unable to read GSNibContainer version #%d.  GSNibContainer version for the installed gui lib is %d.", version, GNUSTEP_NIB_VERSION];
     }
 
   return self;
@@ -310,6 +422,11 @@ static const int currentVersion = 1; // GSNibItem version number...
 {
   return nameTable;
 }
+
+- (NSMutableSet*) topLevelObjects
+{
+  return topLevelObjects;
+}
 @end
 
 // The first standin objects here are for views and normal objects like controllers
@@ -357,7 +474,7 @@ static const int currentVersion = 1; // GSNibItem version number...
       if (cls == nil)
 	{
 	  [NSException raise: NSInternalInconsistencyException
-		       format: @"Unable to find class '%@'", theClass];
+		       format: @"Unable to find class '%@', it is not linked into the application.", theClass];
 	}
       
       obj = [cls allocWithZone: [self zone]];
@@ -386,7 +503,7 @@ static const int currentVersion = 1; // GSNibItem version number...
       if (cls == nil)
 	{
 	  [NSException raise: NSInternalInconsistencyException
-		       format: @"Unable to find class '%@'", theClass];
+		       format: @"Unable to find class '%@', it is not linked into the application.", theClass];
 	}
       
       obj = [cls allocWithZone: [self zone]];
@@ -405,14 +522,16 @@ static const int currentVersion = 1; // GSNibItem version number...
     }
 
   // If this is a nib item and not a custom view, then we need to add it to
-  // the set of things to be retained.
-  if(obj != nil)
+  // the set of things to be retained.  Also, the initial version of the nib container
+  // needed this code, but subsequent versions don't, so don't send the notification,
+  // if the version isn't zero.
+  if(obj != nil && [aCoder versionForClassName: NSStringFromClass([GSNibContainer class])] == 0)
     {
       if([self isKindOfClass: [GSNibItem class]] == YES &&
 	 [self isKindOfClass: [GSCustomView class]] == NO)
 	{
 	  NSNotificationCenter *nc = [NSNotificationCenter defaultCenter];
-	  [nc postNotificationName: @"__GSInternalNibItemAddedNotification"
+	  [nc postNotificationName: GSInternalNibItemAddedNotification
 	      object: obj];
 	}
     }
@@ -485,7 +604,7 @@ static const int currentVersion = 1; // GSNibItem version number...
       if(_superClass == nil)
 	{
 	  [NSException raise: NSInternalInconsistencyException
-		       format: @"Unable to find class '%@'", superClassName];
+		       format: @"Unable to find class '%@', it is not linked into the application.", superClassName];
 	}
     }
   return self;
@@ -528,23 +647,23 @@ static const int currentVersion = 1; // GSNibItem version number...
 
 	  // if we are living within the interface builder app, then don't try to 
 	  // morph into the subclass.
-	  if([self respondsToSelector: @selector(isInInterfaceBuilder)])
-	    {
-	      obj = [_superClass alloc];
-	    }
-	  else
+	  if([self shouldSwapClass])
 	    {
 	      Class aClass = NSClassFromString(_className);
 	      if(aClass == 0)
 		{
 		  [NSException raise: NSInternalInconsistencyException
-			       format: @"Unable to find class '%@'", _className];
+			       format: @"Unable to find class '%@', it is not linked into the application.", _className];
 		}
 	  
 	      // Initialize the object...  dont call decode, since this wont 
 	      // allow us to instantiate the class we want. 
 	      obj = [aClass alloc];
 	    }
+	  else
+	    {
+	      obj = [_superClass alloc];
+	    }
 
 	  // inform the coder that this object is to replace the template in all cases.
 	  [unarchiver replaceObject: self withObject: obj];
@@ -552,10 +671,6 @@ static const int currentVersion = 1; // GSNibItem version number...
 	}
     }
 
-  // Do this here since we are not using decode to do this.
-  // Normally decode does a retain, so we must do it here.
-  // RETAIN(obj); 
-
   // change the class of the instance to the one we want to see...
   return obj;
 }
@@ -572,6 +687,16 @@ static const int currentVersion = 1; // GSNibItem version number...
       [_object encodeWithCoder: aCoder]; 
     }
 }
+
+- (BOOL) shouldSwapClass
+{
+  BOOL result = YES;
+  if([self respondsToSelector: @selector(isInInterfaceBuilder)])
+    {
+      result = !([self isInInterfaceBuilder]);
+    }
+  return result;
+}
 @end
 
 @implementation GSWindowTemplate
@@ -604,9 +729,9 @@ static const int currentVersion = 1; // GSNibItem version number...
       // decode the defer flag...
       [coder decodeValueOfObjCType: @encode(BOOL) at: &_deferFlag];      
 
-      if(![self respondsToSelector: @selector(isInInterfaceBuilder)])
+      if([self shouldSwapClass])
       {
-	if(GSGetInstanceMethodNotInherited([obj class], @selector(initWithContentRect:styleMask:backing:defer:)) != NULL)
+	if(GSGetMethod([obj class], @selector(initWithContentRect:styleMask:backing:defer:), YES, NO) != NULL)
 	  {
 	    // if we are not in interface builder, call 
 	    // designated initializer per spec...
@@ -646,9 +771,9 @@ static const int currentVersion = 1; // GSNibItem version number...
   id obj = [super initWithCoder: coder];
   if(obj != nil)
     {
-      if(![self respondsToSelector: @selector(isInInterfaceBuilder)])
+      if([self shouldSwapClass])
       {
-	if(GSGetInstanceMethodNotInherited([obj class],@selector(initWithFrame:)) != NULL)
+	if(GSGetMethod([obj class],@selector(initWithFrame:), YES, NO) != NULL)
 	  {
 	    NSRect theFrame = [obj frame];
 	    obj =  [obj initWithFrame: theFrame];
@@ -675,9 +800,9 @@ static const int currentVersion = 1; // GSNibItem version number...
   id     obj = [super initWithCoder: coder];
   if(obj != nil)
     {
-      if(![self respondsToSelector: @selector(isInInterfaceBuilder)])
+      if([self shouldSwapClass])
       {
-	if(GSGetInstanceMethodNotInherited([obj class],@selector(initWithFrame:)) != NULL)
+	if(GSGetMethod([obj class],@selector(initWithFrame:), YES, NO) != NULL)
 	  {
 	    NSRect theFrame = [obj frame]; 
 	    obj = [obj initWithFrame: theFrame];
@@ -704,9 +829,9 @@ static const int currentVersion = 1; // GSNibItem version number...
   id     obj = [super initWithCoder: coder];
   if(obj != nil)
     {
-      if(![self respondsToSelector: @selector(isInInterfaceBuilder)])
+      if([self shouldSwapClass])
       {
-	if(GSGetInstanceMethodNotInherited([obj class],@selector(initWithFrame:textContainer:)) != NULL)
+	if(GSGetMethod([obj class],@selector(initWithFrame:textContainer:), YES, NO) != NULL)
 	  {
 	    NSRect theFrame = [obj frame];
 	    id textContainer = [obj textContainer];
@@ -735,9 +860,9 @@ static const int currentVersion = 1; // GSNibItem version number...
   id     obj = [super initWithCoder: coder];
   if(obj != nil)
     {
-      if(![self respondsToSelector: @selector(isInInterfaceBuilder)])
+      if([self shouldSwapClass])
       {
-	if(GSGetInstanceMethodNotInherited([obj class],@selector(initWithTitle:)) != NULL)
+	if(GSGetMethod([obj class],@selector(initWithTitle:), YES, NO) != NULL)
 	  {
 	    NSString *theTitle = [obj title]; 
 	    obj = [obj initWithTitle: theTitle];
@@ -766,9 +891,9 @@ static const int currentVersion = 1; // GSNibItem version number...
   if(obj != nil)
     {
       /* 
-      if(![self respondsToSelector: @selector(isInInterfaceBuilder)])
+      if([self shouldSwapClass])
       {
-	if(GSGetInstanceMethodNotInherited([obj class],@selector(initWithFrame:)) != NULL)
+	if(GSGetMethod([obj class],@selector(initWithFrame:), YES, NO) != NULL)
 	  {
 	    NSRect theFrame = [obj frame]; 
 	    obj = [obj initWithFrame: theFrame];
@@ -795,9 +920,9 @@ static const int currentVersion = 1; // GSNibItem version number...
   id     obj = [super initWithCoder: coder];
   if(obj != nil)
     {
-      if(![self respondsToSelector: @selector(isInInterfaceBuilder)])
+      if([self shouldSwapClass])
       {
-	if(GSGetInstanceMethodNotInherited([obj class],@selector(init)) != NULL)
+	if(GSGetMethod([obj class],@selector(init), YES, NO) != NULL)
 	  {
 	    obj = [self init];
 	  }
