@@ -76,6 +76,7 @@ static GSServicesManager	*manager = nil;
 - (void) dealloc;
 - (void) release;
 - (id) retain;
+- (void) activateIgnoringOtherApps: (BOOL)flag;
 - (id) self;
 @end
 
@@ -388,6 +389,11 @@ NSRegisterServicesProvider(id provider, NSString *name)
   return self;
 }
 
+- (void) activateIgnoringOtherApps: (BOOL)flag
+{
+  [NSApp activateIgnoringOtherApps: flag];
+}
+
 - (id) self
 {
   return self;
@@ -1382,8 +1388,8 @@ GSContactApplication(NSString *appName, NSString *port, NSDate *expire)
   return app;
 }
 
-NSDictionary *
-_serviceFromAnyLocalizedTitle(NSString *title)
+static NSDictionary *
+serviceFromAnyLocalizedTitle(NSString *title)
 {
   NSDictionary	*allServices;
   NSEnumerator	*e1;
@@ -1447,7 +1453,7 @@ NSPerformService(NSString *serviceItem, NSPasteboard *pboard)
   NSString		*userData;
   NSString		*error = nil;
 
-  service = _serviceFromAnyLocalizedTitle(serviceItem);
+  service = serviceFromAnyLocalizedTitle(serviceItem);
   if (service == nil)
     {
       NSRunAlertPanel(nil,
