@@ -66,6 +66,7 @@
 #include "AppKit/NSWorkspace.h"
 #include "AppKit/NSNibLoading.h"
 #include "AppKit/NSPageLayout.h"
+#include "AppKit/PSOperators.h"
 
 #include "GNUstepGUI/GSDisplayServer.h"
 #include "GNUstepGUI/GSServicesManager.h"
@@ -342,16 +343,26 @@ NSApplication	*NSApp = nil;
 // Class variables
 static NSCell* dragCell = nil;
 static NSCell* tileCell = nil;
+static BOOL useWindowmakerIconBackground = NO;
 
 + (void) initialize
 {
   NSImage	*defImage = [NSImage imageNamed: @"GNUstep"];
-  NSImage	*tileImage = [NSImage imageNamed: @"common_Tile"];
+  NSImage	*tileImage;
 
   dragCell = [[NSCell alloc] initImageCell: defImage];
   [dragCell setBordered: NO];
-  tileCell = [[NSCell alloc] initImageCell: tileImage];
-  [tileCell setBordered: NO];
+  if ([[NSUserDefaults standardUserDefaults]
+	  boolForKey: @"GSUseWindowmakerIconBackground"])
+    {
+      useWindowmakerIconBackground = YES; 
+    }
+  else
+    {
+      tileImage = [NSImage imageNamed: @"common_Tile"];
+      tileCell = [[NSCell alloc] initImageCell: tileImage];
+      [tileCell setBordered: NO];
+    }
 }
 
 - (BOOL) acceptsFirstMouse: (NSEvent*)theEvent
@@ -379,8 +390,26 @@ static NSCell* tileCell = nil;
 
 - (void) drawRect: (NSRect)rect
 {
-  [tileCell drawWithFrame: NSMakeRect(0,0,64,64) inView: self];
+  if (useWindowmakerIconBackground)
+    {
+      PScompositerect(_bounds.origin.x,
+      		      _bounds.origin.y,
+		      _bounds.size.width,
+		      _bounds.size.height,
+		      NSCompositeClear);
+    }
+  else
+    {
+      [tileCell drawWithFrame: NSMakeRect(0,0,64,64) inView: self];
+    }
+  
   [dragCell drawWithFrame: NSMakeRect(8,8,48,48) inView: self];
+  if ([NSApp isHidden])
+    {
+      NSRectEdge mySides[] = {NSMinXEdge, NSMinYEdge, NSMaxXEdge, NSMaxYEdge};
+      float myGrays[] = {NSBlack, NSWhite, NSWhite, NSBlack};
+      NSDrawTiledRects(NSMakeRect(4, 4, 3, 2), rect, mySides, myGrays, 4);
+    }
 }
 
 - (id) initWithFrame: (NSRect)frame
@@ -477,8 +506,7 @@ static NSCell* tileCell = nil;
 
   if ([self lockFocusIfCanDraw])
     {
-      [tileCell drawWithFrame: NSMakeRect(0,0,64,64) inView: self];
-      [dragCell drawWithFrame: NSMakeRect(8,8,48,48) inView: self];
+      [self drawRect: NSMakeRect(0,0,64,64)];
       [self unlockFocus];
       [_window flushWindow];
     }
@@ -1574,7 +1602,7 @@ IF_NO_GC(NSAssert([event retainCount] > 0, NSInternalInconsistencyException));
 		|| (type == NSRightMouseDown) || (type == NSRightMouseUp)
 		|| (type == NSMouseMoved))
 		{
-		  [NSCursor unhide];
+		  [NSCursor setHiddenUntilMouseMoves: NO];
 		}
 	    }
 	}
@@ -1859,6 +1887,7 @@ image.
 	  [win orderOut: self];
 	}
       _app_is_hidden = YES;
+      [[_app_icon_window contentView] setNeedsDisplay: YES];
 
       /*
        * On hiding we also deactivate the application which will make the menus
@@ -1920,6 +1949,7 @@ image.
 	  [_hidden_key makeKeyAndOrderFront: self];
 	  _hidden_key = nil;
 	}
+      [[_app_icon_window contentView] setNeedsDisplay: YES];
 
       [nc postNotificationName: NSApplicationDidUnhideNotification
 			object: self];
