@@ -52,6 +52,7 @@
 #include "AppKit/NSEvent.h"
 #include "AppKit/NSWindow.h"
 #include "AppKit/NSBezierPath.h"
+#include "GNUstepGUI/GSDrawFunctions.h"
 
 DEFINE_RINT_IF_MISSING
 
@@ -192,7 +193,7 @@ static NSTextFieldCell *titleCell;
       return;
     }
 
-  NSDrawGrayBezel (cellFrame, NSZeroRect);
+  [GSDrawFunctions drawGrayBezel: cellFrame : NSZeroRect];
   [self drawInteriorWithFrame: cellFrame  inView: controlView];
 }
 @end
@@ -540,6 +541,8 @@ static NSTextFieldCell *titleCell;
             }
           else
             {
+	      // Actually it's always called at 0 column
+	      [[self matrixInColumn: i] deselectAllCells];
               break;
             }
         }
@@ -693,7 +696,7 @@ static NSTextFieldCell *titleCell;
 {
   NSBrowserColumn *bc;
   NSScrollView *sc;
-  NSRect rect = {{0, 0}, {100, 100}};
+  NSRect rect = {{-110, -110}, {100, 100}};
 
   bc = [[NSBrowserColumn alloc] init];
 
@@ -1373,31 +1376,24 @@ static NSTextFieldCell *titleCell;
 /** Updates the horizontal scroller to reflect column positions. */
 - (void) updateScroller
 {
-  int num;
+  int   num = [self numberOfVisibleColumns];
+  float prop = (float)num / (float)(_lastColumnLoaded + 1);
+  int   uc = ((_lastColumnLoaded + 1) - num); // Unvisible columns
+  float f_step = 1.0;                         // Knob moving step
+  float fv = 0.0;
 
-  num = [self numberOfVisibleColumns];
-
-  // If there are not enough columns to scroll with
-  // then the column must be visible
-  if ((_lastColumnLoaded == 0) ||
-      (_lastColumnLoaded <= (num - 1)))
+  if (uc > 0.0)
     {
-      [_horizontalScroller setEnabled: NO];
+      f_step = 1.0 / (float)uc;
     }
-  else
-    {
-      if (!_skipUpdateScroller)
-      	{
-      	  float prop = (float)num / (float)(_lastColumnLoaded + 1);
-      	  float i = _lastColumnLoaded - num + 1;
-      	  float f = 1 + ((_lastVisibleColumn - _lastColumnLoaded) / i);
+  fv = (float)(_firstVisibleColumn * f_step);
 
-          [_horizontalScroller setFloatValue: f knobProportion: prop];
-	}
-      [_horizontalScroller setEnabled: YES];
+  if (_lastVisibleColumn > _lastColumnLoaded)
+    {
+      prop = (float)num / (float)(_lastVisibleColumn + 1);
     }
 
-  [_horizontalScroller setNeedsDisplay: YES];
+  [_horizontalScroller setFloatValue: fv knobProportion: prop];
 }
 
 /** Scrolls columns left or right based on an NSScroller. */
@@ -1430,9 +1426,7 @@ static NSTextFieldCell *titleCell;
 	{
 	  float f = [sender floatValue];
 
-	  _skipUpdateScroller = YES;
 	  [self scrollColumnToVisible: rintf(f * _lastColumnLoaded)];
-	  _skipUpdateScroller = NO;
 	}
       	break;
       
@@ -1550,8 +1544,7 @@ static NSTextFieldCell *titleCell;
       if (_separatesColumns)
 	r.size.width = _frame.size.width - r.origin.x;
       else
-	r.size.width = _frame.size.width 
-	  - (r.origin.x + (2 * bs.width) + ([self numberOfVisibleColumns] - 1));
+	r.size.width = _frame.size.width - (r.origin.x + bs.width);
     }
 
   if (r.size.width < 0)
@@ -1836,7 +1829,7 @@ static NSTextFieldCell *titleCell;
   // If the matrix isn't ours then just return
   if (column < 0 || column > _lastColumnLoaded)
     return;
-
+    
   a = [sender selectedCells];
   aCount = [a count];
   if(aCount == 0)
@@ -1858,16 +1851,9 @@ static NSTextFieldCell *titleCell;
 
   selectedCellsCount = [selectedCells count];
 
-  if (selectedCellsCount == 0)
+  // Select cells that should be selected
+  if (selectedCellsCount > 0)
     {
-      // If we should not select the cell then deselect it
-
-      [sender deselectAllCells];
-    }
-  else if (selectedCellsCount < aCount)
-    {
-      [sender deselectSelectedCell];
-
       enumerator = [selectedCells objectEnumerator];
       while ((cell = [enumerator nextObject]))
 	[sender selectCell: cell];
@@ -1888,6 +1874,8 @@ static NSTextFieldCell *titleCell;
       [sender scrollCellToVisibleAtRow: [sender selectedRow] column: 0];
     }
 
+  [self updateScroller];
+
   // Send the action to target
   [self sendAction];
 
@@ -2060,32 +2048,30 @@ static NSTextFieldCell *titleCell;
     }
 
   // Draws scroller border
-  if (_hasHorizontalScroller)
+  if (_hasHorizontalScroller && _separatesColumns)
     {
       NSRect scrollerBorderRect = _scrollerRect;
       NSSize bs = _sizeForBorderType (NSBezelBorder);
 
       scrollerBorderRect.origin.x = 0;
       scrollerBorderRect.origin.y = 0;
-      scrollerBorderRect.size.width += 2 * bs.width - 1;
+      scrollerBorderRect.size.width += 2 * bs.width;
       scrollerBorderRect.size.height += (2 * bs.height) - 1;
 
       if ((NSIntersectsRect (scrollerBorderRect, rect) == YES) && _window)
       	{
-      	  NSDrawGrayBezel (scrollerBorderRect, rect);
+          [GSDrawFunctions drawGrayBezel: scrollerBorderRect : rect];
 	}
     }
 
   if (!_separatesColumns)
     {
       NSPoint p1,p2;
-      NSRect  browserRect;
       int     i, visibleColumns;
       float   hScrollerWidth = _hasHorizontalScroller ? scrollerWidth : 0;
       
       // Columns borders
-      browserRect = NSMakeRect(0, 0, rect.size.width, rect.size.height);
-      NSDrawGrayBezel (browserRect, rect);
+      [GSDrawFunctions drawGrayBezel: _bounds : rect];
       
       [[NSColor blackColor] set];
       visibleColumns = [self numberOfVisibleColumns]; 
@@ -2254,10 +2240,11 @@ static NSTextFieldCell *titleCell;
       NSMatrix *matrix;
       NSString *sv;
       int i, n, s;
+      int match;
       int selectedColumn;
       SEL lcarcSel = @selector(loadedCellAtRow:column:);
       IMP lcarc = [self methodForSelector: lcarcSel];
-      
+
       selectedColumn = [self selectedColumn];
       if(selectedColumn != -1)
 	{
@@ -2290,7 +2277,7 @@ static NSTextFieldCell *titleCell;
 		  RETAIN(_charBuffer);
 		}
 	    }
-	  
+
 	  _alphaNumericalLastColumn = selectedColumn;
 	  _lastKeyPressed = [theEvent timestamp];
 	  
@@ -2300,35 +2287,42 @@ static NSTextFieldCell *titleCell;
 	  if (([sv length] > 0)
 	      && ([sv hasPrefix: _charBuffer]))
 	    return;
-	  
-	  for (i = s+1; i < n; i++)
+
+	  match = -1;
+	  for (i = s + 1; i < n; i++)
 	    {
 	      sv = [((*lcarc)(self, lcarcSel, i, selectedColumn))
 		     stringValue];
 	      if (([sv length] > 0)
 		  && ([sv hasPrefix: _charBuffer]))
 		{
-		  [self selectRow: i
-			inColumn: selectedColumn];	
-		  [matrix scrollCellToVisibleAtRow: i column: 0];
-		  [matrix performClick: self];
-		  return;
+		  match = i;
+		  break;
 		}
 	    }
-	  for (i = 0; i < s; i++)
+	  if (i == n)
 	    {
-	      sv = [((*lcarc)(self, lcarcSel, i, selectedColumn))
-		     stringValue];
-	      if (([sv length] > 0)
-		  && ([sv hasPrefix: _charBuffer]))
+	      for (i = 0; i < s; i++)
 		{
-		  [self selectRow: i
-			inColumn: selectedColumn];
-		  [matrix scrollCellToVisibleAtRow: i column: 0];
-		  [matrix performClick: self];
-		  return;
+		  sv = [((*lcarc)(self, lcarcSel, i, selectedColumn))
+			 stringValue];
+		  if (([sv length] > 0)
+		      && ([sv hasPrefix: _charBuffer]))
+		    {
+		      match = i;
+		      break;
+		    }
 		}
 	    }
+	  if (match != -1)
+	    {
+	      [matrix deselectAllCells];
+	      [self selectRow: match
+		inColumn: selectedColumn];
+	      [matrix scrollCellToVisibleAtRow: match column: 0];
+	      [matrix performClick: self];
+	      return;
+	    }
 	}
       _lastKeyPressed = 0.;
     }
