@@ -41,6 +41,7 @@
 
 #include "AppKit/AppKitExceptions.h"
 #include "AppKit/NSApplication.h"
+#include "AppKit/NSBezierPath.h"
 #include "AppKit/NSButtonCell.h"
 #include "AppKit/NSButton.h"
 #include "AppKit/NSColor.h"
@@ -52,6 +53,14 @@
 #include "AppKit/NSWindow.h"
 #include "GNUstepGUI/GSDrawFunctions.h"
 
+#include <math.h>
+
+@interface NSButtonCell (Private)
+// Overriden private internal method
+- (void) _drawImage: (NSImage *)anImage inFrame: (NSRect)aRect 
+  isFlipped: (BOOL)flipped;
+@end
+
 @implementation NSButtonCell
 
 /*
@@ -82,6 +91,8 @@
   _keyEquivalentModifierMask = NSCommandKeyMask;
   _keyEquivalent = @"";
   _altContents = @"";
+  _gradient_type = NSGradientNone;
+  _image_dims_when_disabled = NO;
 
   return self;
 }
@@ -118,15 +129,32 @@
   [super dealloc];
 }
 
+
+- (void) setType: (NSCellType)aType
+{
+  /* We do nothing here (we match the Mac OS X behavior) because with
+   * NSButtonCell GNUstep implementation the cell type is binded to the image
+   * position. Such behavior has been choosen because it permits to have
+   * -setFont: -setTitle -setImage: methods which are symetrical by not altering
+   * the cell type, morevover a cell type is more characterized by the potential
+   * visibility of the image (which is under the control of the method
+   * -setImagePosition:) than by the value of the image ivar itself (related to
+   * -setImage: method).  
+   * On Mac OS X, the NSButtonCell cell type is NSTextCellType by default or
+   * NSImageCellType if the initialization has been done with -initImageCell:,
+   * it should be noted that the cell type never changes later.
+   */
+}
+
 /*
  * Setting the Titles
  */
-- (NSString*) title						
+- (NSString*) title
 {
   return [self stringValue];
 }
 
-- (NSString*) alternateTitle					
+- (NSString*) alternateTitle
 {
   return _altContents;
 }
@@ -224,7 +252,7 @@
     }
 }
 
-- (void) setFont: (NSFont*)fontObject		
+- (void) setFont: (NSFont*)fontObject
 {
   int size;
 
@@ -325,12 +353,12 @@
 /*
  * Setting the Images
  */
-- (NSImage*) alternateImage						
+- (NSImage*) alternateImage
 {
   return _altImage;
 }
 
-- (NSCellImagePosition) imagePosition			
+- (NSCellImagePosition) imagePosition
 {
   return _cell.image_position;
 }
@@ -401,12 +429,12 @@
 /*
  * Modifying Graphic Attributes
  */
-- (BOOL) isTransparent					
+- (BOOL) isTransparent
 {
   return _buttoncell_is_transparent;
 }
 
-- (void) setTransparent: (BOOL)flag	
+- (void) setTransparent: (BOOL)flag
 {
   _buttoncell_is_transparent = flag;
 }
@@ -416,22 +444,22 @@
   return !_buttoncell_is_transparent && _cell.is_bordered;
 }
 
-- (NSBezelStyle)bezelStyle
+- (NSBezelStyle) bezelStyle
 {
   return _bezel_style;
 }
 
-- (void)setBezelStyle:(NSBezelStyle)bezelStyle
+- (void) setBezelStyle: (NSBezelStyle)bezelStyle
 {
   _bezel_style = bezelStyle;
 }
 
-- (BOOL)showsBorderOnlyWhileMouseInside
+- (BOOL) showsBorderOnlyWhileMouseInside
 {
   return _shows_border_only_while_mouse_inside;
 }
 
-- (void)setShowsBorderOnlyWhileMouseInside:(BOOL)show
+- (void) setShowsBorderOnlyWhileMouseInside: (BOOL)show
 {
   if (_shows_border_only_while_mouse_inside == show)
     {
@@ -442,22 +470,22 @@
   // FIXME Switch mouse tracking on
 }
 
-- (NSGradientType)gradientType
+- (NSGradientType) gradientType
 {
   return _gradient_type;
 }
 
-- (void)setGradientType:(NSGradientType)gradientType
+- (void) setGradientType: (NSGradientType)gradientType
 {
   _gradient_type = gradientType;
 }
 
-- (BOOL)imageDimsWhenDisabled
+- (BOOL) imageDimsWhenDisabled
 {
   return _image_dims_when_disabled;
 }
 
-- (void)setImageDimsWhenDisabled:(BOOL)flag
+- (void) setImageDimsWhenDisabled:(BOOL)flag
 {
   _image_dims_when_disabled = flag;
 }
@@ -465,17 +493,17 @@
 /*
  * Modifying Graphic Attributes
  */
-- (int) highlightsBy						
+- (int) highlightsBy
 {
   return _highlightsByMask;
 }
 
-- (void) setHighlightsBy: (int)mask	
+- (void) setHighlightsBy: (int)mask
 {
   _highlightsByMask = mask;
 }
 
-- (void) setShowsStateBy: (int)mask	
+- (void) setShowsStateBy: (int)mask
 {
   _showAltStateMask = mask;
 }
@@ -541,17 +569,17 @@
     }
 }
 
-- (int) showsStateBy						
+- (int) showsStateBy
 {
   return _showAltStateMask;
 }
 
-- (void) setIntValue: (int)anInt		
+- (void) setIntValue: (int)anInt
 {
   [self setState: (anInt != 0)];
 }
 
-- (void) setFloatValue: (float)aFloat	
+- (void) setFloatValue: (float)aFloat
 {
   [self setState: (aFloat != 0)];
 }
@@ -561,22 +589,22 @@
   [self setState: (aDouble != 0)];
 }
 
-- (int) intValue							
+- (int) intValue
 {
   return _cell.state;
 }
 
-- (float) floatValue						
+- (float) floatValue
 {
   return _cell.state;
 }
 
-- (double) doubleValue					
+- (double) doubleValue
 {
   return _cell.state;
 }
 
-- (void) setObjectValue: (id)object 
+- (void) setObjectValue: (id)object
 {
   if (object == nil)
     {
@@ -653,6 +681,105 @@
   [self drawInteriorWithFrame: cellFrame inView: controlView];
 }
 
+- (void) drawGradientWithFrame: (NSRect)cellFrame inView: (NSView *)controlView
+{
+  float   start_white = 0.0;
+  float   end_white = 0.0;
+  float   white = 0.0;
+  float   white_step = 0.0;
+  float   h, s, v, a;
+  NSColor *lightGray = nil;
+  NSColor *gray = nil;
+  NSColor *darkGray = nil;
+  NSPoint p1, p2;
+
+  lightGray = [NSColor colorWithDeviceRed:0.83 green:0.83 blue:0.83 alpha:1.0];
+  gray = [NSColor colorWithDeviceRed:0.50 green:0.50 blue:0.50 alpha:1.0];
+  darkGray = [NSColor colorWithDeviceRed:0.32 green:0.32 blue:0.32 alpha:1.0];
+
+  switch (_gradient_type)
+    {
+    case NSGradientNone:
+      return;
+      break;
+
+    case NSGradientConcaveWeak:
+      [gray getHue: &h saturation: &s brightness: &v alpha: &a];
+      start_white = [lightGray brightnessComponent];
+      end_white = [gray brightnessComponent];
+      break;
+      
+    case NSGradientConvexWeak:
+      [darkGray getHue: &h saturation: &s brightness: &v alpha: &a];
+      start_white = [gray brightnessComponent];
+      end_white = [lightGray brightnessComponent];
+      break;
+      
+    case NSGradientConcaveStrong:
+      [lightGray getHue: &h saturation: &s brightness: &v alpha: &a];
+      start_white = [lightGray brightnessComponent];
+      end_white = [darkGray brightnessComponent];
+      break;
+      
+    case NSGradientConvexStrong:
+      [darkGray getHue: &h saturation: &s brightness: &v alpha: &a];
+      start_white = [darkGray brightnessComponent];
+      end_white = [lightGray brightnessComponent];
+      break;
+
+    default:
+      break;
+    }
+
+  white = start_white;
+  white_step = fabs(start_white - end_white)/
+               (cellFrame.size.width + cellFrame.size.height);
+
+  // Start from top left
+  p1 = NSMakePoint(cellFrame.origin.x, 
+		   cellFrame.size.height + cellFrame.origin.y);
+  p2 = NSMakePoint(cellFrame.origin.x, 
+		   cellFrame.size.height + cellFrame.origin.y);
+
+  // Move by Y
+  while (p1.y > cellFrame.origin.y)
+    {
+      [[NSColor 
+	colorWithDeviceHue: h saturation: s brightness: white alpha: 1.0] set];
+      [NSBezierPath strokeLineFromPoint: p1 toPoint: p2];
+      
+      if (start_white > end_white)
+	white -= white_step;
+      else
+	white += white_step;
+
+      p1.y -= 1.0;
+      if (p2.x < (cellFrame.size.width + cellFrame.origin.x))
+	p2.x += 1.0;
+      else
+	p2.y -= 1.0;
+    }
+      
+  // Move by X
+  while (p1.x < (cellFrame.size.width + cellFrame.origin.x))
+    {
+      [[NSColor 
+	colorWithDeviceHue: h saturation: s brightness: white alpha: 1.0] set];
+      [NSBezierPath strokeLineFromPoint: p1 toPoint: p2];
+      
+      if (start_white > end_white)
+	white -= white_step;
+      else
+	white += white_step;
+
+      p1.x += 1.0;
+      if (p2.x >= (cellFrame.size.width + cellFrame.origin.x))
+	p2.y -= 1.0;
+      else
+	p2.x += 1.0;
+    }
+}
+
 - (void) drawInteriorWithFrame: (NSRect)cellFrame inView: (NSView*)controlView
 {
   unsigned	mask;
@@ -744,12 +871,12 @@
       titleToDisplay = [self attributedTitle];
     }
 
-  if (imageToDisplay)
+  if (imageToDisplay && ipos != NSNoImage)
     {
       imageSize = [imageToDisplay size];
     }
 
-  if (titleToDisplay && (ipos == NSImageAbove || ipos == NSImageBelow))
+  if (titleToDisplay && ipos != NSImageOnly)
     {
       titleSize = [titleToDisplay size];
     }
@@ -766,16 +893,36 @@
 	}
     }
   
+  /*
+  The size calculations here should be changed very carefully, and _must_ be
+  kept in sync with -cellSize. Changing the calculations to require more
+  space isn't OK; this breaks interfaces designed using the old sizes by
+  clipping away parts of the title.
+
+  The current size calculations ensure that for bordered or bezeled cells,
+  there's always at least a three point margin between the size returned by
+  -cellSize and the minimum size required not to clip text. (In other words,
+  the text can become three points wider (due to eg. font mismatches) before
+  you lose the last character.)
+  */
   switch (ipos)
     {
+      default:
       case NSNoImage: 
 	imageToDisplay = nil;
 	titleRect = cellFrame;
+	imageRect = NSZeroRect;
+	if (titleSize.width + 6 <= titleRect.size.width)
+	  {
+	    titleRect.origin.x += 3;
+	    titleRect.size.width -= 6;
+	  }
 	break;
 
       case NSImageOnly: 
 	titleToDisplay = nil;
 	imageRect = cellFrame;
+	titleRect = NSZeroRect;
 	break;
 
       case NSImageLeft: 
@@ -785,13 +932,14 @@
 	if (_cell.is_bordered || _cell.is_bezeled) 
 	  {
 	    imageRect.origin.x += 3;
-	    imageRect.size.height -= 2;
-	    imageRect.origin.y += 1;
 	  }
 	titleRect = imageRect;
-	titleRect.origin.x += imageSize.width + xDist;
-	titleRect.size.width = cellFrame.size.width - imageSize.width - xDist;
-        titleRect.size.width -= 3;
+	titleRect.origin.x += imageSize.width + GSCellTextImageXDist;
+	titleRect.size.width = NSMaxX(cellFrame) - titleRect.origin.x;
+	if (titleSize.width + 3 <= titleRect.size.width)
+	  {
+	    titleRect.size.width -= 3;
+	  }
 	break;
 
       case NSImageRight: 
@@ -802,14 +950,16 @@
 	if (_cell.is_bordered || _cell.is_bezeled) 
 	  {
 	    imageRect.origin.x -= 3;
-	    imageRect.size.height -= 2;
-	    imageRect.origin.y += 1;
 	  }
 	titleRect.origin = cellFrame.origin;
-	titleRect.size.width = cellFrame.size.width - imageSize.width - xDist;
+	titleRect.size.width = imageRect.origin.x - titleRect.origin.x
+			       - GSCellTextImageXDist;
 	titleRect.size.height = cellFrame.size.height;
-	titleRect.origin.x += 3;
-	titleRect.size.width -= 3;
+	if (titleSize.width + 3 <= titleRect.size.width)
+	  {
+	    titleRect.origin.x += 3;
+	    titleRect.size.width -= 3;
+	  }
 	break;
 
       case NSImageAbove: 
@@ -818,27 +968,28 @@
 	 * above the text. 
 	 * The drawing code below will then center the image in imageRect.
 	 */
-	titleRect.origin.x = cellFrame.origin.x;
-	titleRect.origin.y = cellFrame.origin.y;
+	titleRect.origin = cellFrame.origin;
 	titleRect.size.width = cellFrame.size.width;
 	titleRect.size.height = titleSize.height;
+	if (_cell.is_bordered || _cell.is_bezeled) 
+	  {
+	    titleRect.origin.y += 3;
+	  }
 
 	imageRect.origin.x = cellFrame.origin.x;
-	imageRect.origin.y = cellFrame.origin.y;
-	imageRect.origin.y += titleRect.size.height + yDist;
+	imageRect.origin.y = NSMaxY(titleRect) + GSCellTextImageYDist;
 	imageRect.size.width = cellFrame.size.width;
-	imageRect.size.height = cellFrame.size.height;
-	imageRect.size.height -= titleSize.height + yDist;
+	imageRect.size.height = NSMaxY(cellFrame) - imageRect.origin.y;
 
 	if (_cell.is_bordered || _cell.is_bezeled) 
 	  {
-	    imageRect.origin.x += 3;
-	    imageRect.size.width -= 6;
-	    imageRect.size.height -= 1;
+	    imageRect.size.height -= 3;
+	  }
+	if (titleSize.width + 6 <= titleRect.size.width)
+	  {
+	    titleRect.origin.x += 3;
+	    titleRect.size.width -= 6;
 	  }
-	titleRect.origin.x += 3;
-	titleRect.origin.y += 4;
-	titleRect.size.width -= 6;
 	break;
 
       case NSImageBelow: 
@@ -848,90 +999,92 @@
 	 * The drawing code below will then center the image in imageRect.
 	 */
 	titleRect.origin.x = cellFrame.origin.x;
-	titleRect.origin.y = cellFrame.origin.y + cellFrame.size.height;
-	titleRect.origin.y -= titleSize.height;
+	titleRect.origin.y = NSMaxY(cellFrame) - titleSize.height;
 	titleRect.size.width = cellFrame.size.width;
 	titleRect.size.height = titleSize.height;
+	if (_cell.is_bordered || _cell.is_bezeled)
+	  {
+	    titleRect.origin.y -= 3;
+	  }
 
 	imageRect.origin.x = cellFrame.origin.x;
 	imageRect.origin.y = cellFrame.origin.y;
 	imageRect.size.width = cellFrame.size.width;
-	imageRect.size.height = cellFrame.size.height;
-	imageRect.size.height -= titleSize.height + yDist;
+	imageRect.size.height = titleRect.origin.y - GSCellTextImageYDist - imageRect.origin.y;
 
 	if (_cell.is_bordered || _cell.is_bezeled) 
 	  {
-	    imageRect.size.width -= 6;
-	    imageRect.origin.x   += 3;
-	    imageRect.size.height -= 1;
-	    imageRect.origin.y    += 1;
+	    imageRect.origin.y += 3;
+	    imageRect.size.height -= 3;
+	  }
+	if (titleSize.width + 6 <= titleRect.size.width)
+	  {
+	    titleRect.origin.x += 3;
+	    titleRect.size.width -= 6;
 	  }
-	titleRect.size.width -= 6;
-	titleRect.origin.x   += 3;
-	titleRect.size.height -= 4;
 	break;
 
       case NSImageOverlaps: 
-	titleRect = cellFrame;
 	imageRect = cellFrame;
-	// TODO: Add distance from border if needed
+	titleRect = cellFrame;
+	if (titleSize.width + 6 <= titleRect.size.width)
+	  {
+	    titleRect.origin.x += 3;
+	    titleRect.size.width -= 6;
+	  }
 	break;
     }
+
+  // Draw gradient
+  if (!_cell.is_highlighted && _gradient_type != NSGradientNone)
+    {
+      [self drawGradientWithFrame: cellFrame inView: controlView];
+    }
+
+  // Draw image
   if (imageToDisplay != nil)
     {
-      NSSize size;
-      NSPoint position;
-
-      size = [imageToDisplay size];
-      position.x = MAX(NSMidX(imageRect) - (size.width/2.),0.);
-      position.y = MAX(NSMidY(imageRect) - (size.height/2.),0.);
-      /*
-       * Images are always drawn with their bottom-left corner at the origin
-       * so we must adjust the position to take account of a flipped view.
-       */
-      if (flippedView)
-	{
-	  position.y += size.height;
-	}
-      [imageToDisplay compositeToPoint: position operation: NSCompositeSourceOver];
+      [self _drawImage: imageToDisplay inFrame: imageRect isFlipped: flippedView];
     }
+
+  // Draw title
   if (titleToDisplay != nil)
     {
       [self _drawAttributedText: titleToDisplay inFrame: titleRect];
     }
 
+  // Draw first responder
   if (_cell.shows_first_responder
       && [[controlView window] firstResponder] == controlView)
     {
-	NSDottedFrameRect(cellFrame);
+      NSDottedFrameRect(cellFrame);
     }
 }
 
-- (NSSize) cellSize 
+- (NSSize) cellSize
 {
   NSSize s;
   NSSize borderSize;
   unsigned	mask;
   NSImage	*imageToDisplay;
   NSAttributedString	*titleToDisplay;
-  NSSize	imageSize;
-  NSSize	titleSize;
+  NSSize	imageSize = NSZeroSize;
+  NSSize	titleSize = NSZeroSize;
   
-  /* 
-   * The following code must be kept in sync with -drawInteriorWithFrame
-   */ 
+  /* The size calculations here must be kept in sync with
+  -drawInteriorWithFrame. */
+
   if (_cell.is_highlighted)
     {
       mask = _highlightsByMask;
+
+      if (_cell.state)
+	mask &= ~_showAltStateMask;
     }
   else if (_cell.state)
-    {
-      mask = _showAltStateMask;
-    }
+    mask = _showAltStateMask;
   else
-    {
-      mask = NSNoCellMask;
-    }
+    mask = NSNoCellMask;
   
   if (mask & NSContentsCellMask)
     {
@@ -956,22 +1109,15 @@
     {
       imageSize = [imageToDisplay size];
     }
-  else
-    { 
-      imageSize = NSZeroSize;
-    }
 
   if (titleToDisplay != nil)
     {
       titleSize = [titleToDisplay size];
     }
-  else
-    {
-      titleSize = NSZeroSize;
-    }
   
   switch (_cell.image_position)
     {
+      default:
       case NSNoImage: 
 	s = titleSize;
 	break;
@@ -982,57 +1128,39 @@
 	
       case NSImageLeft: 
       case NSImageRight: 
-	s.width = imageSize.width + titleSize.width + xDist;
-	if (imageSize.height > titleSize.height)
-	  s.height = imageSize.height;
-	else 
-	  s.height = titleSize.height;
+	s.width = imageSize.width + titleSize.width + GSCellTextImageXDist;
+	s.height = MAX(imageSize.height, titleSize.height);
 	break;
 	
       case NSImageBelow: 
       case NSImageAbove: 
-	if (imageSize.width > titleSize.width)
-	  s.width = imageSize.width;
-	else 
-	  s.width = titleSize.width;
-	s.height = imageSize.height + titleSize.height; // + yDist ??
+	s.width = MAX(imageSize.width, titleSize.width);
+	s.height = imageSize.height + titleSize.height + GSCellTextImageYDist;
 	break;
 	
       case NSImageOverlaps: 
-	if (imageSize.width > titleSize.width)
-	  s.width = imageSize.width;
-	else
-	  s.width = titleSize.width;
-	
-	if (imageSize.height > titleSize.height)
-	  s.height = imageSize.height;
-	else
-	  s.height = titleSize.height;
-
+	s.width = MAX(imageSize.width, titleSize.width);
+	s.height = MAX(imageSize.height, titleSize.height);
 	break;
     }
   
   // Get border size
   if (_cell.is_bordered)
     // Buttons only have three paths for border (NeXT looks)
-    borderSize = NSMakeSize (1.5, 1.5);
+    borderSize = NSMakeSize (3.0, 3.0);
   else
     borderSize = NSZeroSize;
 
-  if ((_cell.is_bordered || _cell.is_bezeled) 
-    && (_cell.image_position != NSImageOnly))
+  if ((_cell.is_bordered && (_cell.image_position != NSImageOnly))
+      || _cell.is_bezeled)
     {
-      borderSize.height += 1;
-      borderSize.width  += 3;
-
-      /* Add some more space because it looks better.  */
-      borderSize.height += 2;
-      borderSize.width  += 3;
+      borderSize.width += 6;
+      borderSize.height += 6;
     }
-  
+
   // Add border size
-  s.width += 2 * borderSize.width;
-  s.height += 2 * borderSize.height;
+  s.width += borderSize.width;
+  s.height += borderSize.height;
 
   return s;
 }
@@ -1081,13 +1209,19 @@
   [(NSView *)[event userData] setNeedsDisplay: YES];
 }
 
-- (void) performClick: (id)sender
+/**
+ * Simulates a single mouse click on the button cell. This method overrides the
+ * cell method performClickWithFrame:inView: to add the possibility to play a sound
+ * associated with the click. 
+ */
+- (void) performClickWithFrame: (NSRect)cellFrame inView: (NSView *)controlView
 {
   if (_sound != nil)
     {
       [_sound play];
     }
-  [super performClick: sender];
+    
+  [super performClickWithFrame: cellFrame inView: controlView];
 }
 
 /*
@@ -1206,3 +1340,40 @@
 }
 
 @end
+
+@implementation NSButtonCell (Private)
+
+// Overriden private internal method
+- (void) _drawImage: (NSImage *)anImage inFrame: (NSRect)aRect 
+  isFlipped: (BOOL)flipped
+{
+  // To keep partially in sync with the NSCell overriden method
+  
+  NSSize size;
+  NSPoint position;
+
+  size = [anImage size];
+  position.x = MAX(NSMidX(aRect) - (size.width / 2.), 0.);
+  position.y = MAX(NSMidY(aRect) - (size.height / 2.), 0.);
+  
+  /*
+   * Images are always drawn with their bottom-left corner at the origin
+   * so we must adjust the position to take account of a flipped view.
+   */
+  if (flipped)
+    {
+      position.y += size.height;
+    }
+	
+  if (_cell.is_disabled && _image_dims_when_disabled)
+    {
+      [anImage dissolveToPoint: position fraction: 0.5];
+    }
+  else
+    {
+      [anImage compositeToPoint: position 
+	                     operation: NSCompositeSourceOver];
+    }
+}
+
+@end
