@@ -60,6 +60,7 @@
 #include "AppKit/NSTextContainer.h"
 #include "AppKit/NSView.h"
 #include "AppKit/NSWindow.h"
+#include "GNUstepGUI/GSDrawFunctions.h"
 
 static Class	colorClass;
 static Class	cellClass;
@@ -343,7 +344,10 @@ static NSColor	*shadowCol;
 		    @"Attempt to use nil as string value");
     }
 
-  _cell.type = NSTextCellType;
+  if (_cell.type != NSTextCellType)
+    {
+      [self setType: NSTextCellType];
+    }
   _cell.contents_is_attributed_string = NO;
 
   if (_formatter == nil)
@@ -738,13 +742,13 @@ static NSColor	*shadowCol;
 
 - (void) setFont: (NSFont*)fontObject
 {
-  // This does not have any influence on attributed strings
-  ASSIGN (_font, fontObject);
-
   if (_cell.type != NSTextCellType)
     {
-      _cell.type = NSTextCellType;
+      [self setType: NSTextCellType];
     }
+
+  // This does not have any influence on attributed strings
+  ASSIGN (_font, fontObject);
 }
 
 - (void) setSelectable: (BOOL)flag
@@ -885,7 +889,7 @@ static NSColor	*shadowCol;
   return textObject;
 }
 
-- (NSString*) title						
+- (NSString*) title
 {
   return [self stringValue];
 }
@@ -969,7 +973,10 @@ static NSColor	*shadowCol;
 		NSInvalidArgumentException);
     }
   
-  _cell.type = NSImageCellType;    
+  if (_cell.type != NSImageCellType)
+    {
+      [self setType: NSImageCellType];
+    }
   
   ASSIGN (_cell_image, anImage);
 }
@@ -1143,41 +1150,57 @@ static NSColor	*shadowCol;
   _cell.refuses_first_responder = flag;
 }
 
+/**
+ * Simulates a single click in the cell (only works with controls which have
+ * no more than one cell). This method is deprecated,
+ * performClickWithFrame:inView: is the right method to use now.
+ */
 - (void) performClick: (id)sender
 {
-  SEL action = [self action];
   NSView *cv = [self controlView];
 
+  if (cv != nil) 
+    [self performClickWithFrame: [cv bounds] inView: cv];
+}
+
+/**
+ * Simulates a single click in the cell. The display of the cell with this event
+ * occurs in the area delimited by <var>cellFrame</var> within
+ * <var>controlView</var>.
+ */
+- (void) performClickWithFrame: (NSRect)cellFrame inView: (NSView *)controlView
+{
+  SEL action = [self action];
+
   if(_cell.is_disabled == YES)
     {
       return;
     }
 
-  if (cv != nil)
+  if (controlView != nil)
     {  
-      NSRect   cvBounds = [cv bounds];
-      NSWindow *cvWin = [cv window];
+      NSWindow *cvWin = [controlView window];
 
-      [cv lockFocus];
+      [controlView lockFocus];
       
       [self setNextState];
-      [self highlight: YES withFrame: cvBounds inView: cv];
+      [self highlight: YES withFrame: cellFrame inView: controlView];
       [cvWin flushWindow];
       
       // Wait approx 1/10 seconds
       [[NSRunLoop currentRunLoop] 
 	runUntilDate: [NSDate dateWithTimeIntervalSinceNow: 0.1]];
       
-      [self highlight: NO withFrame: cvBounds inView: cv];
+      [self highlight: NO withFrame: cellFrame inView: controlView];
       [cvWin flushWindow];
       
-      [cv unlockFocus];
+      [controlView unlockFocus];
 
       if (action)
 	{
 	  NS_DURING
 	    {
-	      [(NSControl*)cv sendAction: action to: [self target]];
+	      [(NSControl*)controlView sendAction: action to: [self target]];
 	    }
 	  NS_HANDLER
 	    {
@@ -1699,7 +1722,7 @@ static NSColor	*shadowCol;
     }
   else if (_cell.is_bezeled)
     {
-      NSDrawWhiteBezel(cellFrame, NSZeroRect);
+      [GSDrawFunctions drawWhiteBezel: cellFrame : NSZeroRect];
     }
 
   [self drawInteriorWithFrame: cellFrame inView: controlView];
@@ -1984,17 +2007,37 @@ static NSColor	*shadowCol;
     {
       NSString *contents = [aDecoder decodeObjectForKey: @"NSContents"];
       NSFont *support = [aDecoder decodeObjectForKey: @"NSSupport"];
-      int cFlags;
-      int cFlags2;
 
       if ([aDecoder containsValueForKey: @"NSCellFlags"])
         {
+	  int cFlags;
+
 	  cFlags = [aDecoder decodeIntForKey: @"NSCellFlags"];
+
+	  // This bit flag is the other way around!
+	  [self setWraps: ((cFlags & 0x40) != 0x40)];
+	  [self setContinuous: ((cFlags & 0x80000) == 0x80000)];
+	  [self setScrollable: ((cFlags & 0x100000) == 0x100000)];
+	  // Strange that this is not a simple bit flag
+	  [self setSelectable: ((cFlags & 0x200001) == 0x200001)];
+	  [self setBezeled: ((cFlags & 0x400000) == 0x400000)];
+	  [self setBordered: ((cFlags & 0x800000) == 0x800000)];
+	  [self setEditable: ((cFlags & 0x10000000) == 0x10000000)];
+	  // This bit flag is the other way around!
+	  [self setEnabled: ((cFlags & 0x20000000) != 0x20000000)];
+	  [self setHighlighted: ((cFlags & 0x40000000) == 0x40000000)];
 	  // FIXME
+	  
 	}
       if ([aDecoder containsValueForKey: @"NSCellFlags2"])
         {
+	  int cFlags2;
+      
 	  cFlags2 = [aDecoder decodeIntForKey: @"NSCellFlags2"];
+	  [self setAllowsMixedState: ((cFlags2 & 0x1000000) == 0x1000000)];
+	  [self setRefusesFirstResponder: ((cFlags2 & 0x2000000) == 0x2000000)];
+	  [self setImportsGraphics: ((cFlags2 & 0x20000000) == 0x20000000)];
+	  [self setAllowsEditingTextAttributes: ((cFlags2 & 0x40000000) == 0x40000000)];
 	  // FIXME
 	}
 
@@ -2189,6 +2232,11 @@ static NSColor	*shadowCol;
   RELEASE (attributes);
 }
 
+- (BOOL) _sendsActionOn:(int)eventTypeMask
+{
+  return (_action_mask & eventTypeMask);
+}
+
 @end
 
 /*
