@@ -2,10 +2,12 @@
 
    <abstract>The colorful color class</abstract>
 
-   Copyright (C) 1996, 1998 Free Software Foundation, Inc.
+   Copyright (C) 1996, 1998, 2001, 2002 Free Software Foundation, Inc.
 
    Author: Scott Christley <scottc@net-community.com>
    Date: 1996
+   Author: Fred Kiefer <fredkiefer@gmx.de>
+   Date: 2001, 2002
 
    This file is part of the GNUstep GUI Library.
 
@@ -25,7 +27,7 @@
    59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
-#include <gnustep/gui/config.h>
+#include "config.h"
 #include <Foundation/NSString.h>
 #include <Foundation/NSArchiver.h>
 #include <Foundation/NSDictionary.h>
@@ -34,14 +36,15 @@
 #include <Foundation/NSUserDefaults.h>
 #include <Foundation/NSBundle.h>
 #include <Foundation/NSDebug.h>
+#include <Foundation/NSScanner.h>
 
-#include <AppKit/NSColor.h>
-#include <AppKit/NSColorList.h>
-#include <AppKit/NSPasteboard.h>
-#include <AppKit/NSView.h>
-#include <AppKit/NSImage.h>
-#include <AppKit/NSGraphics.h>
-#include <AppKit/PSOperators.h>
+#include "AppKit/NSColor.h"
+#include "AppKit/NSColorList.h"
+#include "AppKit/NSPasteboard.h"
+#include "AppKit/NSView.h"
+#include "AppKit/NSImage.h"
+#include "AppKit/NSGraphics.h"
+#include "AppKit/PSOperators.h"
 
 static Class	NSColorClass;
 
@@ -49,10 +52,13 @@ static Class	NSColorClass;
 {
   NSString *_catalog_name;
   NSString *_color_name;
+  NSString *_cached_name_space;
+  NSColor *_cached_color;
 }
 
 - (NSColor*) initWithCatalogName: (NSString *)listName
 		       colorName: (NSString *)colorName;
+- (void) recache;
 
 @end
 
@@ -154,18 +160,17 @@ static Class	NSColorClass;
 static BOOL gnustep_gui_ignores_alpha = YES;
 static NSColorList		*systemColors = nil;
 static NSMutableDictionary	*colorStrings = nil;
-static SEL cwkSel;
-static NSColor*	(*cwkImp)(NSColorList*, SEL, NSString*);
+static NSMutableDictionary	*systemDict = nil;
 
 static
-void initSystemColors()
+void initSystemColors(void)
 {
   NSString *white;
   NSString *lightGray;
   NSString *gray;
   NSString *darkGray;
   NSString *black;
-
+  
   // Set up a dictionary containing the names of all the system colors
   // as keys and with colors in string format as values.
   white = [NSString stringWithFormat: @"%f %f %f",
@@ -178,7 +183,7 @@ void initSystemColors()
 		       NSDarkGray, NSDarkGray, NSDarkGray];
   black = [NSString stringWithFormat: @"%f %f %f",
 		    NSBlack, NSBlack, NSBlack];
-
+  
   colorStrings = [[NSMutableDictionary alloc]
 		     initWithObjectsAndKeys:
 			 lightGray, @"controlBackgroundColor",
@@ -212,52 +217,60 @@ void initSystemColors()
 		     //gray, @"windowFrameColor",
 		     //black, @"windowFrameTextColor",
 		     nil];
+  
+  systemColors = [NSColorList colorListNamed: @"System"];
+  if (systemColors == nil)
+    {
+      systemColors = [[NSColorList alloc] initWithName: @"System"];
+    }
 
-  // Set up default system colors
-  systemColors = [[NSColorList alloc] initWithName: @"System"];
-  cwkSel = @selector(colorWithKey:);
-  cwkImp = (NSColor*(*)(NSColorList*, SEL, NSString*))
-      [systemColors methodForSelector: cwkSel];
+    {
+      NSEnumerator *e;
+      NSString *r;
+      BOOL changed = NO;
+
+      // Set up default system colors
+
+      e = [colorStrings keyEnumerator];
+  
+      while ((r = (NSString *)[e nextObject])) 
+	{
+	  NSString *cs;
+	  NSColor *c;
+
+	  if ([systemColors colorWithKey: r])
+	    continue;
+
+	  cs = [colorStrings objectForKey: r];
+	  c = [NSColorClass colorFromString: cs];
+
+	  NSCAssert1(c, @"couldn't get default system color %@", r);
+
+	  [systemColors setColor: c forKey: r];
+
+	  changed = YES;
+	}
+
+      if (changed)
+	[systemColors writeToFile: nil];
+    }
+
+  systemDict = [NSMutableDictionary new];
 }
 
-/*
- * According to the specification this should return named colours, but
- * as we are currently missing the conversions to the device colour space
- * in all the places where this colours are used, we better return
- * the real colours here.
- */
 static NSColor*
 systemColorWithName(NSString *name)
 {
-  NSString	*rep;
-  NSColor	*color = (*cwkImp)(systemColors, cwkSel, name);
+  NSColor* col = [systemDict objectForKey: name];
 
-  if (color != nil)
-    return color;
-
-  rep = [colorStrings objectForKey: name];
-  if (rep == nil)
+  if (col == nil)
     {
-      NSLog(@"Request for unknown system color - '%@'\n", name);
-      return nil;
+      col = [NSColor colorWithCatalogName: @"System"
+		     colorName: name];
+      [systemDict setObject: col forKey: name];
     }
 
-  if (NSColorClass == 0)
-    {
-      color = [NSColor colorFromString: rep];
-    }
-  else
-    {
-      color = [NSColorClass colorFromString: rep];
-    }
-  if (color == nil)
-    {
-      NSLog(@"System color '%@' has bad string rep - '%@'\n", name, rep);
-      return nil;
-    }
-  [systemColors setColor: color forKey: name];
-
-  return color;
+  return col;
 }
 
 @implementation NSColor
@@ -943,22 +956,33 @@ systemColorWithName(NSString *name)
 {
   NSColor *myColor = [self colorUsingColorSpaceName: NSCalibratedRGBColorSpace];
   NSColor *other = [aColor colorUsingColorSpaceName: NSCalibratedRGBColorSpace];
-  float mr, mg, mb, or, og, ob, red, green, blue;
+  float mr, mg, mb, ma, or, og, ob, oa, red, green, blue, alpha;
+
+  if (fraction <= 0.0)
+    {
+      return self;
+    }
+
+  if (fraction >= 1.0)
+    {
+      return aColor;
+    }
 
   if (myColor == nil || other == nil)
     {
       return nil;
     }
 
-  [myColor getRed: &mr green: &mg blue: &mb alpha: 0];
-  [other getRed: &or green: &og blue: &ob alpha: 0];
-  red = fraction * mr + (1 - fraction) * or;
-  green = fraction * mg + (1 - fraction) * og;
-  blue = fraction * mb + (1 - fraction) * ob;
+  [myColor getRed: &mr green: &mg blue: &mb alpha: &ma];
+  [other getRed: &or green: &og blue: &ob alpha: &oa];
+  red = fraction * or + (1 - fraction) * mr;
+  green = fraction * og + (1 - fraction) * mg;
+  blue = fraction * ob + (1 - fraction) * mb;
+  alpha = fraction * oa + (1 - fraction) * ma;
   return [NSColorClass colorWithCalibratedRed: red
 					green: green
 					 blue: blue
-					alpha: 1.0];
+					alpha: alpha];
 }
 
 - (NSColor*) colorWithAlphaComponent: (float)alpha
@@ -1020,7 +1044,72 @@ systemColorWithName(NSString *name)
 
 - (id) initWithCoder: (NSCoder*)aDecoder
 {
-  if ([aDecoder versionForClassName: @"NSColor"] < 3)
+  if ([aDecoder allowsKeyedCoding])
+    {
+      int colorSpace = [aDecoder decodeIntForKey: @"NSColorSpace"];
+
+      DESTROY(self);
+      if (colorSpace == 1)
+        {
+	  unsigned length;
+	  const uint8_t *data;
+	  float red = 0.0;
+	  float green = 0.0;
+	  float blue = 0.0;
+	  float alpha = 0.0;
+	  NSString *str;
+	  NSScanner *scanner;
+	  
+	  if ([aDecoder containsValueForKey: @"NSRGB"])
+	    {
+	      data = [aDecoder decodeBytesForKey: @"NSRGB"
+			     returnedLength: &length]; 
+	      str = [[NSString alloc] initWithCString: data length: length];
+	      scanner = [[NSScanner alloc] initWithString: str];
+	      [scanner scanFloat: &red];
+	      [scanner scanFloat: &green];
+	      [scanner scanFloat: &blue];
+	    }
+	  
+	  self = [NSColor colorWithCalibratedRed: red
+			  green: green
+			  blue: blue
+			  alpha: alpha];
+	}
+      else if (colorSpace == 3)
+        {
+	  unsigned length;
+	  const uint8_t *data;
+	  float white = 0.0;
+	  float alpha = 0.0;
+	  NSString *str;
+	  NSScanner *scanner;
+	  
+	  if ([aDecoder containsValueForKey: @"NSWhite"])
+	    {
+	      data = [aDecoder decodeBytesForKey: @"NSWhite"
+			     returnedLength: &length]; 
+	      str = [[NSString alloc] initWithCString: data length: length];
+	      scanner = [[NSScanner alloc] initWithString: str];
+	      [scanner scanFloat: &white];
+	    }
+	  
+	  self = [NSColor colorWithDeviceWhite: white
+			  alpha: alpha];
+	}
+      else if (colorSpace == 6)
+        {
+	  NSString *catalog = [aDecoder decodeObjectForKey: @"NSCatalogName"];
+	  NSString *name = [aDecoder decodeObjectForKey: @"NSColorName"];
+	  //NSColor *color = [aDecoder decodeObjectForKey: @"NSColor"];
+	  
+	  self = [NSColor colorWithCatalogName: catalog
+			  colorName: name];
+	}
+      
+      return self;
+    }
+  else if ([aDecoder versionForClassName: @"NSColor"] < 3)
     {
       float red;
       float green;
@@ -1236,16 +1325,21 @@ systemColorWithName(NSString *name)
     }
   else if (str != nil)
     {
-      const char	*ptr = [str cString];
       float		r, g, b;
+      NSScanner *scanner = [[NSScanner alloc] initWithString: str];
 
-      if (sscanf(ptr, "%f %f %f", &r, &g, &b) == 3)
+      if ([scanner scanFloat: &r] &&
+	  [scanner scanFloat: &g] &&
+	  [scanner scanFloat: &b] &&
+	  [scanner isAtEnd])
 	{
 	  return [self colorWithCalibratedRed: r
 					green: g
 					 blue: b
 					alpha: 1.0];
 	}
+
+      DESTROY(scanner);
     }
 
   return nil;
@@ -1254,10 +1348,9 @@ systemColorWithName(NSString *name)
 /*
  *	Go through all the names of system colors - for each color where
  *	there is a value in the defaults database, see if the current
- *	string value of the color differs from the old one.
+ *	value of the color differs from the old one.
  *	Where there is a difference, update the color strings dictionary
- *	and, where a color object exists, update the system colors list
- *	to contain the new color.
+ *	and update the system colors list to contain the new color.
  *	Finally, issue a notification if appropriate.
  */
 + (void) defaultsDidChange: (NSNotification*)notification
@@ -1276,28 +1369,21 @@ systemColorWithName(NSString *name)
 
       if (def != nil)
 	{
-	  NSString	*old = [colorStrings objectForKey: key];
+	  NSColor *old = [systemColors colorWithKey: key];
+	  NSColor *color = [NSColor colorFromString: def];
 
-	  if ([def isEqualToString: old] == NO)
+	  if (color == nil)
+	    {
+	      NSLog(@"System color '%@' has bad string rep - '%@'\n",
+		    key, def);
+	    }
+	  else if ([color isEqual: old] == NO)
 	    {
-	      NSColor *color;
-
 	      didChange = YES;
 	      [colorStrings setObject: def forKey: key];
-	      color = [systemColors colorWithKey: key];
-	      if (color != nil)
-		{
-		  color = [NSColor colorFromString: def];
-		  if (color == nil)
-		    {
-		      NSLog(@"System color '%@' has bad string rep - '%@'\n",
-			    key, def);
-		    }
-		  else
-		    {
-		      [systemColors setColor: color forKey: key];
-		    }
-		}
+	      [systemColors setColor: color forKey: key];
+	      // Refresh the cache for this named colour
+	      [[systemDict objectForKey: key] recache];
 	    }
 	}
     }
@@ -1328,6 +1414,8 @@ systemColorWithName(NSString *name)
 {
   RELEASE(_catalog_name);
   RELEASE(_color_name);
+  RELEASE(_cached_name_space);
+  RELEASE(_cached_color);
   [super dealloc];
 }
 
@@ -1348,7 +1436,9 @@ systemColorWithName(NSString *name)
 
       aCopy->_catalog_name = [_catalog_name copyWithZone: aZone];
       aCopy->_color_name = [_color_name copyWithZone: aZone];
-      return aCopy;
+      aCopy->_cached_name_space = nil;
+      aCopy->_cached_color = nil;
+	 return aCopy;
     }
 }
 
@@ -1429,11 +1519,29 @@ systemColorWithName(NSString *name)
       return self;
     }
 
+
+  // Is there a cache hit?
+  // FIXME How would we detect that the cache has become invalid by a
+  // change to the colour list?
+  if ([colorSpace isEqualToString: _cached_name_space])
+    {
+      return _cached_color;
+    }
+
   list = [NSColorList colorListNamed: _catalog_name];
   real = [list colorWithKey: _color_name];
 
-  return [real colorUsingColorSpaceName: colorSpace
-	       device: deviceDescription];
+  ASSIGN(_cached_color, [real colorUsingColorSpaceName: colorSpace
+		       device: deviceDescription]);
+  ASSIGN(_cached_name_space, colorSpace);
+
+  return _cached_color;
+}
+
+- (void) recache
+{
+  DESTROY(_cached_name_space);
+  DESTROY(_cached_color);
 }
 
 //
@@ -2425,8 +2533,7 @@ systemColorWithName(NSString *name)
 
 - (NSString *)colorSpaceName
 {
-//  return NSPatternImageColorSpace;
-  return nil;
+  return NSPatternColorSpace;
 }
 
 - (NSImage*) patternImage
