@@ -509,9 +509,15 @@ static Class actionCellClass;
 /*
  * Activation
  */
+
+/**
+ * Simulates a single mouse click on the control. This method calls the cell's
+ * method performClickWithFrame:inView:. Take note that <var>sender</var> is not
+ * used. 
+ */
 - (void) performClick: (id)sender
 {
-  [_cell performClick: sender];
+  [_cell performClickWithFrame: [self bounds] inView: self];
 }
 
 - (BOOL)refusesFirstResponder
@@ -547,7 +553,7 @@ static Class actionCellClass;
     return;
 
   if (_ignoresMultiClick && ([theEvent clickCount] > 1))
-    {
+    {  
       [super mouseDown: theEvent];
       return;
     }
