@@ -26,16 +26,19 @@
    59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */ 
 
-#include <gnustep/gui/config.h>
+#include "config.h"
 #include <Foundation/NSArray.h>
 #include <Foundation/NSSet.h>
-#include <AppKit/NSFontManager.h>
-#include <AppKit/NSApplication.h>
-#include <AppKit/NSFont.h>
-#include <AppKit/NSFontPanel.h>
-#include <AppKit/NSMenu.h>
-#include <AppKit/NSMenuItem.h>
-#include <AppKit/GSFontInfo.h>
+#include <Foundation/NSString.h>
+#include <Foundation/NSValue.h>
+#include <Foundation/NSDebug.h>
+#include "AppKit/NSFontManager.h"
+#include "AppKit/NSApplication.h"
+#include "AppKit/NSFont.h"
+#include "AppKit/NSFontPanel.h"
+#include "AppKit/NSMenu.h"
+#include "AppKit/NSMenuItem.h"
+#include "GNUstepGUI/GSFontInfo.h"
 
 
 /*
@@ -46,9 +49,6 @@ static NSFontPanel	*fontPanel = nil;
 static Class		fontManagerClass = Nil;
 static Class		fontPanelClass = Nil;
 
-@interface NSFontManager (GNUstepPrivate)
-- (BOOL) _includeFont: (NSString*)fontName;
-@end
 
 @implementation NSFontManager
 
@@ -59,8 +59,6 @@ static Class		fontPanelClass = Nil;
 {
   if (self == [NSFontManager class])
     {
-      NSDebugLog(@"Initialize NSFontManager class\n");
-
       // Initial version
       [self setVersion: 1];
 
@@ -87,7 +85,6 @@ static Class		fontPanelClass = Nil;
 {
   if (!sharedFontManager)
     {
-      NSDebugLog(@"Initializing NSFontManager fonts\n");
       sharedFontManager = [[fontManagerClass alloc] init];
     }
   return sharedFontManager;
@@ -125,20 +122,7 @@ static Class		fontPanelClass = Nil;
  */
 - (NSArray*) availableFonts
 {
-  int i;
-  NSArray *fontsList = [_fontEnumerator availableFonts];
-  NSMutableArray *fontNames = [NSMutableArray 
-				  arrayWithCapacity: [fontsList count]];
-
-  for (i = 0; i < [fontsList count]; i++)
-    {
-      NSString *name = [fontsList objectAtIndex: i];
-      
-      if ([self _includeFont: name])
-	[fontNames addObject: name];
-    }
-
-  return fontNames;
+  return [_fontEnumerator availableFonts];
 }
 
 - (NSArray*) availableFontFamilies
@@ -148,7 +132,7 @@ static Class		fontPanelClass = Nil;
 
 - (NSArray*) availableFontNamesWithTraits: (NSFontTraitMask)fontTraitMask
 {
-  int i, j;
+  unsigned int i, j;
   NSArray *fontFamilies = [self availableFontFamilies];
   NSMutableArray *fontNames = [NSMutableArray array];
   NSFontTraitMask traits;
@@ -156,7 +140,7 @@ static Class		fontPanelClass = Nil;
   for (i = 0; i < [fontFamilies count]; i++)
     {
       NSArray *fontDefs = [self availableMembersOfFontFamily: 
-				  [fontFamilies objectAtIndex: i]];
+				 [fontFamilies objectAtIndex: i]];
       
       for (j = 0; j < [fontDefs count]; j++)
 	{
@@ -165,12 +149,7 @@ static Class		fontPanelClass = Nil;
 	  traits = [[fontDef objectAtIndex: 3] unsignedIntValue];
 	  // Check if the font has exactly the given mask
 	  if (traits == fontTraitMask)
-	    {
-	      NSString *name = [fontDef objectAtIndex: 0];
-	
-	      if ([self _includeFont: name])
-		[fontNames addObject: name];
-	    }
+	    [fontNames addObject: [fontDef objectAtIndex: 0]];
 	}
     }
 
@@ -179,21 +158,7 @@ static Class		fontPanelClass = Nil;
 
 - (NSArray*) availableMembersOfFontFamily: (NSString*)family
 {
-  int i;
-  NSArray *fontsList = [_fontEnumerator availableMembersOfFontFamily: family];
-  NSMutableArray *fonts = [NSMutableArray 
-			      arrayWithCapacity: [fontsList count]];
-
-  for (i = 0; i < [fontsList count]; i++)
-    {
-      NSArray *fontDef = [fontsList objectAtIndex: i];
-      NSString *name = [fontDef objectAtIndex: 0];
-      
-      if ([self _includeFont: name])
-	[fonts addObject: fontDef];
-    }
-
-  return fonts;
+  return [_fontEnumerator availableMembersOfFontFamily: family];
 }
 
 - (NSString*) localizedNameForFamily: (NSString*)family 
@@ -217,7 +182,9 @@ static Class		fontPanelClass = Nil;
 	  _multiple = flag;
 	  // The panel should also know if multiple changed
 	  if (fontPanel != nil)
-	    [fontPanel setPanelFont: fontObject isMultiple: flag];
+	    {
+	      [fontPanel setPanelFont: fontObject isMultiple: flag];
+	    }
 	}
       return;
     }
@@ -226,11 +193,13 @@ static Class		fontPanelClass = Nil;
   ASSIGN(_selectedFont, fontObject);
 
   if (fontPanel != nil)
-    [fontPanel setPanelFont: fontObject isMultiple: flag];
+    {
+      [fontPanel setPanelFont: fontObject isMultiple: flag];
+    }
   
   if (_fontMenu != nil)
     {
-      NSMenuItem *menuItem;
+      id <NSMenuItem> menuItem;
       NSFontTraitMask trait = [self traitsOfFont: fontObject];
 
       /*
@@ -368,7 +337,7 @@ static Class		fontPanelClass = Nil;
 - (NSFont*) convertFont: (NSFont*)fontObject
 {
   NSFont *newFont = fontObject;
-  int i;
+  unsigned int i;
   float size;
   float sizes[] = {4.0, 6.0, 8.0, 9.0, 10.0, 11.0, 12.0, 13.0, 
 		   14.0, 16.0, 18.0, 24.0, 36.0, 48.0, 64.0};
@@ -492,12 +461,12 @@ static Class		fontPanelClass = Nil;
   else if (trait == NSUnboldFontMask)
     {
       return [self convertFont: fontObject 
-		   toNotHaveTrait: NSBoldFontMask];
+		toNotHaveTrait: NSBoldFontMask];
    }
   else if (trait == NSUnitalicFontMask)
     {
       return [self convertFont: fontObject 
-		   toNotHaveTrait: NSItalicFontMask];
+		toNotHaveTrait: NSItalicFontMask];
    }
   else
     {
@@ -532,10 +501,13 @@ static Class		fontPanelClass = Nil;
 
   // This is a bit strange but is stated in the specification
   if (trait & NSUnboldFontMask)
+    {
       trait = (trait | NSBoldFontMask) & ~NSUnboldFontMask;
+    }
   if (trait & NSUnitalicFontMask)
+    {
       trait = (trait | NSItalicFontMask) & ~NSUnitalicFontMask;
-
+    }
   if (!(t & trait))
     {
       // If already do not have that trait then just return it
@@ -552,8 +524,9 @@ static Class		fontPanelClass = Nil;
 
       // We cannot reuse the weight in an unbold
       if (trait & NSBoldFontMask)
-	weight = 5;
-
+	{
+	  weight = 5;
+	}
       t &= ~trait;
       newFont = [self fontWithFamily: family 
 			      traits: t
@@ -603,7 +576,7 @@ static Class		fontPanelClass = Nil;
 
   if (upFlag)
     {
-      int i;
+      unsigned int i;
       // The documentation is a bit unclear about the range of weights
       // sometimes it says 0 to 9 and sometimes 0 to 15
       int next_w = 15;
@@ -642,7 +615,7 @@ static Class		fontPanelClass = Nil;
     }
   else
     {
-      int i;
+      unsigned int i;
       int next_w = 0;
 
       for (i = 0; i < [fontDefs count]; i++)
@@ -650,8 +623,8 @@ static Class		fontPanelClass = Nil;
 	  NSArray *fontDef = [fontDefs objectAtIndex: i];
 	  int w1 = [[fontDef objectAtIndex: 2] intValue];
 
-	  if (w1 < w && w1 > next_w && 
-	      [[fontDef objectAtIndex: 3] unsignedIntValue] == trait)
+	  if (w1 < w && w1 > next_w
+	    && [[fontDef objectAtIndex: 3] unsignedIntValue] == trait)
 	    {
 	      next_w = w1;
 	      fontName = [fontDef objectAtIndex: 0];
@@ -668,8 +641,8 @@ static Class		fontPanelClass = Nil;
 	      NSArray *fontDef = [fontDefs objectAtIndex: i];
 	      int w1 = [[fontDef objectAtIndex: 2] intValue];
 	      
-	      if (w1 < w && w1 > next_w && 
-		  [[fontDef objectAtIndex: 3] unsignedIntValue] == trait)
+	      if (w1 < w && w1 > next_w
+		&& [[fontDef objectAtIndex: 3] unsignedIntValue] == trait)
 	        {
 		  next_w = w1;
 		  fontName = [fontDef objectAtIndex: 0];
@@ -679,9 +652,10 @@ static Class		fontPanelClass = Nil;
     }
 
   if (fontName != nil)
-    newFont = [NSFont fontWithName: fontName
-		      size: size];
-
+    {
+      newFont = [NSFont fontWithName: fontName
+				size: size];
+    }
   if (newFont == nil)
     return fontObject;
   else 
@@ -697,7 +671,7 @@ static Class		fontPanelClass = Nil;
 		      size: (float)size
 {
   NSArray *fontDefs = [self availableMembersOfFontFamily: family];
-  int i;
+  unsigned int i;
 
   //NSLog(@"Searching font %@: %i: %i", family, weight, traits);  
 
@@ -784,9 +758,9 @@ static Class		fontPanelClass = Nil;
 //
 // Examining a font
 //
-- (NSFontTraitMask) traitsOfFont: (NSFont*)fontObject
+- (NSFontTraitMask) traitsOfFont: (NSFont*)aFont
 {
-  return [[fontObject fontInfo] traits];
+  return [[aFont fontInfo] traits];
 }
 
 - (int) weightOfFont: (NSFont*)fontObject
@@ -799,7 +773,7 @@ static Class		fontPanelClass = Nil;
 {
   // TODO: This method is implemented very slow, but I dont 
   // see any use for it, so why change it?
-  int i, j;
+  unsigned int i, j;
   NSArray *fontFamilies = [self availableFontFamilies];
   NSFontTraitMask traits;
   
@@ -865,7 +839,7 @@ static Class		fontPanelClass = Nil;
 {
   if (create && _fontMenu == nil)
     {
-      NSMenuItem *menuItem;
+      id <NSMenuItem> menuItem;
       
       // As the font menu is stored in a instance variable we 
       // dont autorelease it
@@ -875,7 +849,7 @@ static Class		fontPanelClass = Nil;
       // First an entry to start the font panel
       menuItem = [_fontMenu addItemWithTitle: @"Font Panel"
 			    action: @selector(orderFrontFontPanel:)
-			    keyEquivalent: @"f"];
+			    keyEquivalent: @"t"];
       [menuItem setTarget: self];
 
       // Entry for italic
@@ -984,18 +958,3 @@ static Class		fontPanelClass = Nil;
 
 @end
 
-@implementation NSFontManager (GNUstepPrivate)
-
-/*
- * Ask delegate if to include a font
- */
-- (BOOL) _includeFont: (NSString*)fontName
-{
-  if ((_delegate != nil)
-    && [_delegate respondsToSelector: @selector(fontManager: willIncludeFont:)])
-    return [_delegate fontManager: self willIncludeFont: fontName];
-  else
-    return YES;
-}
-
-@end
