@@ -1302,7 +1302,7 @@ container
  
 #define GBUF_SIZE 16 /* TODO: tweak */
   NSGlyph gbuf[GBUF_SIZE];
-  int gbuf_len;
+  int gbuf_len, gbuf_size;
   NSPoint gbuf_point;
 
   NSView *controlView = nil;
@@ -1311,6 +1311,11 @@ container
     return;
   [self _doLayoutToGlyph: range.location + range.length - 1];
 
+  if ([ctxt isDrawingToScreen])
+    gbuf_size = GBUF_SIZE;
+  else
+    gbuf_size = 1;
+
   for (i = 0, tc = textcontainers; i < num_textcontainers; i++, tc++)
     if (tc->pos + tc->length > range.location)
       break;
@@ -1437,11 +1442,14 @@ container
 		  cellFrame.size = la->size;
 		  cellFrame.origin.y -= cellFrame.size.height;
 
-		  /* Drawing the cell might mess up our state. */
+		  /* Drawing the cell might mess up our state, so we reset
+		  the font and color afterwards. */
 		  /* TODO:
-		  optimize this. probably cheaper to not save and
-		  explicitly reset just the stuff we actually use, or to
-		  collect attachments and draw them in bunches of eg. 4
+		  optimize this?
+		  collect attachments and draw them in bunches of eg. 4?
+
+		  probably not worth effort. better to optimize font and
+		  color setting :)
 
 		  should they really be drawn in our coordinate system?
 		  */
@@ -1450,10 +1458,6 @@ container
 		    characterIndex: char_index
 		    layoutManager: self];
 		  [f set];
-		  if (color == nil)
-		    {
-		      color = defaultTextColor;
-		    }
 		  [color set];
 		}
 	      continue;
@@ -1468,10 +1472,10 @@ container
 		}
 	      else
 		{
-		  if (gbuf_len == GBUF_SIZE)
+		  if (gbuf_len == gbuf_size)
 		    {
 		      DPSmoveto(ctxt, gbuf_point.x, gbuf_point.y);
-		      GSShowGlyphs(ctxt, gbuf, GBUF_SIZE);
+		      GSShowGlyphs(ctxt, gbuf, gbuf_size);
 		      DPSnewpath(ctxt);
 		      gbuf_len = 0;
 		      gbuf_point = p;
@@ -1773,9 +1777,7 @@ this file describes this.
     invalidatedRange: (NSRange)invalidatedRange
 {
   NSRange r;
-  unsigned int original_last_glyph, new_last_glyph;
-  int glyph_delta;
-  BOOL had_layout;
+  unsigned int original_last_glyph;
 
 /*  printf("\n*** invalidating\n");
   [self _dumpLayout];*/
@@ -1788,7 +1790,6 @@ this file describes this.
   TODO: make sure last_glyph is set as expected
   */
   original_last_glyph = layout_glyph;
-  had_layout = layout_char > 0;
 
   if (!(mask & NSTextStorageEditedCharacters))
     lengthChange = 0;
@@ -1797,26 +1798,17 @@ this file describes this.
 	changeInLength: lengthChange
 	actualCharacterRange: &r];
 
-  if (layout_char > r.location)
-    {
-      layout_char += lengthChange;
-      if (layout_char < r.location)
-        layout_char = r.location;
-    }
+  /*
+  If we had layout information and we had layout information for the range
+  of characters that was modified, we need to invalidate layout information.
 
-  if (!layout_char)
-    new_last_glyph = 0;
-  else if (layout_char == [_textStorage length])
-    new_last_glyph = [self numberOfGlyphs];
-  else
-    new_last_glyph = [self glyphRangeForCharacterRange: NSMakeRange(layout_char, 1)
-				  actualCharacterRange: NULL].location;
-
-  glyph_delta = new_last_glyph - original_last_glyph;
-/*  printf("original=%i, new=%i, delta %i\n",
-    original_last_glyph,new_last_glyph,glyph_delta);*/
-
-  if (had_layout && r.location <= layout_char)
+  TODO: This is broken. Even if we don't have layout for the modified
+  characters, we might have layout for the preceeding line, and we then need
+  to invalidate that line. Need to rework this a bit... I really really need
+  to know the glyph length change here. :/
+  (Alexander Malmberg 2004-03-22)
+  */
+  if (layout_char > 0 && layout_char >= r.location)
     {
       unsigned int glyph_index, last_glyph;
       textcontainer_t *tc;
@@ -1824,10 +1816,38 @@ this file describes this.
       int i, j, k;
       int new_num;
       NSRange char_range;
+      unsigned int new_last_glyph;
+      int glyph_delta;
 
       /*
-      Note that r.location might equal layout_char, in which case
-      r.location won't actually have any text container or line frag.
+      If we had layout beyond the modified characters, update layout_char.
+      Otherwise, just pretend that we have layout up to the end of the range
+      after the change. This will give glyph_delta and last_glyph incorrect
+      values, strictly speaking, but glyph_delta is only used if we have
+      layout beyond the modified range, and last_glyph is used in a way that
+      makes it safe to overestimate it (as we do here).
+
+      When I can get exact information about the modified glyphs (TODO above),
+      all this will become much cleaner...
+      */
+      if (layout_char >= r.location + r.length - lengthChange)
+	layout_char += lengthChange;
+      else
+	layout_char = r.location + r.length;
+
+      if (!layout_char)
+	new_last_glyph = 0;
+      else if (layout_char == [_textStorage length])
+	new_last_glyph = [self numberOfGlyphs];
+      else
+	new_last_glyph = [self glyphRangeForCharacterRange: NSMakeRange(layout_char, 1)
+				      actualCharacterRange: NULL].location;
+
+      glyph_delta = new_last_glyph - original_last_glyph;
+
+      /*
+      Note that r.location might not actually have any text container or
+      line frag.
       */
       if (!r.location)
 	{
@@ -2074,7 +2094,7 @@ no_soft_invalidation:
 
   [self _didInvalidateLayout];
 
-  if ((mask & NSTextStorageEditedCharacters) && lengthChange)
+  if (mask & NSTextStorageEditedCharacters)
     {
       /*
       Adjust the selected range so it's still valid. We don't try to
@@ -2096,7 +2116,7 @@ no_soft_invalidation:
       --------------------------
       after     after     location += lengthChange;
       in        after     length = NSMaxRange(sel)-NSMaxRange(range)-lengthChange; location=NSMaxRange(range);
-      in        in        length = 0; location=range.location;
+      in        in        length = 0; location=NSMaxRange(range);
       before    after     length += lengthChange;
       before    in        length = range.location-location;
       before    before    do nothing
@@ -2104,37 +2124,46 @@ no_soft_invalidation:
       In other words, unless the selection spans over the entire changed
       range, the changed range is deselected.
 
-      One important property of this behavios is that if length is 0 before,
+      One important property of this behavior is that if length is 0 before,
       it will be 0 after.
       */
+      NSRange newRange = _selected_range;
+
       if (_selected_range.location >= NSMaxRange(range) - lengthChange)
 	{ /* after after */
-	  _selected_range.location += lengthChange;
+	  newRange.location += lengthChange;
 	}
       else if (_selected_range.location >= range.location)
 	{
 	  if (NSMaxRange(_selected_range) > NSMaxRange(range) - lengthChange)
 	    { /* in after */
-	      _selected_range.length = NSMaxRange(_selected_range) - NSMaxRange(range) - lengthChange;
-	      _selected_range.location = NSMaxRange(range);
+	      newRange.length = NSMaxRange(_selected_range) - NSMaxRange(range) - lengthChange;
+	      newRange.location = NSMaxRange(range);
 	    }
 	  else
 	    { /* in in */
-	      _selected_range.length = 0;
-	      _selected_range.location = range.location;
+	      newRange.length = 0;
+	      newRange.location = NSMaxRange(range);
 	    }
 	}
       else if (NSMaxRange(_selected_range) > NSMaxRange(range) - lengthChange)
 	{ /* before after */
-	  _selected_range.length += lengthChange;
+	  newRange.length += lengthChange;
 	}
       else if (NSMaxRange(_selected_range) > range.location)
 	{ /* before in */
-	  _selected_range.length = range.location - _selected_range.location;
+	  newRange.length = range.location - _selected_range.location;
 	}
       else
 	{ /* before before */
 	}
+
+      /* If there are text views attached to use, let them handle the
+      change. */
+      if ([self firstTextView])
+	[[self firstTextView] setSelectedRange: newRange];
+      else
+	_selected_range = newRange;
     }
 }
 
