@@ -32,7 +32,7 @@
    59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */ 
 
-#include <gnustep/gui/config.h>
+#include "config.h"
 #include <Foundation/NSCoder.h>
 #include <Foundation/NSArray.h>
 #include <Foundation/NSException.h>
@@ -40,40 +40,102 @@
 #include <Foundation/NSString.h>
 #include <Foundation/NSNotification.h>
 
-#include <AppKit/NSMatrix.h>
-#include <AppKit/NSApplication.h>
-#include <AppKit/NSWindow.h>
-#include <AppKit/NSEvent.h>
-#include <AppKit/NSFont.h>
-#include <AppKit/NSMenu.h>
-#include <AppKit/NSMenuItem.h>
-#include <AppKit/NSMenuView.h>
-#include <AppKit/NSPopUpButton.h>
-#include <AppKit/NSPopUpButtonCell.h>
-#include <AppKit/NSScreen.h>
+#include "AppKit/NSMatrix.h"
+#include "AppKit/NSApplication.h"
+#include "AppKit/NSWindow.h"
+#include "AppKit/NSEvent.h"
+#include "AppKit/NSFont.h"
+#include "AppKit/NSImage.h"
+#include "AppKit/NSMenu.h"
+#include "AppKit/NSMenuItem.h"
+#include "AppKit/NSMenuView.h"
+#include "AppKit/NSPanel.h"
+#include "AppKit/NSPopUpButton.h"
+#include "AppKit/NSPopUpButtonCell.h"
+#include "AppKit/NSScreen.h"
+#include "AppKit/NSAttributedString.h"
 
-/* A menu's title is an instance of this class */
-@interface NSMenuWindowTitleView : NSView
-{
-  id  menu;
-  NSButton *button;
-}
 
-- (void) addCloseButton;
-- (void) releaseCloseButton;
-- (void) createButton;
-- (void) setMenu: (NSMenu*)menu;
-- (NSMenu*) menu;
+/*
+  Drawing related:
 
+              NSMenu superMenu   (if not root menu, the parent meu)
+                ^
+                |
+                |    +------------------> NSMenuView view  (content, draws the menu items)
+                |    |
+              NSMenu +----------+-------> NSMenuPanel A    (regular window, torn off window)
+                |    |          `-------> NSMenuPanel B    (transient window)
+                |    |           
+                |    +------------------> NSString title   (title)
+                |
+                v
+              NSMenu attachedMenu  (the menu that is attached to this one, during navigation)
+              
+
+
+   +--[NSMenuPanel]------+
+   | +-[NSMenuView]----+ |
+   | | title if applic | |
+   | | +-------------+ | |
+   | | | NSMenuItem- | | |
+   | | | Cell        | | |
+   | | +-------------+ | |
+   | |       .         | |
+   | |       .         | |
+   | +-----------------+ |
+   +---------------------+
+
+   The two windows
+   ---------------
+
+   Basically we have for a menu two windows, window A and window B.
+   Window A is the regular window and Window B is used for transient windows.
+
+   At any one time, the views, like title view, NSMenuView are put either in
+   window A or in window B.  They are moved over from one window to the oter
+   when needed.
+
+   the code is supposed to know when it is using window A or B.
+   But it will probably only work correctly when
+
+   window A correspond to _transient == NO
+   window B correspond to _transient == YES
+*/
+
+
+/* Subclass of NSPanel since menus cannot become key */
+@interface NSMenuPanel : NSPanel
 @end
 
+@interface NSMenuView (GNUstepPrivate)
+- (NSArray *)_itemCells;
+@end
+
+
 static NSZone	*menuZone = NULL;
 static NSString	*NSMenuLocationsKey = @"NSMenuLocations";
+static NSString *NSEnqueuedMenuMoveName = @"EnqueuedMoveNotificationName";
 static NSNotificationCenter *nc;
 
 @interface	NSMenu (GNUstepPrivate)
-- (NSString*) _locationKey;
-- (NSPanel*) _createWindow;
+
+- (NSString *) _locationKey;
+- (NSMenuPanel *) _createWindow;
+- (void) _updateUserDefaults: (id) notification;
+- (void) _rightMouseDisplay: (NSEvent*)theEvent;
+
+@end
+
+
+@implementation NSMenuPanel
+- (BOOL) canBecomeKeyWindow
+{
+  /* See [NSWindow-_lossOfKeyOrMainWindow] */
+  if (self == (NSMenuPanel *)[[NSApp mainMenu] window])
+    return YES;
+  return NO;
+}
 @end
 
 @implementation	NSMenu (GNUstepPrivate)
@@ -103,20 +165,95 @@ static NSNotificationCenter *nc;
 }
 
 /* Create a non autorelease window for this menu.  */
-- (NSPanel*) _createWindow
+- (NSMenuPanel*) _createWindow
 {
-  NSPanel *win = [[NSPanel alloc] 
+  NSMenuPanel *win = [[NSMenuPanel alloc] 
 		     initWithContentRect: NSZeroRect
 		     styleMask: NSBorderlessWindowMask
 		     backing: NSBackingStoreBuffered
 		     defer: YES];
   [win setLevel: NSSubmenuWindowLevel];
-  [win setWorksWhenModal: YES];
+  [win setWorksWhenModal: NO];
   [win setBecomesKeyOnlyIfNeeded: YES];
 
   return win;
 }
 
+/**
+   Will track the mouse movement.  It will trigger the updating of the user
+   defaults in due time.
+*/
+- (void) _menuMoved: (id) notification
+{
+  NSNotification *resend;
+
+  resend = [NSNotification notificationWithName: NSEnqueuedMenuMoveName
+					 object: self];
+  
+  [[NSNotificationQueue defaultQueue]
+    enqueueNotification: resend
+    postingStyle: NSPostASAP
+    coalesceMask: NSNotificationCoalescingOnSender
+    forModes:  [NSArray arrayWithObject: NSDefaultRunLoopMode]];
+}
+
+/**
+   Save the current menu position in the standard user defaults
+*/
+- (void) _updateUserDefaults: (id) notification
+{
+  NSString *key;
+
+  NSDebugLLog (@"NSMenu", @"Synchronizing user defaults");
+  key = [self _locationKey];
+  if (key != nil)
+    {
+      NSUserDefaults		*defaults;
+      NSMutableDictionary	*menuLocations;
+      NSString			*locString;
+
+      defaults = [NSUserDefaults standardUserDefaults];
+      menuLocations = [defaults objectForKey: NSMenuLocationsKey];
+      if ([menuLocations isKindOfClass: [NSDictionary class]])
+	menuLocations = [menuLocations mutableCopy];
+      else
+	menuLocations = nil;
+
+      if ([_aWindow isVisible]
+	&& ([self isTornOff] || ([NSApp mainMenu] == self)))
+        {
+          if (menuLocations == nil)
+            {
+              menuLocations = AUTORELEASE([[NSMutableDictionary alloc]
+		initWithCapacity: 2]);
+            }
+          locString = [[self window] stringWithSavedFrame];
+          [menuLocations setObject: locString forKey: key];
+        }
+      else
+        {
+          [menuLocations removeObjectForKey: key];
+        }
+
+      if ([menuLocations count] > 0)
+        {
+          [defaults setObject: menuLocations forKey: NSMenuLocationsKey];
+        }
+      else
+        {
+          [defaults removeObjectForKey: NSMenuLocationsKey];
+        }
+      [defaults synchronize];
+    }
+}
+
+- (void) _rightMouseDisplay: (NSEvent*)theEvent
+{
+  [self displayTransient];
+  [_view mouseDown: theEvent];
+  [self closeTransient];
+}
+
 @end
 
 
@@ -146,33 +283,35 @@ static NSNotificationCenter *nc;
 }
 
 /*
- * Initializing a New NSMenu
+ *
  */
 - (id) init
 {
-  return [self initWithTitle: @"Menu"];
+  return [self initWithTitle: [[NSProcessInfo processInfo] processName]];
 }
 
 - (void) dealloc
 {
-  NSDebugLog (@"NSMenu `%@' dealloc", _title);
-
   [nc removeObserver: self];
 
+  // Now clean the pointer to us stored each _items element
+  [_items makeObjectsPerformSelector: @selector(setMenu:) withObject: nil];
+
   RELEASE(_notifications);
   RELEASE(_title);
   RELEASE(_items);
   RELEASE(_view);
   RELEASE(_aWindow);
   RELEASE(_bWindow);
-  RELEASE(_titleView);
 
   [super dealloc];
 }
 
+/*
+   
+*/
 - (id) initWithTitle: (NSString*)aTitle
 {
-  float  height;
   NSView *contentView;
 
   [super init];
@@ -183,19 +322,12 @@ static NSNotificationCenter *nc;
   // Create an array to store out menu items.
   _items = [[NSMutableArray alloc] init];
 
-  // We have no supermenu.
-  // _superMenu = nil;
-  // _is_tornoff = NO;
-  // _follow_transient = NO;
-
   _changedMessagesEnabled = YES;
   _notifications = [[NSMutableArray alloc] init];
-  _changed = YES;
+  _needsSizing = YES;
   // According to the spec, menus do autoenable by default.
   _autoenable = YES;
 
-  // Transient windows private stuff.
-  // _oldAttachedMenu = nil;
 
   /* Please note that we own all this menu network of objects.  So, 
      none of these objects should be retaining us.  When we are deallocated,
@@ -205,20 +337,14 @@ static NSNotificationCenter *nc;
   // Create the windows that will display the menu.
   _aWindow = [self _createWindow];
   _bWindow = [self _createWindow];
+  [_bWindow setLevel: NSPopUpMenuWindowLevel];
 
   // Create a NSMenuView to draw our menu items.
-  _view = [[NSMenuView alloc] initWithFrame: NSMakeRect(0,0,50,50)];
+  _view = [[NSMenuView alloc] initWithFrame: NSZeroRect];
   [_view setMenu: self];
 
-  // Create the title view
-  height = [[_view class] menuBarHeight];
-  _titleView = [[NSMenuWindowTitleView alloc] initWithFrame: 
-						NSMakeRect(0, 0, 50, height)];
-  [_titleView setMenu: self];
-
   contentView = [_aWindow contentView];
   [contentView addSubview: _view];
-  [contentView addSubview: _titleView];
 
   /* Set up the notification to start the process of redisplaying
      the menus where the user left them the last time.  
@@ -232,6 +358,21 @@ static NSNotificationCenter *nc;
       name: NSApplicationDidFinishLaunchingNotification 
       object: NSApp];
 
+  [nc addObserver: self
+      selector: @selector(_showOnActivateApp:)
+      name: NSApplicationWillBecomeActiveNotification 
+      object: NSApp];
+
+  [nc addObserver: self
+      selector: @selector (_menuMoved:)
+      name: NSWindowDidMoveNotification
+      object: _aWindow];
+
+  [nc addObserver: self
+      selector: @selector (_updateUserDefaults:)
+      name: NSEnqueuedMenuMoveName
+      object: self];
+  
   return self;
 }
 
@@ -255,10 +396,13 @@ static NSNotificationCenter *nc;
    * isn't added.
    */
   if ([newItem menu] != nil)
-    return;
+    {
+      NSLog(@"The object %@ is already attached to a menu, then it isn't possible to add it.\n", newItem);
+      return;
+    }
   
   [_items insertObject: newItem atIndex: index];
-  _changed = YES;
+  _needsSizing = YES;
   
   // Create the notification for the menu representation.
   d = [NSDictionary
@@ -276,9 +420,6 @@ static NSNotificationCenter *nc;
 
   // Set this after the insert notification has been send.
   [newItem setMenu: self];
-
-  // Update the menu.
-  [self update];
 }
 
 - (id <NSMenuItem>) insertItemWithTitle: (NSString*)aString
@@ -333,7 +474,7 @@ static NSNotificationCenter *nc;
 
   [anItem setMenu: nil];
   [_items removeObjectAtIndex: index];
-  _changed = YES;
+  _needsSizing = YES;
   
   d = [NSDictionary dictionaryWithObject: [NSNumber numberWithInt: index]
 		    forKey: @"NSMenuItemIndex"];
@@ -346,9 +487,6 @@ static NSNotificationCenter *nc;
     [nc postNotification: removed];
   else
     [_notifications addObject: removed];
-
-  // Update the menu.
-  [self update];
 }
 
 - (void) itemChanged: (id <NSMenuItem>)anObject
@@ -360,7 +498,7 @@ static NSNotificationCenter *nc;
   if (-1 == index)
     return;
 
-  _changed = YES;
+  _needsSizing = YES;
 
   d = [NSDictionary dictionaryWithObject: [NSNumber numberWithInt: index]
 		    forKey: @"NSMenuItemIndex"];
@@ -413,7 +551,7 @@ static NSNotificationCenter *nc;
 
 - (id <NSMenuItem>) itemAtIndex: (int)index
 {
-  if (index >= [_items count] || index < 0)
+  if (index >= (int)[_items count] || index < 0)
     [NSException  raise: NSRangeException
 		 format: @"Range error in method -itemAtIndex: "];
 
@@ -532,18 +670,34 @@ static NSNotificationCenter *nc;
 
 - (void) submenuAction: (id)sender
 {
-  [_view detachSubmenu];
 }
 
+
 - (NSMenu *) attachedMenu
 {
-  if (_attachedMenu && _follow_transient
-      && !_attachedMenu->_follow_transient)
+  if (_attachedMenu && _transient
+      && !_attachedMenu->_transient)
     return nil;
 
   return _attachedMenu;
 }
 
+
+/**
+   Look for the semantics in the header.  Note that
+   this implementation works because there are ... cases:
+   <enum>
+   <item>
+   This menu is transient, its supermenu is also transient.
+   In this case we just do the check between the transient windows
+   and everything is fine
+   </item>
+   <item>
+   The menu is transient, its supermenu is not transient.
+   This can go WRONG
+   </item>
+   </enum>
+*/
 - (BOOL) isAttached
 {
   return _superMenu && [_superMenu attachedMenu] == self;
@@ -602,7 +756,6 @@ static NSNotificationCenter *nc;
 	{
 	  NSMenuItem *item = [_items objectAtIndex: i];
 	  SEL	      action = [item action];
-	  id	      target;
 	  id	      validator = nil;
 	  BOOL	      wasEnabled = [item isEnabled];
 	  BOOL	      shouldBeEnabled;
@@ -614,41 +767,30 @@ static NSNotificationCenter *nc;
 	  // If there is no action - there can be no validator for the item.
 	  if (action)
 	    {
-	      // If there is a target use that for validation (or nil).
-	      if (nil != (target = [item target]))
-		{
-		  if ([target respondsToSelector: action])
-		    {  
-		      validator = target;
-		    }
-		}
-	      else
-		{
-		  validator = [NSApp targetForAction: action];
-		}
+	      validator = [NSApp targetForAction: action 
+				 to: [item target]
+				 from: item];
 	    }
 	  else if (_popUpButtonCell != nil)
 	    {
 	      if (NULL != (action = [_popUpButtonCell action]))
 	        {
-		  // If there is a target use that for validation (or nil).
-		  if (nil != (target = [_popUpButtonCell target]))
-		    {
-		      if ([target respondsToSelector: action])
-		        {  
-			  validator = target;
-			}
-		    }
-		  else
-		    {
-		      validator = [NSApp targetForAction: action];
-		    }
+		  validator = [NSApp targetForAction: action
+				     to: [_popUpButtonCell target]
+				     from: [_popUpButtonCell controlView]];
 		}
 	    }
 
 	  if (validator == nil)
 	    {
-	      shouldBeEnabled = NO;
+	      if ((action == NULL) && (_popUpButtonCell != nil))
+		{
+		  shouldBeEnabled = YES;
+		}
+	      else 
+		{
+		  shouldBeEnabled = NO;
+		}
 	    }
 	  else if ([validator
 		     respondsToSelector: @selector(validateMenuItem:)])
@@ -670,9 +812,12 @@ static NSNotificationCenter *nc;
       [self setMenuChangedMessagesEnabled: YES];
     }
 
-  if (_changed)
-    [self sizeToFit];
-
+  if (_needsSizing && ([_aWindow isVisible] || [_bWindow isVisible]))
+    {
+      NSDebugLLog (@"NSMenu", @" Calling Size To Fit (A)");
+      [self sizeToFit];
+    }
+  
   return;
 }
 
@@ -708,7 +853,10 @@ static NSNotificationCenter *nc;
           if ([[item keyEquivalent] isEqualToString: 
 				      [theEvent charactersIgnoringModifiers]])
 	    {
-	      [_view performActionWithHighlightingForItemAtIndex: i];
+	      if ([item isEnabled])
+		{
+		  [_view performActionWithHighlightingForItemAtIndex: i];
+		}
 	      return YES;
 	    }
 	}
@@ -733,7 +881,15 @@ static NSNotificationCenter *nc;
   [nc postNotificationName: NSMenuWillSendActionNotification
                     object: self
                   userInfo: d];
-  if (NULL != (action = [item action]))
+
+
+  if (_popUpButtonCell != nil)
+    {
+      // Tell the popup button, which item was selected
+      [_popUpButtonCell selectItemAtIndex: index];
+    }
+
+  if ((action = [item action]) != NULL)
     {
       [NSApp sendAction: action
 	     to: [item target]
@@ -741,9 +897,7 @@ static NSNotificationCenter *nc;
     }
   else if (_popUpButtonCell != nil)
     {
-      // Tell the popup button, which item was selected
-      [_popUpButtonCell selectItemAtIndex: index];
-      if (NULL != (action = [_popUpButtonCell action]))
+      if ((action = [_popUpButtonCell action]) != NULL)
 	[NSApp sendAction: action
 	       to: [_popUpButtonCell target]
 	       from: [_popUpButtonCell controlView]];
@@ -760,7 +914,12 @@ static NSNotificationCenter *nc;
 - (void) setTitle: (NSString*)aTitle
 {
   ASSIGN(_title, aTitle);
-  [self sizeToFit];
+
+  _needsSizing = YES;
+  if ([_aWindow isVisible] || [_bWindow isVisible])
+    {
+      [self sizeToFit];
+    }
 }
   
 - (NSString*) title
@@ -800,6 +959,9 @@ static NSNotificationCenter *nc;
 //
 // Updating the Menu Layout
 //
+// Wim 20030301: Question, what happens when the notification trigger
+// new notifications?  I think it is not allowed to add items
+// to the _notifications array while enumerating it.
 - (void) setMenuChangedMessagesEnabled: (BOOL)flag
 { 
   if (_changedMessagesEnabled != flag)
@@ -830,47 +992,51 @@ static NSNotificationCenter *nc;
 
 - (void) sizeToFit
 {
-  NSRect windowFrame;
+  NSRect oldWindowFrame;
+  NSRect newWindowFrame;
   NSRect menuFrame;
   NSSize size;
 
-  //if ([_view needsSizing])
-    [_view sizeToFit];
-
+  [_view sizeToFit];
+  
   menuFrame = [_view frame];
   size = menuFrame.size;
-
-  windowFrame = [_aWindow frame];
-
+ 
+	// Main
+  oldWindowFrame = [_aWindow frame];
+  newWindowFrame = [NSWindow frameRectForContentRect: menuFrame
+                             styleMask: [_aWindow styleMask]];
+  
+  if (oldWindowFrame.size.height > 1)
+    {
+      newWindowFrame.origin = NSMakePoint (oldWindowFrame.origin.x,
+                                           oldWindowFrame.origin.y
+                                           + oldWindowFrame.size.height
+                                           - newWindowFrame.size.height);
+    }
+  [_aWindow setFrame: newWindowFrame display: NO];
+  
+  // Transient
+  oldWindowFrame = [_bWindow frame];
+  newWindowFrame = [NSWindow frameRectForContentRect: menuFrame
+                             styleMask: [_bWindow styleMask]];
+  if (oldWindowFrame.size.height > 1)
+    {
+      newWindowFrame.origin = NSMakePoint (oldWindowFrame.origin.x,
+                                           oldWindowFrame.origin.y
+                                           + oldWindowFrame.size.height
+                                           - newWindowFrame.size.height);
+    }
+  [_bWindow setFrame: newWindowFrame display: NO];
+  
   if (_popUpButtonCell == nil)
     {
-      float height = [[_view class] menuBarHeight];
-
-      size.height += height;
-      [_aWindow setContentSize: size];
-      [_aWindow setFrameTopLeftPoint:
-	NSMakePoint(NSMinX(windowFrame),NSMaxY(windowFrame))];
-
-      windowFrame = [_bWindow frame];
-      [_bWindow setContentSize: size];
-      [_bWindow setFrameTopLeftPoint:
-	NSMakePoint(NSMinX(windowFrame),NSMaxY(windowFrame))];
-
       [_view setFrameOrigin: NSMakePoint (0, 0)];
-      [_titleView setFrame: NSMakeRect (0, size.height - height, 
-				       size.width, height)];
-      [_titleView setNeedsDisplay: YES];
     }
-  else
-    {
-      [_aWindow setContentSize: size];
-      [_aWindow setFrameTopLeftPoint:
-	NSMakePoint(NSMinX(windowFrame),NSMaxY(windowFrame))];
-    }
-
+  
   [_view setNeedsDisplay: YES];
-
-  _changed = NO;
+  
+  _needsSizing = NO;
 }
 
 /*
@@ -885,7 +1051,7 @@ static NSNotificationCenter *nc;
 		withEvent: (NSEvent*)event
 		  forView: (NSView*)view
 {
-  // TODO
+  [menu _rightMouseDisplay: event];
 }
 
 /*
@@ -914,17 +1080,25 @@ static NSNotificationCenter *nc;
   [encoder encodeValueOfObjCType: @encode(BOOL) at: &_autoenable];
 }
 
-- (id) initWithCoder: (NSCoder*)decoder
+- (id) initWithCoder: (NSCoder*)aDecoder
 {
   NSString	*dTitle;
   NSArray	*dItems;
   BOOL		dAuto;
   unsigned	i;
 
-  dTitle = [decoder decodeObject];
-  dItems = [decoder decodeObject];
-  [decoder decodeValueOfObjCType: @encode(BOOL) at: &dAuto];
-
+  if ([aDecoder allowsKeyedCoding])
+    {
+      dAuto = YES;
+      dTitle = [aDecoder decodeObjectForKey: @"NSTitle"];
+      dItems = [aDecoder decodeObjectForKey: @"NSMenuItems"];
+    }
+  else
+    {
+      dTitle = [aDecoder decodeObject];
+      dItems = [aDecoder decodeObject];
+      [aDecoder decodeValueOfObjCType: @encode(BOOL) at: &dAuto];
+    }
   self = [self initWithTitle: dTitle];
   [self setAutoenablesItems: dAuto];
 
@@ -978,23 +1152,22 @@ static NSNotificationCenter *nc;
 #define IS_OFFSCREEN(WINDOW) \
   !(NSContainsRect([[NSScreen mainScreen] frame], [WINDOW frame]))
 
-- (void) _setTornOff: (BOOL)flag
+- (void) setTornOff: (BOOL)flag
 {
   NSMenu	*supermenu;
 
-  _is_tornoff = flag;
+  _is_tornoff = flag; 
 
   if (flag)
-    [_titleView addCloseButton];
-  else
-    [_titleView releaseCloseButton];
-
-  supermenu = [self supermenu];
-  if (supermenu != nil)
     {
-      [[supermenu menuRepresentation] setHighlightedItemIndex: -1];
-      supermenu->_attachedMenu = nil;
+      supermenu = [self supermenu];
+      if (supermenu != nil)
+        {
+          [[supermenu menuRepresentation] setHighlightedItemIndex: -1];
+          supermenu->_attachedMenu = nil;
+        }
     }
+  [_view update];
 }
 
 - (void) _showTornOffMenuIfAny: (NSNotification*)notification
@@ -1012,122 +1185,81 @@ static NSNotificationCenter *nc;
 
 	  defaults  = [NSUserDefaults standardUserDefaults];
 	  menuLocations = [defaults objectForKey: NSMenuLocationsKey];
-      
-	  location = [menuLocations objectForKey: key];
+
+	  if ([menuLocations isKindOfClass: [NSDictionary class]])
+	    location = [menuLocations objectForKey: key];
+	  else
+	    location = nil;
 	  if (location && [location isKindOfClass: [NSString class]])
 	    {
-	      [self _setTornOff: YES];
+	      [self setTornOff: YES];
 	      [self display];
 	    }
 	}
     }
 }
 
-- (BOOL) isFollowTransient
+- (void) _showOnActivateApp: (NSNotification*)notification
 {
-  return _follow_transient;
+  if ([NSApp mainMenu] == self)
+  {
+    [self display];
+    // we must make sure that any attached submenu is visible too.
+    [[self attachedMenu] display];
+  }
+}
+
+- (BOOL) isTransient
+{
+  return _transient;
 } 
 
 - (BOOL) isPartlyOffScreen
 {
-  return _isPartlyOffScreen;
-}
-
-- (void) nestedCheckOffScreen
-{
-  // This method is used when the menu is moved.
-  if (_attachedMenu)
-    [_attachedMenu nestedCheckOffScreen];
-
-  _isPartlyOffScreen = IS_OFFSCREEN(_aWindow);
+  return IS_OFFSCREEN ([self window]);
 }
 
 - (void) _performMenuClose: (id)sender
 {
-  NSString		*key;
-
   if (_attachedMenu)
     [_view detachSubmenu];
-
-  key = [self _locationKey];
-  if (key != nil)
-    {
-      NSUserDefaults		*defaults;
-      NSMutableDictionary	*menuLocations;
-
-      defaults = [NSUserDefaults standardUserDefaults];
-      menuLocations = [[defaults objectForKey: NSMenuLocationsKey] mutableCopy];
-      [menuLocations removeObjectForKey: key];
-      if ([menuLocations count] > 0)
-        [defaults setObject: menuLocations forKey: NSMenuLocationsKey];
-      else
-        [defaults removeObjectForKey: NSMenuLocationsKey];
-      RELEASE(menuLocations);
-      [defaults synchronize];
-    }
-
+  
   [_view setHighlightedItemIndex: -1];
-  [self _setTornOff: NO];
   [self close];
+  [self setTornOff: NO];
+  [self _updateUserDefaults: nil];
 } 
 
-- (void) _rightMouseDisplay: (NSEvent*)theEvent
-{
-  [self displayTransient];
-  [_view mouseDown: theEvent];
-  [self closeTransient];
-}
-
 - (void) display
 {
-  NSString *key;
-
-  if (_changed)
-    [self sizeToFit];
+  if (_transient)
+    {
+      NSDebugLLog (@"NSMenu", 
+                   @"trying to display while alreay displayed transient");
+    }
 
+  if (_needsSizing)
+    {
+      [self sizeToFit];
+    }
+  
   if (_superMenu && ![self isTornOff])
     {                 
       // query super menu for position
       [_aWindow setFrameOrigin: [_superMenu locationForSubmenu: self]];
       _superMenu->_attachedMenu = self;
     }
-  else if (nil != (key = [self _locationKey]))
+  else if ([_aWindow frame].origin.y <= 0 
+           && _popUpButtonCell == nil)   // get geometry only if not set
     {
-      NSUserDefaults *defaults;
-      NSDictionary *menuLocations;
-      NSString *location;
-
-      defaults = [NSUserDefaults standardUserDefaults];
-      menuLocations = [defaults objectForKey: NSMenuLocationsKey];
-      location = [menuLocations objectForKey: key];
-      if (location && [location isKindOfClass: [NSString class]])
-        {
-	  [_aWindow setFrameFromString: location];
-	  /*
-	   * May need resize in case saved frame is out of sync
-	   * with number of items in menu.
-	   */
-	  [self sizeToFit];
-	}
-      else
-        {
-	  NSPoint aPoint = {0, [[NSScreen mainScreen] frame].size.height
-			    - [_aWindow frame].size.height};
-	  
-	  [_aWindow setFrameOrigin: aPoint];
-	  [_bWindow setFrameOrigin: aPoint];
-	}
+      [self setGeometry];
     }
-
+  
+  NSDebugLLog (@"NSMenu", 
+               @"Display, origin: %@", 
+               NSStringFromPoint ([_aWindow frame].origin));
+  
   [_aWindow orderFrontRegardless];
-
-  _isPartlyOffScreen = IS_OFFSCREEN(_aWindow);
-
-  /*
-   * If we have just been made visible, we must make sure that any attached
-   * submenu is visible too.
-   */
-  [[self attachedMenu] display];
 }
 
 - (void) displayTransient
@@ -1135,8 +1267,20 @@ static NSNotificationCenter *nc;
   NSPoint location;
   NSView *contentView;
 
-  _follow_transient = YES;
+  if (_transient)
+    {
+      NSDebugLLog (@"NSMenu", @"displaying transient while it is transient");
+      return;
+    }
 
+  if (_needsSizing)
+    {
+      [self sizeToFit];
+    }
+  
+  _oldHiglightedIndex = [[self menuRepresentation] highlightedItemIndex];
+  _transient = YES;
+  
   /*
    * Cache the old submenu if any and query the supermenu our position.
    * Otherwise, raise menu under the mouse.
@@ -1162,24 +1306,63 @@ static NSNotificationCenter *nc;
   [_bWindow setFrameOrigin: location];
 
   [_view removeFromSuperviewWithoutNeedingDisplay];
-  [_titleView removeFromSuperviewWithoutNeedingDisplay];
-
-  if (_is_tornoff)
-    [_titleView releaseCloseButton];
 
   contentView = [_bWindow contentView];
   [contentView addSubview: _view];
-  [contentView addSubview: _titleView];
 
+  [_view update];
+  
   [_bWindow orderFront: self];
+}
 
-  _isPartlyOffScreen = IS_OFFSCREEN(_bWindow);
+- (void) setGeometry
+{
+  NSString       *key;
+  NSUserDefaults *defaults;
+  NSDictionary   *menuLocations;
+  NSString       *location;
+  
+  NSPoint        origin;
+  NSScanner      *scanner;
+  int            value;
+
+  origin = NSMakePoint (0, [[NSScreen mainScreen] frame].size.height 
+      - [_aWindow frame].size.height);
+      
+  if (nil != (key = [self _locationKey]))
+    {
+      defaults = [NSUserDefaults standardUserDefaults];
+      menuLocations = [defaults objectForKey: NSMenuLocationsKey];
+
+      if ([menuLocations isKindOfClass: [NSDictionary class]])
+	location = [menuLocations objectForKey: key];
+      else
+	location = nil;
+ 
+      if (location && [location isKindOfClass: [NSString class]])
+        {
+          scanner = [NSScanner scannerWithString: location];
+          [scanner scanInt: &value];
+          origin.x = value;
+          [scanner scanInt: &value];
+          origin.y = value;
+        }
+    }
+  
+  [_aWindow setFrameOrigin: origin];
+  [_bWindow setFrameOrigin: origin];
 }
 
 - (void) close
 {
   NSMenu *sub = [self attachedMenu];
 
+
+  if (_transient)
+    {
+      NSDebugLLog (@"NSMenu", @"We should not close ordinary menu while transient version is still open");
+    }
+  
   /*
    * If we have an attached submenu, we must close that too - but then make
    * sure we still have a record of it so that it can be re-displayed if we
@@ -1192,39 +1375,49 @@ static NSNotificationCenter *nc;
     }
   [_aWindow orderOut: self];
 
-  if (_superMenu)
-    _superMenu->_attachedMenu = nil;
+  if (_superMenu && ![self isTornOff])
+    {
+      _superMenu->_attachedMenu = nil;
+      [[_superMenu menuRepresentation] setHighlightedItemIndex: -1];
+    }
 }
 
 - (void) closeTransient
 {
   NSView *contentView;
+
+  if (_transient == NO)
+    {
+      NSDebugLLog (@"NSMenu",
+	@"Closing transient: %@ while it is NOT transient now", _title);
+      return;
+    }
   
   [_bWindow orderOut: self];
   [_view removeFromSuperviewWithoutNeedingDisplay];
-  [_titleView removeFromSuperviewWithoutNeedingDisplay];
 
   contentView = [_aWindow contentView];
   [contentView addSubview: _view];
 
-  if (_is_tornoff)
-    [_titleView addCloseButton];
-
-  [contentView addSubview: _titleView];
-  [contentView setNeedsDisplay: YES];
-
+  [contentView setNeedsDisplay: YES]; 
+  
   // Restore the old submenu (if any).
   if (_superMenu != nil)
-    _superMenu->_attachedMenu = _oldAttachedMenu;
+    {
+      _superMenu->_attachedMenu = _oldAttachedMenu;
+      [[_superMenu menuRepresentation] setHighlightedItemIndex:
+	[_superMenu indexOfItemWithSubmenu: _superMenu->_attachedMenu]];
+    }
 
-  _follow_transient = NO;
-
-  _isPartlyOffScreen = IS_OFFSCREEN(_aWindow);
+  [[self menuRepresentation] setHighlightedItemIndex: _oldHiglightedIndex];
+  
+  _transient = NO;
+  [_view update];
 }
 
 - (NSWindow*) window
 {
-  if (_follow_transient)
+  if (_transient)
     return (NSWindow *)_bWindow;
   else
     return (NSWindow *)_aWindow;
@@ -1237,7 +1430,7 @@ static NSNotificationCenter *nc;
 */
 - (void) nestedSetFrameOrigin: (NSPoint) aPoint
 {
-  NSWindow *theWindow = _follow_transient ? _bWindow : _aWindow;
+  NSWindow *theWindow = [self window];
 
   // Move ourself and get our width.
   [theWindow setFrameOrigin: aPoint];
@@ -1254,63 +1447,60 @@ static NSNotificationCenter *nc;
 
 - (void) shiftOnScreen
 {
-  NSWindow *theWindow = _follow_transient ? _bWindow : _aWindow;
+  NSWindow *theWindow = _transient ? _bWindow : _aWindow;
   NSRect    frameRect = [theWindow frame];
+  NSRect    screenRect = [[NSScreen mainScreen] frame];
   NSPoint   vector    = {0.0, 0.0};
-  BOOL      moveIt    = YES;
-
-  if (frameRect.origin.y < 0)
+  BOOL      moveIt    = NO;
+  
+  // 1 - determine the amount we need to shift in the y direction.
+  if (NSMinY (frameRect) < 0)
     {
-      if (frameRect.origin.y + SHIFT_DELTA <= 0)
-	vector.y = SHIFT_DELTA;
-      else
-	vector.y = -frameRect.origin.y;
+      vector.y = MIN (SHIFT_DELTA, -NSMinY (frameRect));
+      moveIt = YES;
     }
-  else if (frameRect.origin.x < 0)
+  else if (NSMaxY (frameRect) > NSMaxY (screenRect))
     {
-      if (frameRect.origin.x + SHIFT_DELTA <= 0)
-	vector.x = SHIFT_DELTA;
-      else
-	vector.x = -frameRect.origin.x;
-    }
-  else
-    {
-      vector.x  = frameRect.origin.x + frameRect.size.width;
-      vector.x -= [[NSScreen mainScreen] frame].size.width;
-
-      if (vector.x > 0)
-	{
-	  if (vector.x - SHIFT_DELTA <= 0)
-	    vector.x = -SHIFT_DELTA;
-	  else
-	    vector.x = -vector.x - 2;
-	}
-      else
-	moveIt = NO;
+      vector.y = -MIN (SHIFT_DELTA, NSMaxY (frameRect) - NSMaxY (screenRect));
+      moveIt = YES;
     }
 
+  // 2 - determine the amount we need to shift in the x direction.
+  if (NSMinX (frameRect) < 0)
+    {
+      vector.x = MIN (SHIFT_DELTA, -NSMinX (frameRect));
+      moveIt = YES;
+    }
+  // Note the -3.  This is done so the menu, after shifting completely
+  // has some spare room on the right hand side.  This is needed otherwise
+  // the user can never access submenus of this menu.
+  else if (NSMaxX (frameRect) > NSMaxX (screenRect) - 3)
+    {
+      vector.x
+	= -MIN (SHIFT_DELTA, NSMaxX (frameRect) - NSMaxX (screenRect) + 3);
+      moveIt = YES;
+    }
+  
   if (moveIt)
     {
-        NSMenu  *candidateMenu;
-	NSMenu  *masterMenu;
-	NSPoint  masterLocation;
-	NSPoint  destinationPoint;
-
-	// Look for the "master" menu, i.e. the one to move from.
-	for (candidateMenu = masterMenu = self;
-	     (candidateMenu = masterMenu->_superMenu)
-	       && (!masterMenu->_is_tornoff
-		   || masterMenu->_follow_transient);
-	     masterMenu = candidateMenu);
-
-	masterLocation = [[masterMenu window] frame].origin;
-	destinationPoint.x = masterLocation.x + vector.x;
-	destinationPoint.y = masterLocation.y + vector.y;
-
-	[masterMenu nestedSetFrameOrigin: destinationPoint];
+      NSMenu  *candidateMenu;
+      NSMenu  *masterMenu;
+      NSPoint  masterLocation;
+      NSPoint  destinationPoint;
+      
+      // Look for the "master" menu, i.e. the one to move from.
+      for (candidateMenu = masterMenu = self;
+           (candidateMenu = masterMenu->_superMenu)
+             && (!masterMenu->_is_tornoff
+                 || masterMenu->_transient);
+           masterMenu = candidateMenu);
+      
+      masterLocation = [[masterMenu window] frame].origin;
+      destinationPoint.x = masterLocation.x + vector.x;
+      destinationPoint.y = masterLocation.y + vector.y;
+      
+      [masterMenu nestedSetFrameOrigin: destinationPoint];
     }
-  else
-    _isPartlyOffScreen = NO;
 }
 
 - (BOOL)_ownedByPopUp
@@ -1325,181 +1515,18 @@ static NSNotificationCenter *nc;
       _popUpButtonCell = popUp;
       if (popUp != nil)
 	{
-	  [_titleView removeFromSuperviewWithoutNeedingDisplay];
 	  [_aWindow setLevel: NSPopUpMenuWindowLevel];
 	  [_bWindow setLevel: NSPopUpMenuWindowLevel];
 	}
-      // FIXME
     }
+  [self update];
+}
+
+- (NSString*) description
+{
+  return [NSString stringWithFormat: @"NSMenu: %@ (%@)",
+            _title, _transient ? @"Transient": @"Normal"];
 }
 
 @end
 
-
-@implementation NSMenuWindowTitleView
-
-- (BOOL) acceptsFirstMouse: (NSEvent *)theEvent
-{
-  return YES;
-} 
- 
-- (void) setMenu: (NSMenu*)aMenu
-{
-  menu = aMenu;
-}
-
-- (NSMenu*) menu
-{
-  return menu;
-}
-  
-- (void) drawRect: (NSRect)rect
-{
-  NSRect workRect = [self bounds];
-  NSRectEdge sides[] = {NSMinXEdge, NSMaxYEdge};
-  float grays[] = {NSDarkGray, NSDarkGray};
-  /* Cache the title attributes */
-  static NSDictionary *attr = nil;
-
-  // Draw the dark gray upper left lines.
-  workRect = NSDrawTiledRects(workRect, workRect, sides, grays, 2);
-  
-  // Draw the title box's button.
-  NSDrawButton(workRect, workRect);
-  
-  // Paint it Black!
-  workRect.origin.x += 1;
-  workRect.origin.y += 2;
-  workRect.size.height -= 3;
-  workRect.size.width -= 3;
-  [[NSColor windowFrameColor] set];
-  NSRectFill(workRect);
-
-  // Draw the title
-  if (attr == nil)
-    {
-      attr = [[NSDictionary alloc] 
-	       initWithObjectsAndKeys: 
-		 [NSFont boldSystemFontOfSize: 0], NSFontAttributeName,
-	       [NSColor windowFrameTextColor], NSForegroundColorAttributeName,
-	       nil];
-    }
-
-  // This gives the correct position
-  workRect.origin.x += 5;
-  workRect.size.width -= 5;
-  workRect.size.height -= 2;
-  [[menu title] drawInRect: workRect  withAttributes: attr];
-}
-
-- (void) mouseDown: (NSEvent*)theEvent
-{
-  NSString		*key;
-  NSPoint		lastLocation;
-  NSPoint		location;
-  unsigned		eventMask = NSLeftMouseUpMask | NSLeftMouseDraggedMask;
-  BOOL			done = NO;
-  NSDate		*theDistantFuture = [NSDate distantFuture];
-
-  lastLocation = [theEvent locationInWindow];
-   
-  if (![menu isTornOff] && [menu supermenu])
-    {
-      [menu _setTornOff: YES];
-    }
- 
-  while (!done)
-    {
-      theEvent = [NSApp nextEventMatchingMask: eventMask
-                                     untilDate: theDistantFuture
-                                        inMode: NSEventTrackingRunLoopMode
-                                       dequeue: YES];
-  
-      switch ([theEvent type])
-        {
-	case NSLeftMouseUp: 
-	  done = YES; 
-	  break;
-	case NSLeftMouseDragged:   
-	  location = [_window mouseLocationOutsideOfEventStream];
-	  if (NSEqualPoints(location, lastLocation) == NO)
-	    {
-	      NSPoint origin = [_window frame].origin;
-
-	      origin.x += (location.x - lastLocation.x);
-	      origin.y += (location.y - lastLocation.y);
-	      [menu nestedSetFrameOrigin: origin];
-	      [menu nestedCheckOffScreen];
-	    }
-	  break;
-         
-	default: 
-	  break;
-        }
-    }
-
-  /*
-   * Same current menu frame in defaults database.
-   */
-  key = [menu _locationKey];
-  if (key != nil)
-    {
-      NSUserDefaults		*defaults;
-      NSMutableDictionary	*menuLocations;
-      NSString			*locString;
-
-      defaults = [NSUserDefaults standardUserDefaults];
-      menuLocations = [[defaults objectForKey: NSMenuLocationsKey] mutableCopy];
-      if (menuLocations == nil)
-	{
-	  menuLocations = AUTORELEASE([[NSMutableDictionary alloc] initWithCapacity: 2]);
-	}
-      locString = [[menu window] stringWithSavedFrame];
-      [menuLocations setObject: locString forKey: key];
-      [defaults setObject: menuLocations forKey: NSMenuLocationsKey];
-      [defaults synchronize];
-    }
-}
-
-- (void) createButton
-{
-  // create the menu's close button
-  NSImage* closeImage = [NSImage imageNamed: @"common_Close"];
-  NSImage* closeHImage = [NSImage imageNamed: @"common_CloseH"];
-  NSSize imageSize = [closeImage size];
-  NSRect rect = { { _frame.size.width - imageSize.width - 4,
-		    (_frame.size.height - imageSize.height) / 2},
-		  { imageSize.height, imageSize.width } };
-
-  button = [[NSButton alloc] initWithFrame: rect];
-  [button setButtonType: NSMomentaryLight];
-  [button setImagePosition: NSImageOnly];
-  [button setImage: closeImage];
-  [button setAlternateImage: closeHImage];
-  [button setBordered: NO];
-  [button setTarget: menu];
-  [button setAction: @selector(_performMenuClose:)];
-  [button setAutoresizingMask: NSViewMinXMargin];
-  
-  [self setAutoresizingMask: NSViewMinXMargin | NSViewMinYMargin | NSViewMaxYMargin];
-}
-            
-- (void) releaseCloseButton
-{
-  [button removeFromSuperview];
-}
-  
-- (void) addCloseButton
-{
-  if (button == nil)
-    [self createButton];
-  [self addSubview: button];
-  [self setNeedsDisplay: YES];
-}
-
-- (void) rightMouseDown: (NSEvent*)theEvent
-{
-  // Dont show our menu
-}
-
-@end /* NSMenuWindowTitleView */
