@@ -1,6 +1,6 @@
 /** <title>NSOpenPanel</title> -*-objc-*-
 
-   <abstract>Standard open panel for opening files</abstract>
+   <abstract>Standard panel for opening files</abstract>
 
    Copyright (C) 1996, 1998, 1999, 2000 Free Software Foundation, Inc.
 
@@ -37,18 +37,19 @@
    59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
-#include <gnustep/gui/config.h>
-#include <string.h>
 #include <Foundation/NSString.h>
 #include <Foundation/NSArray.h>
 #include <Foundation/NSException.h>
-#include <AppKit/GMArchiver.h>
-#include <AppKit/IMLoading.h>
-#include <AppKit/NSApplication.h>
-#include <AppKit/NSBrowser.h>
-#include <AppKit/NSBrowserCell.h>
-#include <AppKit/NSMatrix.h>
-#include <AppKit/NSOpenPanel.h>
+#include <Foundation/NSFileManager.h>
+#include <Foundation/NSNotification.h>
+#include <Foundation/NSURL.h>
+#include "AppKit/NSApplication.h"
+#include "AppKit/NSBrowser.h"
+#include "AppKit/NSBrowserCell.h"
+#include "AppKit/NSButton.h"
+#include "AppKit/NSForm.h"
+#include "AppKit/NSMatrix.h"
+#include "AppKit/NSOpenPanel.h"
 
 static NSOpenPanel *_gs_gui_open_panel = nil;
 
@@ -59,6 +60,7 @@ static NSOpenPanel *_gs_gui_open_panel = nil;
 - (void) _selectTextInColumn: (int)column;
 - (void) _setupForDirectory: (NSString *)path file: (NSString *)filename;
 - (BOOL) _shouldShowExtension: (NSString *)extension isDir: (BOOL *)isDir;
+- (NSComparisonResult) _compareFilename: (NSString *)n1 with: (NSString *)n2;
 @end
 
 @implementation NSOpenPanel (_PrivateMethods)
@@ -77,15 +79,19 @@ static NSOpenPanel *_gs_gui_open_panel = nil;
 {
   BOOL found = YES;
 
-  if (_fileTypes)
+  if (_fileTypes != nil)
     {
       if ([_fileTypes containsObject: extension] == YES)
 	{
 	  if ([self treatsFilePackagesAsDirectories] == NO)
-	    *isDir = NO;
+	    {
+	      *isDir = NO;
+	    }
 	}
       else
-	found = NO;
+	{
+	  found = NO;
+	}
     }
 
   if (*isDir == YES || (found == YES && _canChooseFiles == YES))
@@ -98,10 +104,10 @@ static NSOpenPanel *_gs_gui_open_panel = nil;
 {
   NSMatrix *matrix;
 
-  if(column == -1)
+  if (column == -1)
     return;
 
-  matrix = [_browser matrixInColumn:column];
+  matrix = [_browser matrixInColumn: column];
 
   if ([_browser allowsMultipleSelection])
     {
@@ -109,41 +115,41 @@ static NSOpenPanel *_gs_gui_open_panel = nil;
 
       selectedCells = [matrix selectedCells];
 
-      if([selectedCells count] <= 1)
+      if ([selectedCells count] <= 1)
 	{
-	  if(_canChooseDirectories == NO ||
+	  if (_canChooseDirectories == NO ||
 	     [[matrix selectedCell] isLeaf] == YES)
-	    [super _selectTextInColumn:column];
+	    [super _selectTextInColumn: column];
 	  else
 	    {
-	      [self _selectCellName:[[_form cellAtIndex: 0] stringValue]];
-	      //	      [_form selectTextAtIndex:0];
-	      [_okButton setEnabled:YES];
+	      [self _selectCellName: [[_form cellAtIndex: 0] stringValue]];
+	      //	      [_form selectTextAtIndex: 0];
+	      [_okButton setEnabled: YES];
 	    }
 	}
       else
 	{
 	  [_form abortEditing];
 	  [[_form cellAtIndex: 0] setStringValue:@""];
-	  //	  [_form selectTextAtIndex:0];
-	  [_form setNeedsDisplay:YES];
-	  [_okButton setEnabled:YES];
+	  //	  [_form selectTextAtIndex: 0];
+	  [_form setNeedsDisplay: YES];
+	  [_okButton setEnabled: YES];
 	}
     }
   else
     {
-      if(_canChooseDirectories == NO || [[matrix selectedCell] isLeaf] == YES)
-	[super _selectTextInColumn:column];
+      if (_canChooseDirectories == NO || [[matrix selectedCell] isLeaf] == YES)
+	[super _selectTextInColumn: column];
       else
 	{
-	  if([[[_form cellAtIndex: 0] stringValue] length] > 0)
+	  if ([[[_form cellAtIndex: 0] stringValue] length] > 0)
 	    {
-	      [self _selectCellName:[[_form cellAtIndex: 0] stringValue]];
-	      //	      [_form selectTextAtIndex:0];
-	      [_form setNeedsDisplay:YES];
+	      [self _selectCellName: [[_form cellAtIndex: 0] stringValue]];
+	      //	      [_form selectTextAtIndex: 0];
+	      [_form setNeedsDisplay: YES];
 	    }
 
-	  [_okButton setEnabled:YES];
+	  [_okButton setEnabled: YES];
 	}
     }
 }
@@ -157,14 +163,14 @@ static NSOpenPanel *_gs_gui_open_panel = nil;
   NSRange             range;
   int                 i, titleLength, cellLength, numberOfCells;
 
-  matrix = [_browser matrixInColumn:[_browser lastColumn]];
-  if([matrix selectedCell])
+  matrix = [_browser matrixInColumn: [_browser lastColumn]];
+  if ([matrix selectedCell])
     return;
 
   titleLength = [title length];
-  if(!titleLength)
+  if (!titleLength)
     {
-      [_okButton setEnabled:NO];
+      [_okButton setEnabled: NO];
       return;
     }
 
@@ -174,24 +180,25 @@ static NSOpenPanel *_gs_gui_open_panel = nil;
   cells = [matrix cells];
   numberOfCells = [cells count];
 
-  for(i = 0; i < numberOfCells; i++)
+  for (i = 0; i < numberOfCells; i++)
     {
-      cellString = [[matrix cellAtRow:i column:0] stringValue];
+      cellString = [[matrix cellAtRow: i column: 0] stringValue];
 
       cellLength = [cellString length];
-      if(cellLength < titleLength)
+      if (cellLength < titleLength)
 	continue;
 
-      result = [cellString compare:title options:0 range:range];
+      result = [self _compareFilename: [cellString substringWithRange: range]
+                                 with: title];
 
-      if(result == NSOrderedSame)
+      if (result == NSOrderedSame)
 	{
-	  [matrix selectCellAtRow:i column:0];
-	  [matrix scrollCellToVisibleAtRow:i column:0];
-	  [_okButton setEnabled:YES];
+	  [matrix selectCellAtRow: i column: 0];
+	  [matrix scrollCellToVisibleAtRow: i column: 0];
+	  [_okButton setEnabled: YES];
 	  return;
 	}
-      else if(result == NSOrderedDescending)
+      else if (result == NSOrderedDescending)
 	break;
     }
 }
@@ -208,13 +215,13 @@ static NSOpenPanel *_gs_gui_open_panel = nil;
 
   if (filename == nil)
     filename = @"";
-  else if([filename isEqual: @""] == NO)
+  else if ([filename isEqual: @""] == NO)
     [_okButton setEnabled: YES];
 
   if (_canChooseDirectories == NO)
     {
-      if([_browser allowsMultipleSelection] == YES)
-	[_browser setAllowsBranchSelection:NO];
+      if ([_browser allowsMultipleSelection] == YES)
+	[_browser setAllowsBranchSelection: NO];
     }
   
   [super _setupForDirectory: path file: filename];
@@ -222,6 +229,28 @@ static NSOpenPanel *_gs_gui_open_panel = nil;
 
 @end
 
+/** 
+    <p>Implements a panel that allows the user to select a file or files.
+    NSOpenPanel is based on the NSSavePanel implementation and shares
+    a lot of similarities with it.
+    </p>
+    <p>
+    There is only one open panel per application and this panel is obtained
+    by calling the +openPanel class method. From here, you should set the
+    characteristics of the file selection mechanism using the
+    -setCanChooseFiles:, -setCanChooseDirectories: and 
+    -setAllowsMultipleSelection: methods. The default is YES except for
+    allowing multiple selection. When ready to show the panel, use the
+    -runModalForTypes:, or a similar method to show the panel in a modal
+    session. Other methods allow you to set the initial directory and
+    initially selected file. The method will return one of NSOKButton
+    or NSCancelButton depending on which button the user pressed.
+    </p>
+    <p>
+    Use the -filename or -filenames method to retrieve the name of the
+    file the user selected.
+    </p>
+ */
 @implementation NSOpenPanel
 
 /*
@@ -238,6 +267,7 @@ static NSOpenPanel *_gs_gui_open_panel = nil;
 /*
  * Accessing the NSOpenPanel shared instance
  */
+/** Returns the shared NSOpenPanel instance */
 + (NSOpenPanel *) openPanel
 {
   if (!_gs_gui_open_panel)
@@ -248,50 +278,68 @@ static NSOpenPanel *_gs_gui_open_panel = nil;
   return _gs_gui_open_panel;
 }
 
+- (void) dealloc
+{
+  TEST_RELEASE(_fileTypes);
+  [super dealloc];
+}
+
 - (id) init
 {
-  [super init];
-  _canChooseDirectories = YES;
-  _canChooseFiles = YES;
+  self = [super init];
+  if (self != nil)
+    {
+      _canChooseDirectories = YES;
+      _canChooseFiles = YES;
+    }
   return self;
 }
 
-// dealloc is the same as NSSavePanel
 
 /*
  * Filtering Files
  */
+/** Allows the user to select multiple files if flag is YES. */
 - (void) setAllowsMultipleSelection: (BOOL)flag
 {
   [_browser setAllowsMultipleSelection: flag];
 }
 
+/** Returns YES if the user is allowed to select multiple files. The
+    default behavior is not to allow mutiple selections. */
 - (BOOL) allowsMultipleSelection
 {
   return [_browser allowsMultipleSelection];
 }
 
+/** Allows the user to choose directories if flag is YES. */
 - (void) setCanChooseDirectories: (BOOL)flag
 {
   _canChooseDirectories = flag;
   [_browser setAllowsBranchSelection: flag];
 }
 
+/** Returns YES if the user is allowed to choose directories  The
+    default behavior is to allow choosing directories. */
 - (BOOL) canChooseDirectories
 {
   return _canChooseDirectories;
 }
 
+/** Allows the user to choose files if flag is YES */
 - (void) setCanChooseFiles: (BOOL)flag
 {
   _canChooseFiles = flag;
 }
 
+/** Returns YES if the user is allowed to choose files.  The
+    default behavior it to allow choosing files.  */
 - (BOOL) canChooseFiles
 {
   return _canChooseFiles;
 }
 
+/** Returns the absolute path of the file selected by the user.  */
 - (NSString*) filename
 {
   NSArray *ret;
@@ -304,9 +352,10 @@ static NSOpenPanel *_gs_gui_open_panel = nil;
     return nil;
 }
 
-/*
- * Querying the Chosen Files
- */
+/** Returns an array containing the absolute paths (as NSString
+ objects) of the selected files and directories.  If multiple
+ selections aren't allowed, the array contains a single name.
+*/
 - (NSArray *) filenames
 {
   if ([_browser allowsMultipleSelection])
@@ -350,6 +399,7 @@ static NSOpenPanel *_gs_gui_open_panel = nil;
     }
 }
 
+/** Returns an array of the selected files as URLs */
 - (NSArray *) URLs
 {
   NSMutableArray *ret = [NSMutableArray new];
@@ -367,51 +417,59 @@ static NSOpenPanel *_gs_gui_open_panel = nil;
 /*
  * Running the NSOpenPanel
  */
+/** Displays the open panel in a modal session, filtering for
+    files that have the specified types 
+*/
 - (int) runModalForTypes: (NSArray *)fileTypes
 {
   return [self runModalForDirectory: nil
-			       file: nil
+			       file: @""
 			      types: fileTypes];
 }
 
+/** Displays the open panel in a modal session, with the directory
+    path shown and file name (if any) selected. Files are filtered for the
+    specified types. If the directory is nil, then the directory shown in 
+    the open panel is the last directory selected.
+*/
+- (int) runModalForDirectory: (NSString *)path
+			file: (NSString *)name
+		       types: (NSArray *)fileTypes
+{
+  ASSIGN (_fileTypes, fileTypes);
+
+  return [self runModalForDirectory: path 
+			       file: name];  
+}
+
 - (int) runModalForDirectory: (NSString *)path
 			file: (NSString *)name
-			types: (NSArray *)fileTypes
+		       types: (NSArray *)fileTypes
+	    relativeToWindow: (NSWindow*)window
 {
   ASSIGN (_fileTypes, fileTypes);
 
   return [self runModalForDirectory: path 
-	       file: name];  
+			       file: name
+		   relativeToWindow: window];
 }
 
-- (int)runModalForDirectory:(NSString *)path
-		       file:(NSString *)name
-		      types:(NSArray *)fileTypes
-	   relativeToWindow:(NSWindow*)window
-{
-  ASSIGN (_fileTypes, fileTypes);
-
-  return [self runModalForDirectory: path 
-	       file: name
-	       relativeToWindow: window];
-}
-
-- (void)beginSheetForDirectory:(NSString *)path
-			  file:(NSString *)name
-			 types:(NSArray *)fileTypes
-		modalForWindow:(NSWindow *)docWindow
-		 modalDelegate:(id)delegate
-		didEndSelector:(SEL)didEndSelector
-		   contextInfo:(void *)contextInfo
+- (void) beginSheetForDirectory: (NSString *)path
+			   file: (NSString *)name
+			  types: (NSArray *)fileTypes
+		 modalForWindow: (NSWindow *)docWindow
+		  modalDelegate: (id)delegate
+		 didEndSelector: (SEL)didEndSelector
+		    contextInfo: (void *)contextInfo
 {
   ASSIGN (_fileTypes, fileTypes);
 
   [self beginSheetForDirectory: path
-	file: name
-	modalForWindow: docWindow
-	modalDelegate: delegate
-	didEndSelector: didEndSelector
-	contextInfo: contextInfo];
+			  file: name
+		modalForWindow: docWindow
+		 modalDelegate: delegate
+		didEndSelector: didEndSelector
+		   contextInfo: contextInfo];
 }
 
 - (void) ok: (id)sender
@@ -420,6 +478,7 @@ static NSOpenPanel *_gs_gui_open_panel = nil;
   NSBrowserCell *selectedCell = nil;
   NSArray       *selectedCells = nil;
   int            selectedColumn, lastColumn;
+  NSString	*tmp;
 
   selectedColumn = [_browser selectedColumn];
   lastColumn = [_browser lastColumn];
@@ -431,8 +490,7 @@ static NSOpenPanel *_gs_gui_open_panel = nil;
 	{
 	  selectedCells = [matrix selectedCells];
 
-	  if (selectedColumn == lastColumn &&
-	      [selectedCells count] == 1)
+	  if (selectedColumn == lastColumn && [selectedCells count] == 1)
 	    selectedCell = [selectedCells objectAtIndex: 0];
 	}
       else
@@ -460,21 +518,29 @@ static NSOpenPanel *_gs_gui_open_panel = nil;
 	}
     }
   else if (_canChooseDirectories == NO
-	   && (![_browser allowsMultipleSelection] || !selectedCells
-	       || selectedColumn != lastColumn || ![selectedCells count]))
+    && (![_browser allowsMultipleSelection] || !selectedCells
+	 || selectedColumn != lastColumn || ![selectedCells count]))
     {
       [_form selectTextAtIndex: 0];
       [_form setNeedsDisplay: YES];
       return;
     }
 
-  ASSIGN (_directory, [_browser pathToColumn:[_browser lastColumn]]);
+  ASSIGN (_directory, [_browser pathToColumn: [_browser lastColumn]]);
+
   if (selectedCell)
-    ASSIGN (_fullFileName, [_directory stringByAppendingPathComponent:
-					 [selectedCell stringValue]]);
+    tmp = [selectedCell stringValue];
   else
-    ASSIGN (_fullFileName, [_directory stringByAppendingPathComponent:
-					 [[_form cellAtIndex: 0] stringValue]]);
+    tmp = [[_form cellAtIndex: 0] stringValue];
+
+  if ([tmp isAbsolutePath] == YES)
+    {
+      ASSIGN (_fullFileName, tmp);
+    }
+  else
+    {
+      ASSIGN (_fullFileName, [_directory stringByAppendingPathComponent: tmp]);
+    }
 
   if (_delegateHasValidNameFilter)
     {
@@ -491,10 +557,22 @@ static NSOpenPanel *_gs_gui_open_panel = nil;
     }
 
   _OKButtonPressed = YES;
-  [NSApp stopModal];
+  [NSApp stopModalWithCode: NSOKButton];
+
   [self close];
 }
 
+- (BOOL) resolvesAliases
+{
+  // FIXME
+  return YES;
+}
+
+- (void) setResolvesAliases: (BOOL) flag
+{
+  // FIXME
+}
+
 /*
  * NSCoding protocol
  */
@@ -516,55 +594,6 @@ static NSOpenPanel *_gs_gui_open_panel = nil;
   return self;
 }
 @end
-//
-// NSOpenPanel browser delegate methods
-//
-@interface NSOpenPanel (_BrowserDelegate)
-- (BOOL) browser: (NSBrowser *)sender
-selectCellWithString: (NSString *)title
-	inColumn: (int)column;
-@end
-
-@implementation NSOpenPanel (_BrowserDelegate)
-- (BOOL) browser: (NSBrowser *)sender
-selectCellWithString: (NSString *)title
-	inColumn: (int)column
-{
-  NSMatrix *m;
-  NSArray *c;
-  BOOL isLeaf;
-
-  m = [_browser matrixInColumn: column];
-  c = [m selectedCells];
-  
-  if ([c count] == 1)
-    {
-      isLeaf = [[c objectAtIndex: 0] isLeaf];
-
-      if (_canChooseDirectories == NO)
-	{
-	  [_okButton setEnabled: isLeaf];
-	  return [super browser: sender
-			selectCellWithString: title
-			inColumn: column];
-	}
-      else // _canChooseDirectories
-	{
-	  BOOL ret;
-	  ret = [super browser: sender
-		       selectCellWithString: title
-		       inColumn: column];
-	  if (isLeaf == NO)
-	    ASSIGN (_fullFileName, _directory);
-	  return ret;
-	}
-    }
-  else // Multiple Selection, and it is not the first item of the selection
-    {
-      return YES;
-    }
-}
-@end
 
 //
 // NSForm delegate methods
@@ -584,17 +613,26 @@ selectCellWithString: (NSString *)title
   NSComparisonResult  result;
   NSRange             range;
 
-  matrix = [_browser matrixInColumn: [_browser lastColumn]];
   s = [[[aNotification userInfo] objectForKey: @"NSFieldEditor"] string];
 
+  /*
+   * If the user typed in an absolute path, display it.
+   */
+  if ([s isAbsolutePath] == YES)
+    {
+      [self setDirectory: s];
+    }
+
   sLength = [s length];
   range.location = 0;
   range.length = sLength;
 
-  if(sLength == 0)
+  matrix = [_browser matrixInColumn: [_browser lastColumn]];
+
+  if (sLength == 0)
     {
       [matrix deselectAllCells];
-      if(_canChooseDirectories == NO)
+      if (_canChooseDirectories == NO)
 	[_okButton setEnabled: NO];
       return;
     }
@@ -604,20 +642,20 @@ selectCellWithString: (NSString *)title
   selectedRow = [matrix selectedRow];
   cells = [matrix cells];
 
-  if(selectedString)
+  if (selectedString)
     {
       cellLength = [selectedString length];
 
-      if(cellLength < sLength)
+      if (cellLength < sLength)
 	range.length = cellLength;
 
       result = [selectedString compare: s options: 0 range: range];
 
-      if(result == NSOrderedSame)
+      if (result == NSOrderedSame)
 	return;
-      else if(result == NSOrderedAscending)
+      else if (result == NSOrderedAscending)
 	result = NSOrderedDescending;
-      else if(result == NSOrderedDescending)
+      else if (result == NSOrderedDescending)
 	result = NSOrderedAscending;
 
       range.length = sLength;
@@ -625,21 +663,21 @@ selectCellWithString: (NSString *)title
   else
     result = NSOrderedDescending;
 
-  if(result == NSOrderedDescending)
+  if (result == NSOrderedDescending)
     {
       int numberOfCells = [cells count];
 
-      for(i = selectedRow+1; i < numberOfCells; i++)
+      for (i = selectedRow+1; i < numberOfCells; i++)
 	{
 	  selectedString = [[matrix cellAtRow: i column: 0] stringValue];
 
 	  cellLength = [selectedString length];
-	  if(cellLength < sLength)
+	  if (cellLength < sLength)
 	    continue;
 
 	  result = [selectedString compare: s options: 0 range: range];
 
-	  if(result == NSOrderedSame)
+	  if (result == NSOrderedSame)
 	    {
 	      [matrix deselectAllCells];
 	      [matrix selectCellAtRow: i column: 0];
@@ -651,17 +689,17 @@ selectCellWithString: (NSString *)title
     }
   else
     {
-      for(i = selectedRow; i >= 0; --i)
+      for (i = selectedRow; i >= 0; --i)
 	{
 	  selectedString = [[matrix cellAtRow: i column: 0] stringValue];
 
 	  cellLength = [selectedString length];
-	  if(cellLength < sLength)
+	  if (cellLength < sLength)
 	    continue;
 
 	  result = [selectedString compare: s options: 0 range: range];
 
-	  if(result == NSOrderedSame)
+	  if (result == NSOrderedSame)
 	    {
 	      [matrix deselectAllCells];
 	      [matrix selectCellAtRow: i column: 0];
