@@ -28,12 +28,12 @@
 */ 
 
 #include <Foundation/Foundation.h>
-#include <AppKit/NSPopUpButton.h>
-#include <AppKit/NSPopUpButtonCell.h>
-#include <AppKit/NSApplication.h>
-#include <AppKit/NSMenu.h>
-#include <AppKit/NSMenuItem.h>
-#include <AppKit/NSMenuView.h>
+#include "AppKit/NSPopUpButton.h"
+#include "AppKit/NSPopUpButtonCell.h"
+#include "AppKit/NSApplication.h"
+#include "AppKit/NSMenu.h"
+#include "AppKit/NSMenuItem.h"
+#include "AppKit/NSMenuView.h"
 
 /*
  * class variables
@@ -91,6 +91,18 @@ Class _nspopupbuttonCellClass = 0;
   return self;
 }
 
+
+/*
+In NSView, -menuForEvent: returns [self menu] as the context menu of the
+view. Since our -menu returns the menu for our pop-up, we need to override
+this to return nil to indicate that we have no context menu.
+*/
+- (NSMenu *)menuForEvent:(NSEvent *)theEvent
+{
+  return nil;
+}
+
+
 - (void) setMenu: (NSMenu*)menu
 {
   [_cell setMenu: menu];
@@ -272,6 +284,7 @@ Class _nspopupbuttonCellClass = 0;
 - (void) setTitle: (NSString*)aString
 {
   [_cell setTitle: aString];
+  [self synchronizeTitleAndSelectedItem];
 }
 
 - (void) synchronizeTitleAndSelectedItem
@@ -287,18 +300,30 @@ Class _nspopupbuttonCellClass = 0;
   return [super resignFirstResponder];
 }
 
-- (void) resignKeyWindow
-{
-  [_cell dismissPopUp];
-  [super resignKeyWindow];
-}
-
 - (BOOL) performKeyEquivalent: (NSEvent*)theEvent
 {
-  NSMenu *m = [self menu];
+  NSMenu     *m = [self menu];
+  NSMenuItem *oldSelectedItem = [_cell selectedItem];
 
   if (m != nil)
-    return [m performKeyEquivalent: theEvent];
+    {
+      if ([m performKeyEquivalent: theEvent])
+	{
+	  // pullsDown does not change selected item
+	  if ([_cell pullsDown])
+	    {
+	      [self selectItem: oldSelectedItem];
+	    }
+	  else
+	    {
+	      /* If the key equivalent was performed, redisplay ourselves
+	       * to account for potential changes in the selected item.
+	       */
+	      [self setNeedsDisplay: YES];
+	    }
+	  return YES;
+	}
+    }
   return NO;
 }
 
@@ -308,13 +333,21 @@ Class _nspopupbuttonCellClass = 0;
   NSWindow   *menuWindow = [mr window];
   NSEvent    *e;
   NSPoint    p;
+  int        lastSelectedItem = [_cell indexOfSelectedItem];
+  int        highlightedItemIndex;
 
   if ([self isEnabled] == NO)
     return;
 
+  if ([[_cell menu] numberOfItems] == 0)
+    {
+      NSBeep ();
+      return;
+    }
+
   // Attach the popUp
   [_cell attachPopUpWithFrame: _bounds
-	 inView: self];
+	               inView: self];
   
   p = [_window convertBaseToScreen: [theEvent locationInWindow]];
   p = [menuWindow convertScreenToBase: p];
@@ -330,7 +363,27 @@ Class _nspopupbuttonCellClass = 0;
 	       eventNumber: [theEvent eventNumber]
 	       clickCount: [theEvent clickCount] 
 	       pressure: [theEvent pressure]];
-  [menuWindow sendEvent: e];
+  [NSApp sendEvent: e];
+
+  // Get highlighted item index from _cell because NSMenuView:
+  // - tells to NSPopUpButtonCell about current selected item index;
+  // - sets own selected item index to -1;
+  //
+  // So, at this point [mr highlightedItemIndex] always = -1
+  highlightedItemIndex = [_cell indexOfSelectedItem];
+
+  // Selection remains unchanged if selected item is disabled
+  // or mouse left menu (highlightedItemIndex == -1).
+  if ( highlightedItemIndex < 0
+       || highlightedItemIndex == lastSelectedItem
+       || [[self itemAtIndex: highlightedItemIndex] isEnabled] == NO)
+    {
+      [mr setHighlightedItemIndex: lastSelectedItem];
+    }
+  else
+    {
+      [mr setHighlightedItemIndex: highlightedItemIndex];
+    }
 
   // Dismiss the popUp
   [_cell dismissPopUp];
@@ -374,7 +427,13 @@ Class _nspopupbuttonCellClass = 0;
 	  {
 	    int selectedIndex;
 	    NSMenuView *menuView;
-	    NSNotificationCenter *nc = [NSNotificationCenter defaultCenter];
+
+	    // Beep, as on OS, and then return.
+	    if ([[_cell menu] numberOfItems] == 0)
+	      {
+		NSBeep();
+		return;
+	      }
 
 	    menuView = [[_cell menu] menuRepresentation];
 	    if ([[menuView window] isVisible] == NO)
@@ -388,22 +447,9 @@ Class _nspopupbuttonCellClass = 0;
 		selectedIndex = [self indexOfSelectedItem];
 		if (selectedIndex > -1)
 		  [menuView setHighlightedItemIndex: selectedIndex];
-
-		/* This covers an obscure case where the user subsequently
-		   uses the mouse to select an item. We'll never know
-		   this was done (and thus cannot dismiss the popUp) without
-		   getting this notification */
-		[nc addObserver: self
-		       selector: @selector(_handleNotification:)
-		           name: NSMenuDidSendActionNotification
-		         object: [_cell menu]];
-
 	      }
 	    else
 	      {
-		[nc removeObserver: self
-		              name: NSMenuDidSendActionNotification
-		            object: [_cell menu]];
 		selectedIndex = [menuView highlightedItemIndex];
 		[[_cell menu] performActionForItemAtIndex: selectedIndex];
 
@@ -461,17 +507,4 @@ Class _nspopupbuttonCellClass = 0;
   [super keyDown: theEvent];
 }
 
-/*
- * NSCoding protocol
- */
-- (void) encodeWithCoder: (NSCoder*)aCoder
-{
-  [super encodeWithCoder: aCoder];
-}
-
-- (id) initWithCoder: (NSCoder*)aDecoder
-{
-  return [super initWithCoder: aDecoder];
-}
-
 @end
