@@ -383,24 +383,32 @@ static BOOL _gs_display_reading_progress = NO;
   selectedCell = [matrix selectedCell];
   isLeaf = [selectedCell isLeaf];
 
+  if (_delegateHasSelectionDidChange) 
+    {
+      [_delegate panelSelectionDidChange: self];
+    }
+
   if (isLeaf)
     {
       [[_form cellAtIndex: 0] setStringValue: [selectedCell stringValue]];
       //      [_form selectTextAtIndex:0];
-      [_form setNeedsDisplay:YES];
-      [_okButton setEnabled:YES];
+      [_okButton setEnabled: YES];
     }
   else
     {
+      if (_delegateHasDirectoryDidChange)
+        {
+	  [_delegate panel: self directoryDidChange: [_browser pathToColumn: column]];
+	}
+
       if ([[[_form cellAtIndex: 0] stringValue] length] > 0)
 	{
-	  [_okButton setEnabled:YES];
-	  [self _selectCellName:[[_form cellAtIndex: 0] stringValue]];
+	  [_okButton setEnabled: YES];
+	  [self _selectCellName: [[_form cellAtIndex: 0] stringValue]];
 	  //	  [_form selectTextAtIndex:0];
-	  [_form setNeedsDisplay:YES];
 	}
       else
-	[_okButton setEnabled:NO];
+	[_okButton setEnabled: NO];
     }
 }
 
@@ -623,7 +631,9 @@ selectCellWithString: (NSString*)title
 -(id) init
 {
   [self _initWithoutGModel];
-  
+
+/*
+ * All these are set automatically  
   _directory = nil;
   _fullFileName = nil;
   _requiredFileType = nil;
@@ -633,6 +643,9 @@ selectCellWithString: (NSString*)title
   _delegateHasCompareFilter = NO;
   _delegateHasShowFilenameFilter = NO;
   _delegateHasValidNameFilter = NO;
+  _delegateHasDirectoryDidChange = NO;
+  _delegateHasSelectionDidChange = NO;
+*/
 
   [self _getOriginalSize];
   return self;
@@ -796,6 +809,29 @@ selectCellWithString: (NSString*)title
   return _accessoryView;
 }
 
+- (void) setNameFieldLabel: (NSString *)label
+{
+  // FIXME
+}
+
+- (NSString *) nameFieldLabel
+{
+  // FIXME
+  return _(@"Save As");
+}
+
+- (void) setMessage: (NSString *)message
+{
+  // FIXME
+}
+
+- (NSString *) message
+{
+  // FIXME
+  return nil;
+}
+
+
 /**
  * Sets the current path name in the Save panel's browser. 
  * The path argument must be an absolute path name.
@@ -833,6 +869,27 @@ selectCellWithString: (NSString*)title
   return _requiredFileType;
 }
 
+- (void) setAllowedFileTypes: (NSArray *)types
+{
+  // FIXME
+}
+
+- (void) setAllowsOtherFileTypes: (BOOL)flag
+{
+  _allowsOtherFileTypes = flag;
+}
+
+- (NSArray *) allowedFileTypes
+{
+  // FIXME
+  return nil;
+}
+
+- (BOOL) allowsOtherFileTypes
+{
+  return _allowsOtherFileTypes;
+}
+
 /** Returns YES if file packages are shown as directories. The default
     is NO.  */
 - (BOOL) treatsFilePackagesAsDirectories
@@ -867,6 +924,16 @@ selectCellWithString: (NSString*)title
   [_browser validateVisibleColumns];
 }
 
+- (void) setCanCreateDirectories: (BOOL)flag
+{
+  _canCreateDirectories = flag;
+}
+
+- (BOOL) canCreateDirectories
+{
+  return _canCreateDirectories;
+}
+
 /**
  * Shows the save panel for the user. This method invokes
  * -runModalForDirectory:file: with empty strings for the filename.
@@ -1124,23 +1191,36 @@ selectCellWithString: (NSString*)title
   else
     _delegateHasUserEnteredFilename = NO;
 
+  if ([aDelegate respondsToSelector: @selector(panel:directoryDidChange:)])
+     _delegateHasDirectoryDidChange = YES;
+  else 
+    _delegateHasDirectoryDidChange = NO;      
+
+  if ([aDelegate respondsToSelector: @selector(panelSelectionDidChange:)])
+     _delegateHasSelectionDidChange = YES;
+  else 
+    _delegateHasSelectionDidChange = NO;      
+
   [super setDelegate: aDelegate];
 }
 
 - (void) setCanSelectHiddenExtension: (BOOL) flag
 {
-  // FIXME
+  _canSelectHiddenExtension = flag;
+}
+- (BOOL) canSelectHiddenExtension
+{
+  return _canSelectHiddenExtension;
 }
 
 - (BOOL) isExtensionHidden
 {
-  // FIXME
-  return NO;
+  return _isExtensionHidden;
 }
 
 - (void) setExtensionHidden: (BOOL) flag
 {
-  // FIXME
+  _isExtensionHidden = flag;
 }
 
 - (BOOL) isExpanded
@@ -1152,17 +1232,20 @@ selectCellWithString: (NSString*)title
 //
 // NSCoding protocol
 //
-- (id) initWithCoder: (NSCoder*)aCoder
+- (id) initWithCoder: (NSCoder*)aDecoder
 {
-  // TODO
+  self = [super initWithCoder: aDecoder];
 
-  return nil;
+  // TODO
+  return self;
 }
 
 - (void) encodeWithCoder: (NSCoder*)aCoder
 {
+  [super encodeWithCoder: aCoder]; 
   // TODO
 }
+
 @end
 
 //
