@@ -275,32 +275,36 @@ static NSColor *scrollBarColor = nil;
     return;
 
   upCell = [NSButtonCell new];
-  [upCell setHighlightsBy: NSChangeBackgroundCellMask];
+  [upCell setHighlightsBy: NSChangeBackgroundCellMask | NSContentsCellMask];
   [upCell setImage: [NSImage imageNamed: @"common_ArrowUp"]];
+  [upCell setAlternateImage: [NSImage imageNamed: @"common_ArrowUpH"]];
   [upCell setImagePosition: NSImageOnly];
   [upCell setContinuous: YES];
   [upCell sendActionOn: (NSLeftMouseDownMask | NSPeriodicMask)];
   [upCell setPeriodicDelay: 0.3 interval: 0.03];
 
   downCell = [NSButtonCell new];
-  [downCell setHighlightsBy: NSChangeBackgroundCellMask];
+  [downCell setHighlightsBy: NSChangeBackgroundCellMask | NSContentsCellMask];
   [downCell setImage: [NSImage imageNamed: @"common_ArrowDown"]];
+  [downCell setAlternateImage: [NSImage imageNamed: @"common_ArrowDownH"]];
   [downCell setImagePosition: NSImageOnly];
   [downCell setContinuous: YES];
   [downCell sendActionOn: (NSLeftMouseDownMask | NSPeriodicMask)];
   [downCell setPeriodicDelay: 0.3 interval: 0.03];
 
   leftCell = [NSButtonCell new];
-  [leftCell setHighlightsBy: NSChangeBackgroundCellMask];
+  [leftCell setHighlightsBy: NSChangeBackgroundCellMask | NSContentsCellMask];
   [leftCell setImage: [NSImage imageNamed: @"common_ArrowLeft"]];
+  [leftCell setAlternateImage: [NSImage imageNamed: @"common_ArrowLeftH"]];
   [leftCell setImagePosition: NSImageOnly];
   [leftCell setContinuous: YES];
   [leftCell sendActionOn: (NSLeftMouseDownMask | NSPeriodicMask)];
   [leftCell setPeriodicDelay: 0.3 interval: 0.03];
 
   rightCell = [NSButtonCell new];
-  [rightCell setHighlightsBy: NSChangeBackgroundCellMask];
+  [rightCell setHighlightsBy: NSChangeBackgroundCellMask | NSContentsCellMask];
   [rightCell setImage: [NSImage imageNamed: @"common_ArrowRight"]];
+  [rightCell setAlternateImage: [NSImage imageNamed: @"common_ArrowRightH"]];
   [rightCell setImagePosition: NSImageOnly];
   [rightCell setContinuous: YES];
   [rightCell sendActionOn: (NSLeftMouseDownMask | NSPeriodicMask)];
@@ -369,7 +373,7 @@ static NSColor *scrollBarColor = nil;
     {
       return;
     }
-  
+
   _isEnabled = flag;
   _cacheValid = NO;
   [self setNeedsDisplay: YES];
@@ -417,23 +421,44 @@ static NSColor *scrollBarColor = nil;
       /* Most likely our trackKnob method initiated this via NSScrollView */
       return;
     }
+
   if (ratio < 0)
     {
-      _knobProportion = 0;
+      _pendingKnobProportion = 0;
     }
   else if (ratio > 1)
     {
-      _knobProportion = 1;
+      _pendingKnobProportion = 1;
     }
   else
     {
-      _knobProportion = ratio;
+      _pendingKnobProportion = ratio;
+    }
+    
+  if (_hitPart == NSScrollerNoPart)
+    {
+      _knobProportion = _pendingKnobProportion;
+      _pendingKnobProportion = 0;
     }
 
-  /* Make sure we mark ourselves as needing redisplay.  */
-  _floatValue = -1;
+  // Handle the case when parts should disappear
+  if (_knobProportion == 1)
+    {
+      [self setEnabled: NO];
+    }
+  else
+    {
+      [self setEnabled: YES];
+    }
   
-  [self setFloatValue: aFloat];
+  // Don't set float value if knob is being dragged
+  if (_hitPart != NSScrollerKnobSlot && _hitPart != NSScrollerKnob)
+    {
+      /* Make sure we mark ourselves as needing redisplay.  */
+      _floatValue = -1;
+
+      [self setFloatValue: aFloat];
+    }
 }
 
 - (void) setFrame: (NSRect)frameRect
@@ -567,23 +592,6 @@ static NSColor *scrollBarColor = nil;
     {
       case NSScrollerIncrementLine:
       case NSScrollerDecrementLine:
-	/*
-	 * A hit on a scroller button should be a page movement
-	 * if the alt key is pressed.
-	 */
-	if ([theEvent modifierFlags] & NSAlternateKeyMask)
-	  {
-	    if (_hitPart == NSScrollerIncrementLine)
-	      {
-		_hitPart = NSScrollerIncrementPage;
-	      }
-	    else
-	      {
-		_hitPart = NSScrollerDecrementPage;
-	      }
-	  }
-	/* Fall through to next case */
-
       case NSScrollerIncrementPage:
       case NSScrollerDecrementPage:
 	[self trackScrollButtons: theEvent];
@@ -612,6 +620,14 @@ static NSColor *scrollBarColor = nil;
     }
 
   _hitPart = NSScrollerNoPart;
+  if (_pendingKnobProportion)
+    {
+      [self setFloatValue: _floatValue knobProportion: _pendingKnobProportion];
+    }
+  else
+    {
+      [self setNeedsDisplay:YES];
+    }
 }
 
 - (void) trackKnob: (NSEvent*)theEvent
@@ -802,7 +818,7 @@ static NSColor *scrollBarColor = nil;
   static NSRect rectForPartIncrementLine;
   static NSRect rectForPartDecrementLine;
   static NSRect rectForPartKnobSlot;
-  
+
   if (_cacheValid == NO)
     {
       rectForPartIncrementLine = [self rectForPart: NSScrollerIncrementLine];
@@ -867,7 +883,7 @@ static NSColor *scrollBarColor = nil;
       rect = [self rectForPart: NSScrollerKnobSlot];
     }
 
-    [scrollBarColor set];
+  [scrollBarColor set];
   NSRectFill (rect);
 }
 
@@ -940,7 +956,9 @@ static NSColor *scrollBarColor = nil;
 
 	  if (usableParts == NSNoScrollerParts
 	    || usableParts == NSOnlyScrollerArrows)
-	    return NSZeroRect;
+	    {
+	      return NSZeroRect;
+	    }
 
 	  /* calc the slot Height */
 	  slotHeight = height - (_arrowsPosition == NSScrollerArrowsNone
@@ -952,11 +970,12 @@ static NSColor *scrollBarColor = nil;
 
 	  /* calc knob's position */
 	  knobPosition = _floatValue * (slotHeight - knobHeight);
-	  knobPosition = (float)floor(knobPosition);
+	  knobPosition = floor(knobPosition);
+
 
 	  /* calc actual position */
-	  y += knobPosition + (_arrowsPosition == NSScrollerArrowsMaxEnd
-			       || _arrowsPosition == NSScrollerArrowsNone 
+	  y += knobPosition + ((_arrowsPosition == NSScrollerArrowsMaxEnd
+			       || _arrowsPosition == NSScrollerArrowsNone)
 			       ?  0 : buttonsSize);
 	  height = knobHeight;
 	  width = buttonsWidth;
