@@ -23,15 +23,15 @@
    59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */ 
 
-#include <gnustep/gui/config.h>
-#include <AppKit/NSGraphicsContext.h>
-#include <AppKit/NSColor.h>
-#include <AppKit/DPSOperators.h>
-#include <AppKit/PSOperators.h>
-#include <AppKit/NSFont.h>
-#include <AppKit/NSGraphics.h>
-#include <AppKit/NSStepperCell.h>
-#include <AppKit/NSText.h>
+#include "config.h"
+#include "AppKit/NSGraphicsContext.h"
+#include "AppKit/NSColor.h"
+#include "AppKit/DPSOperators.h"
+#include "AppKit/PSOperators.h"
+#include "AppKit/NSFont.h"
+#include "AppKit/NSGraphics.h"
+#include "AppKit/NSStepperCell.h"
+#include "AppKit/NSText.h"
 
 @implementation NSStepperCell
 + (void) initialize
@@ -65,9 +65,9 @@
   return _maxValue;
 }
 
-- (void) setMaxValue: (double)newMaxValue
+- (void) setMaxValue: (double)maxValue
 {
-  _maxValue = newMaxValue;
+  _maxValue = maxValue;
 }
 
 - (double) minValue
@@ -75,9 +75,9 @@
   return _minValue;
 }
 
-- (void) setMinValue: (double)newMinValue
+- (void) setMinValue: (double)minValue
 {
-  _minValue = newMinValue;
+  _minValue = minValue;
 }
 
 - (double) increment
@@ -85,9 +85,9 @@
   return _increment;
 }
 
-- (void) setIncrement: (double)newIncrement
+- (void) setIncrement: (double)increment
 {
-  _increment = newIncrement;
+  _increment = increment;
 }
 
 
@@ -97,9 +97,9 @@
   return _autorepeat;
 }
 
-- (void)setAutorepeat: (BOOL)newAutorepeat
+- (void)setAutorepeat: (BOOL)autorepeat
 {
-  _autorepeat = newAutorepeat;
+  _autorepeat = autorepeat;
 }
 
 - (BOOL)valueWraps
@@ -107,9 +107,9 @@
   return _valueWraps;
 }
 
-- (void)setValueWraps: (BOOL)newValueWraps
+- (void)setValueWraps: (BOOL)valueWraps
 {
-  _valueWraps = newValueWraps;
+  _valueWraps = valueWraps;
 }
 
 - (void) dealloc
@@ -124,7 +124,7 @@
   return c;
 }
 
-inline void DrawUpButton(NSRect aRect)
+static inline void DrawUpButton(NSRect aRect)
 {
   NSRectEdge up_sides[] = {NSMinXEdge, NSMaxYEdge, 
 			   NSMaxXEdge, NSMinYEdge};
@@ -154,7 +154,7 @@ inline void DrawUpButton(NSRect aRect)
   PSstroke();
 }
 
-inline void HighlightUpButton(NSRect aRect)
+static inline void HighlightUpButton(NSRect aRect)
 {
   NSRectEdge up_sides[] = {NSMinXEdge, NSMaxYEdge, 
 			   NSMaxXEdge, NSMinYEdge};
@@ -184,7 +184,7 @@ inline void HighlightUpButton(NSRect aRect)
   PSstroke();
 }
 
-inline void DrawDownButton(NSRect aRect)
+static inline void DrawDownButton(NSRect aRect)
 {
   NSRectEdge up_sides[] = {NSMinXEdge, NSMaxYEdge, 
 			   NSMaxXEdge, NSMinYEdge};
@@ -215,7 +215,7 @@ inline void DrawDownButton(NSRect aRect)
   PSstroke();
 }
 
-inline void HighlightDownButton(NSRect aRect)
+static inline void HighlightDownButton(NSRect aRect)
 {
   NSRectEdge up_sides[] = {NSMinXEdge, NSMaxYEdge, 
 			   NSMaxXEdge, NSMinYEdge};
@@ -253,7 +253,6 @@ inline void HighlightDownButton(NSRect aRect)
   NSRect downRect;
   NSRect twoButtons;
   NSGraphicsContext *ctxt;
-  [controlView lockFocus];
   ctxt = GSCurrentContext();
 
   {
@@ -283,8 +282,6 @@ inline void HighlightDownButton(NSRect aRect)
 		       up_sides, grays, 2);
     }
   }
-
-  [controlView unlockFocus];
 }
 
 - (void) highlight: (BOOL) highlight
@@ -295,7 +292,6 @@ inline void HighlightDownButton(NSRect aRect)
   NSRect upRect;
   NSRect downRect;
   NSGraphicsContext *ctxt;
-  [controlView lockFocus];
   ctxt = GSCurrentContext();
   {
     upRect = [self upButtonRectWithFrame: frame];
@@ -317,7 +313,6 @@ inline void HighlightDownButton(NSRect aRect)
 	  DrawDownButton(downRect);
       }
   }
-  [controlView unlockFocus];
 }
 
 - (NSRect) upButtonRectWithFrame: (NSRect) frame
