@@ -23,13 +23,13 @@
    59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
-#include <AppKit/NSColor.h>
-#include <AppKit/NSFont.h>
-#include <AppKit/NSImage.h>
-#include <AppKit/NSTabViewItem.h>
-#include <AppKit/PSOperators.h>
-#include <AppKit/NSGraphics.h>
-#include <AppKit/NSAttributedString.h>
+#include "AppKit/NSColor.h"
+#include "AppKit/NSFont.h"
+#include "AppKit/NSImage.h"
+#include "AppKit/NSTabViewItem.h"
+#include "AppKit/PSOperators.h"
+#include "AppKit/NSGraphics.h"
+#include "AppKit/NSAttributedString.h"
 
 @implementation NSTabViewItem
 - (id) initWithIdentifier:(id)identifier
@@ -155,7 +155,7 @@
 	   inRect:(NSRect)tabRect
 {
   NSGraphicsContext     *ctxt = GSCurrentContext();
-//  NSRect lRect;
+  NSRect lRect;
   NSRect fRect;
   NSDictionary *attr;
   NSString *string;
@@ -197,13 +197,9 @@
 			       [NSColor blackColor], NSForegroundColorAttributeName,
 			       nil];
 
-  // For some unclear reason, somehow connected with clipping,
-  // drawInRect does not work here. But drawAtPoint works fine.
-  [string drawAtPoint: NSMakePoint(tabRect.origin.x, NSMaxY(tabRect)) 
-	  withAttributes: attr];
-//  lRect = tabRect;
-//  lRect.origin.y += 3;
-//  [_label drawInRect: lRect withAttributes: attr];
+  lRect = tabRect;
+  lRect.origin.y += 3;
+  [string drawInRect: lRect withAttributes: attr];
   RELEASE(attr);
 
   DPSgrestore(ctxt);
@@ -224,13 +220,26 @@
 
 - (id) initWithCoder: (NSCoder*)aDecoder
 {
-  [aDecoder decodeValueOfObjCType: @encode(id) at: &_ident];
-  [aDecoder decodeValueOfObjCType: @encode(id) at: &_label];
-  [aDecoder decodeValueOfObjCType: @encode(id) at: &_view];
-  [aDecoder decodeValueOfObjCType: @encode(id) at: &_color];
-  [aDecoder decodeValueOfObjCType: @encode(NSTabState) at:&_state];
-  [aDecoder decodeValueOfObjCType: @encode(id) at: &_first_responder];
-  [aDecoder decodeValueOfObjCType: @encode(id) at: &_tabview];
+  if ([aDecoder allowsKeyedCoding])
+    {
+      id identifier = [aDecoder decodeObjectForKey: @"NSIdentifier"];
+
+      self = [self initWithIdentifier: identifier];
+      [self setLabel: [aDecoder decodeObjectForKey: @"NSLabel"]];
+      [self setView: [aDecoder decodeObjectForKey: @"NSView"]];
+      [self setColor: [aDecoder decodeObjectForKey: @"NSColor"]];
+      [self _setTabView: [aDecoder decodeObjectForKey: @"NSTabView"]];
+    }
+  else
+    {
+      [aDecoder decodeValueOfObjCType: @encode(id) at: &_ident];
+      [aDecoder decodeValueOfObjCType: @encode(id) at: &_label];
+      [aDecoder decodeValueOfObjCType: @encode(id) at: &_view];
+      [aDecoder decodeValueOfObjCType: @encode(id) at: &_color];
+      [aDecoder decodeValueOfObjCType: @encode(NSTabState) at:&_state];
+      [aDecoder decodeValueOfObjCType: @encode(id) at: &_first_responder];
+      [aDecoder decodeValueOfObjCType: @encode(id) at: &_tabview];
+    }
 
   return self;
 }
