@@ -29,15 +29,18 @@
 */
 
 #include <Foundation/NSArray.h>
-#include <AppKit/NSTableHeaderCell.h>
-#include <AppKit/NSTableHeaderView.h>
-#include <AppKit/NSTableColumn.h>
-#include <AppKit/NSTableView.h>
-#include <AppKit/NSEvent.h>
-#include <AppKit/NSApplication.h>
-#include <AppKit/NSColor.h>
-#include <AppKit/NSScrollView.h>
-#include <AppKit/DPSOperators.h>
+#include <Foundation/NSDebug.h>
+#include <Foundation/NSRunLoop.h>
+#include "AppKit/NSTableHeaderCell.h"
+#include "AppKit/NSTableHeaderView.h"
+#include "AppKit/NSTableColumn.h"
+#include "AppKit/NSTableView.h"
+#include "AppKit/NSWindow.h"
+#include "AppKit/NSEvent.h"
+#include "AppKit/NSApplication.h"
+#include "AppKit/NSColor.h"
+#include "AppKit/NSScrollView.h"
+#include "AppKit/NSGraphics.h"
 
 @interface NSTableView (GNUstepPrivate)
 - (void) _userResizedTableColumn: (int)index
@@ -183,20 +186,11 @@
       if ((column == highlightedTableColumn)
 	  || [_tableView isColumnSelected: i])
 	{
-	  /* FIXME/TODO - cell can be any subclass of NSCell
-	     (specially a custom subclass), which might not implement
-	     setBackgroundColor: (or might not want us to mess up its
-	     custom drawing by hardcoding a background color here) -
-	     so we should not call it.  Instead, NSTableHeaderCell
-	     should override setHighlighted: to change the background
-	     color.  */
 	  [cell setHighlighted: YES];
-	  [cell setBackgroundColor: [NSColor controlColor]];
 	}
       else
 	{
 	  [cell setHighlighted: NO];
-	  [cell setBackgroundColor: [NSColor controlShadowColor]];
 	}
       [cell drawWithFrame: drawingRect
 			   inView: self];
@@ -212,12 +206,10 @@
 	  || [_tableView isColumnSelected: lastColumnToDraw])
 	{
 	  [cell setHighlighted: YES];
-	  [cell setBackgroundColor: [NSColor controlColor]];
 	}
       else
 	{
 	  [cell setHighlighted: NO];
-	  [cell setBackgroundColor: [NSColor controlShadowColor]];
 	}
       [cell drawWithFrame: drawingRect
 			   inView: self];
@@ -233,12 +225,10 @@
 	  || [_tableView isColumnSelected: lastColumnToDraw])
 	{
 	  [cell setHighlighted: YES];
-	  [cell setBackgroundColor: [NSColor controlColor]];
 	}
       else
 	{
 	  [cell setHighlighted: NO];
-	  [cell setBackgroundColor: [NSColor controlShadowColor]];
 	}
       [cell drawWithFrame: drawingRect
 			   inView: self];
@@ -247,14 +237,10 @@
 
 
   {
-    NSGraphicsContext *ctxt = GSCurrentContext();
-    [self lockFocus];
-    DPSsetgray(ctxt, NSBlack);
-    DPSrectfill(ctxt,_bounds.origin.x, _bounds.origin.y,
-		_bounds.size.width, 1.);
-    DPSrectfill(ctxt, NSMaxX(_bounds)-1., NSMinY(_bounds),
-		1., _bounds.size.height);
-    [self unlockFocus];
+    NSRectEdge up_sides[] = {NSMinYEdge, NSMaxXEdge};
+    float grays[] = {NSBlack, NSBlack};
+    
+    NSDrawTiledRects(_bounds, aRect, up_sides, grays, 2);
   }
 }
 
@@ -439,6 +425,7 @@
 			  NSHighlightRect (oldRect);
 			}
 		      NSHighlightRect (r);
+		      [_window flushWindow];
 		      lit = YES;
 		      oldRect = r;
 		    }
@@ -452,6 +439,7 @@
 		      if (lit)
 			{
 			  NSHighlightRect (oldRect);
+			  [_window flushWindow];
 			  lit = NO;
 			}
 		      tvRect = [_tableView visibleRect];
@@ -496,6 +484,7 @@
 	  if (lit == YES)
 	    {
 	      NSHighlightRect(oldRect);
+	      [_window flushWindow];
 	    }
 
 	  [self unlockFocus];
@@ -523,13 +512,14 @@
 			objectAtIndex: columnIndex]];
 	rect.origin.y++;
 	rect.size.height--;
-	[[currentColumn headerCell]
-	  setBackgroundColor: [NSColor controlColor]];
+	[[currentColumn headerCell] setHighlighted: YES];
 
-	[[currentColumn headerCell] 
+	[self lockFocus];
+	[[currentColumn headerCell]
 	  highlight: YES
 	  withFrame: rect
 	  inView: self];
+	[self unlockFocus];
 	[_window flushWindow];
       }
 
@@ -647,10 +637,12 @@
 			      if (lit)
 				NSHighlightRect(oldRect);
 			      NSHighlightRect(highlightRect);
+			      [_window flushWindow];
 			    }
 			  else if (!lit)
 			    {
 			      NSHighlightRect(highlightRect);
+			      [_window flushWindow];
 			    }
 			  oldRect = highlightRect;
 			  lit = YES;
@@ -668,13 +660,15 @@
 			    {
 			      if (lit)
 				NSHighlightRect(oldRect);
-			      //			      NSHighlightRect(highlightRect);
+			      //  NSHighlightRect(highlightRect);
+			      [_window flushWindow];
 			    }
 			  else if (!lit)
 			    {
-			      //			      NSHighlightRect(highlightRect);
+			      //  NSHighlightRect(highlightRect);
+			      // [_window flushWindow];
 			    }
-			  //			  oldRect = highlightRect;
+			  // oldRect = highlightRect;
 			  oldRect = NSZeroRect;
 			  lit = NO; //lit = YES;
 			}
@@ -686,6 +680,7 @@
 		      if (lit)
 			{
 			  NSHighlightRect(oldRect);
+			  [_window flushWindow];
 			  lit = NO;
 			  oldRect = NSZeroRect;
 			}
@@ -727,6 +722,7 @@
 	  if (lit)
 	    {
 	      NSHighlightRect(highlightRect);
+	      [_window flushWindow];
 	      lit = NO;
 	    }
 
@@ -754,12 +750,14 @@
 		rect.origin.y++;
 		rect.size.height--;
 		[[currentColumn headerCell]
-		  setBackgroundColor: [NSColor controlShadowColor]];
+		  setHighlighted: NO];
 		
+		[self lockFocus];
 		[[currentColumn headerCell] 
 		  highlight: NO
 		  withFrame: rect
 		  inView: self];
+		[self unlockFocus];
 		[_window flushWindow];
 	      }
 	      if (i > columnIndex)
@@ -799,7 +797,8 @@
 	  
 	  if (![self mouse: point inRect: cellFrame])
 	    {
-	      NSDebugLog(@"NSCell point not in cell frame\n");
+	      NSDebugLLog(@"NSCell", 
+			  @"tableheaderview point not in cell frame\n");
 	      {
 		NSRect rect = [self headerRectOfColumn: columnIndex];
 		[_tableView _mouseDownInHeaderOfTableColumn: 
@@ -808,12 +807,14 @@
 		rect.origin.y++;
 		rect.size.height--;
 		[[currentColumn headerCell]
-		  setBackgroundColor: [NSColor controlShadowColor]];
+		  setHighlighted: NO];
 		
+		[self lockFocus];
 		[[currentColumn headerCell] 
 		  highlight: NO
 		  withFrame: rect
 		  inView: self];
+		[self unlockFocus];
 		[_window flushWindow];
 	      }
 
@@ -838,8 +839,7 @@
 		  rect.origin.y++;
 		  rect.size.height--;
 		  NSLog(@"highlight");
-		  [[currentColumn headerCell]
-		    setBackgroundColor: [NSColor controlShadowColor]];
+		  [[currentColumn headerCell] setHighlighted: NO];
 		  
 		  [[currentColumn headerCell] 
 		    highlight: NO
